import os

separator='/'
otherSeparator='\\'
def _nameCorrection(name) :
	t=name
	while(True) :
		s=t
		s=s.replace(otherSeparator,separator)
		if( s==t ) :
			return s
		t=s
		
class ShortFileName(object) :
	"""
		Encapsulation of short file name (not including directory path),
		such as readme.txt.
	"""
	def __init__(self,name,ext='') :
		"""
			__init__(self,name,ext='')
			Takes ShortFileName or str for name.
			Takes str for ext.
		"""
		if( isinstance(name,ShortFileName) ) :
			self._name=name._name
			self._ext=name._ext
			return
		if( not isinstance(name,str) ) :
			raise(Exception('Unexpected argument type in ShortFileName.__init__'))
		name=_nameCorrection(name)
		name=name.strip()
		if( name[0]==separator ) :
			name=name[1:]
		if( name[-1]==separator ) :
			name=name[:-1]
		if( name.count('.')==1 and ext=='') :
			s=name.split('.')
			name=s[0]
			ext=s[1]
		self._name=name
		if( ext.startswith('.') ) :
			ext=ext[1:]
		self._ext=ext
	def __str__(self) :
		return self._name
	def __repr__(self,name) :
		return self.__class__.__name__+"("+self.__str__()+")"
		
class FullFileName(object) :
	"""
		Encapsulation of short, partial or full file name, such as
		c:\tmp\readme.txt
		or
		\tmp\readme.txt
		or
		readme.txt
	"""
	def __init__(self,name,ext='') :
		"""
			__init__(self,name,ext='')
			Takes FullFileName or str for name.
			Takes str for ext.
		"""
		if( isinstance(name,FullFileName) ) :
			self._name=name._name
			self._ext=name._ext
			return
		if( not isinstance(name,str) ) :
			raise(Exception('Unexpected argument type in FullFileName.__init__'))
		name=_nameCorrection(name)
		self._name=[]
		for x in name.split(separator) :
			if( not x=='' ) :
				self._name.append(x)
		if( len(self._name)>0 ) :
			last=self._name[-1]
			if( ext=='' and last.count('.')==1 ) :
				s=last.split('.')
				last=s[0]
				ext=s[1]
				if( last=='' ) :
					self._name=self._name[:-1]
				else :
					self._name[len(self._name)-1]=last
		if( ext.startswith('.') ) :
			ext=ext[1:]
		self._ext=ext
	def shortName(self) :
		return ShortFileName(name=self._name[-1],ext=self._ext)
	def lastName(self) :
		return self._name[-1]
	def ext(self) :
		return self._ext
	def replaceExt(self,ext) :
		self._ext=ext
	def has(self,other) :
		"""
			has(self,other)
			"other" may be FullFileName, ShortFileName or str.
			Checks whether the "self" contains "other".
		"""
		if( isinstance(other,FullFileName) ) :
			names=other._name
			ext=other._ext
		elif( isinstance(other,ShortFileName) ) :
			names=[other._name]
			ext=other._ext
		elif( isinstance(other,str) ) :
			other=FullFileName(other)
			names=other._name
			ext=other._ext			
		else :
			raise(Exception('Wrong argument type in FullFileName.has'))
		if( ext!='' and ext!=self._ext ) :
			return False
		for i in range(0,len(self._name)) :
			for j in range(0,len(names)) :
				if( i+j>=len(self._name) ) :
					return False
				if( names[j]!=self._name[i+j] ) :
					break
				if( j==len(names)-1 ) :
					return True
		return False
	def __add__(self,other) :
		if( not self._ext=='' ) :
			if( len(self._name)==0 ) :
				self._name.append('.'+self._ext)
				self._ext=''
			else :
				x=self._name[-1]
				self._name=self._name[:-1]
				self._name.append(x+'.'+self._ext)
				self._ext=''
		if( isinstance(other,str) ) :
			if( other.startswith('.') and other.count('.')==1 ) :
				ext=other[1:]
				return FullFileName(name=separator.join(self._name),ext=ext)
			else :
				return self+ShortFileName(other)
		if( isinstance(other,ShortFileName) ) :
			name=[]
			for x in self._name :
				name.append(x)
			name.append(other._name)
			ext=other._ext
			r=FullFileName('','')
			r._name=name
			r._ext=ext
			return r
		raise(Exception("Unexpected case in FullFileName.__add__"))
	def toStr(self,separator) :
		s=separator.join(self._name)
		if( not self._ext=='' ) :
			return s+'.'+self._ext
		else :
			return s
	def toStrForward(self) :
		return self.toStr(separator=separator)
	def toStrBackward(self) :
		return self.toStr(separator=otherSeparator)		
	def __str__(self) :
		return self.toStrForward()
	def __repr__(self) :
		return self.__class__.__name__+'("'+self.__str__()+'")'
		
ADrive=FullFileName("a:")
BDrive=FullFileName("b:")
CDrive=FullFileName("c:")
DDrive=FullFileName("d:")
EDrive=FullFileName("e:")
FDrive=FullFileName("f:")
GDrive=FullFileName("g:")
HDrive=FullFileName("h:")
IDrive=FullFileName("i:")
JDrive=FullFileName("j:")
