def Rules() :
	return [
		_CppRule(),
		_ObjRule(),
		_CuRule(),
		_CuObjRule(),
		_IdlRule(),
		_TlbRule(),
		_ResRule(),
		_DefRule(),
		_RcRule()
	]

class File(object) :
	def __init__(self,dirPath,name,ext) :
		self.dirPath=dirPath
		self.name=name
		self.ext=ext
	def __str__(self) :	
		import ots.io as io
		return str(self.dirPath)+io.separator+str(self.name)+'.'+str(self.ext)

class NotApplicable(Exception) :
	def __init__(self) :
		Exception.__init__(self,'Not applicable')
		
class IRule(object) :
	def process(self,file,macros,clauses,generator) :
		self.file=file
		import ots.make.generator as g
		if( not isinstance(self.file,g.File) ) :
			raise(Exception('ots.make.generator.File object expected'))
		if( not self._isApplicable() ) :
			raise(NotApplicable())
		self.macros=macros
		self.clauses=clauses
		self.generator=generator
		self._preprocessing()
		return self._process()
	def _preprocessing(self) :
		pass
	def _process(self) :
		raise(Exception('process is not implemented'))
	def _isApplicable(self) :
		raise(Exception('process is not implemented'))
		
class IInitialRule(IRule) :
	def _preprocessing(self) :
		self._makeDirMacros()
	def _makeDirMacros(self) :
		self.macros.setIfDoesNotExist('inputDirs',lambda:[])
		self.macros.setIfDoesNotExist('outputDirs',lambda:[])
		try :
			i=self.macros.inputDirs.index(self.file.dirPath)
		except :
			self.macros.inputDirs.append(self.file.dirPath)
			i=len(self.macros.inputDirs)-1
			newDir=str(self.generator.objDir+('dir'+str(i)))
			import os
			if( not os.path.exists(newDir) ) :
				os.makedirs(newDir)
			self.macros.outputDirs.append(newDir)
			self.macros.add(self._makeObjDirMacro(i))
			self.macros.add(self._makeSrcDirMacro(i))
		self._inputDirIndex=i
	def _makeObjDirMacro(self,index) :
		return Macro(name='OBJ_DIR'+str(index),clause=self.macros.outputDirs[index])
	def _makeSrcDirMacro(self,index) :
		return Macro(name='SRC_DIR'+str(index),clause=self.macros.inputDirs[index])
		
class IFinalRule(IRule) :
	def _preprocessing(self) :
		self._makeFinalMacros()
	def _makeFinalMacros(self) :
		self.macros.addIfDoesNotExist('outputDirMacro',self._makeOutputDirMacro)
		self.macros.addIfDoesNotExist('resultNameMacro',self._makeResultNameMacro)
		self.macros.addIfDoesNotExist('linkerOptions',self._makeLinkerOptionsMacro)
		self.macros.addIfDoesNotExist('objs',self._makeObjsMacro)
		self.macros.addIfDoesNotExist('libs',self._makeLibsMacro)
		self.clauses.setIfDoesNotExist('main',self._makeMainClause)
		self.clauses.addIfDoesNotExist('linkerClause',self._makeLinkClause)
	def _makeLibsMacro(self) :
		return Macro(name='LN_LIBS',clause=_getOptions(self.generator,'libs'))
	def _makeObjsMacro(self) :
		return Macro(name='OBJS',clause=[])
	def _makeLinkerOptionsMacro(self) :
		import ots.make.options as opt
		import ots.io as io
		lo=_getOptions(self.generator,'link')
		lo.append( opt.Linker.Out(
			'"$(OUTPUT_DIR)'+io.separator+'$(RESULT_NAME)'+'.'+self.generator.output.ext()+'"'
		))
		if( self.generator.output.idString() in ['dll','pyd'] ) :
			lo.append( opt.Linker.ImpLib('$(OUTPUT_DIR)'+io.separator+'$(RESULT_NAME)'+'.lib') )
		return Macro(name='LN_OPTS',clause=lo)		
	def _makeResultNameMacro(self) :
		return Macro(name='RESULT_NAME',clause=str(self.generator.outputName))
	def _makeOutputDirMacro(self) :
		return Macro(name='OUTPUT_DIR',clause=str(self.generator.outputDir))
	def _makeMainClause(self) :
		import ots.io as io
		resultString='$(OUTPUT_DIR)'+io.separator+'$(RESULT_NAME).'+self.generator.output.idString()
		return Clause(goal='goal : ' + '"'+resultString+'"')
	def _makeLinkClause(self) :
		import ots.io as io
		import ots.make.options as opt
		resultString='$(OUTPUT_DIR)'+io.separator+'$(RESULT_NAME).'+self.generator.output.idString()
		makeLines=['link $(LN_OPTS) $(OBJS) $(LN_LIBS)']
		if( self.generator.output.idString()=='exe' ) :
			suffix='1'
		elif( self.generator.output.idString() in ['dll','pyd'] ) :
			suffix='2'
		else :
			raise(Exception('Unexpected output.idString'))
		if( self.generator.runtime==opt.Runtime.Msvc9 ) :
			makeLines.append(
				'mt -nologo -manifest "'
				+resultString
				+'.manifest" -outputresource:"'
				+resultString
				+';#'
				+suffix
				+'"'
			)
		return Clause(
			goal='"'+resultString+'" : $(OBJS)',
			makeLines=makeLines
			)
			
class IInitialRule_and_IFinalRule(IInitialRule,IFinalRule) :
	def _preprocessing(self) :
		IInitialRule._preprocessing(self)
		IFinalRule._preprocessing(self)
	
	
	
class _CppRule(IInitialRule) :
	def _isApplicable(self) :
		return self.file.ext=='cpp' or self.file.ext=='c'
	def _process(self) :
		self.macros.addIfDoesNotExist('compilerOptions',self._makeCompilerOptionsMacro)
		i=self._inputDirIndex
		r=File(dirPath='$(OBJ_DIR'+str(i)+')',name=self.file.name,ext='obj')
		self.clauses.add(self._makeClause(i,r))
		return r
	def _makeClause(self,index,objFile) :
		import ots.io as io
		return Clause(
			goal='"'+str(objFile)
				+'" : "$(SRC_DIR'+str(index)+')'
				+io.separator
				+str(self.file.name)+'.'+str(self.file.ext)+'"',
			makeLines=['cl $? -Fo$@ $(CP_OPTS)']
			)
	def _makeCompilerOptionsMacro(self) :
		return Macro(name='CP_OPTS',clause=_getOptions(self.generator,'compile'))
		
class _ObjRule(IFinalRule) :	
	def _isApplicable(self) :
		return self.file.ext=='obj'
	def _process(self) :
		self.macros.objs.add(self.file)
		return None
		

class _CuRule(IInitialRule) :
	def _isApplicable(self) :
		return self.file.ext=='cu'
	def _process(self) :
		self.macros.addIfDoesNotExist('nvccOptions',self._makeNvccOptionsMacro)
		self.macros.addIfDoesNotExist('cudaBin',self._makeCudaBinMacro)
		self.macros.addIfDoesNotExist('cudaInclude',self._makeCudaIncludeMacro)
		self.macros.addIfDoesNotExist('cppBin',self._makeCppBinMacro)
		i=self._inputDirIndex
		r=File(dirPath='$(OBJ_DIR'+str(i)+')',name=self.file.name,ext='cu.obj')
		self.clauses.add(self._makeClause(i,r))
		return r
	def _makeClause(self,index,objFile) :
		import ots.io as io
		return Clause(
			goal='"'+str(objFile)
				+'" : "$(SRC_DIR'+str(index)+')'
				+io.separator
				+str(self.file.name)+'.'+str(self.file.ext)+'"',
			makeLines=['"$(CUDA_BIN)"'+io.separator+'nvcc.exe $(NVCC_OPTS) --output-file $@ $?']
			)
	def _makeNvccOptionsMacro(self) :
		return Macro(name='NVCC_OPTS',clause=_getOptions(self.generator,'cudaCompile'))
	def _makeCudaBinMacro(self) :
		import ots.make.options as opt
		return Macro(name='CUDA_BIN',clause=opt.Dir.CudaBin)
	def _makeCppBinMacro(self) :
		import ots.make.options as opt
		return Macro(name='CPP_BIN',clause=opt.Dir.Msvc9Bin)
	def _makeCudaIncludeMacro(self) :
		import ots.make.options as opt
		return Macro(name='CUDA_INCLUDE',clause=opt.Dir.CudaInclude)
		
class _CuObjRule(IFinalRule) :	
	def _isApplicable(self) :
		return self.file.ext=='cu.obj'
	def _process(self) :
		self.macros.addIfDoesNotExist('cudaLibs',self._makeCudaLibsMacro)
		self.macros.objs.add(self.file)
		try :
			self.didModifyLinkerOptions
		except :
			self._modifyLinkerOptions()
			self._modifyLibs()
			self.didModifyLinkerOptions=True
		return None
	def _makeCudaLibsMacro(self) :
		import ots.make.options as opt
		return Macro(name='CUDA_LIBS',clause=opt.Dir.CudaLibs)		
	def _modifyLinkerOptions(self) :
		import ots.make.options as opt
		macro=self.macros.linkerOptions
		macro.add(opt.Linker.LibPath('"$(CUDA_LIBS)"'))
		macro.add(opt.Linker.TargetPlatform.Machine_X86)
		macro.add(opt.Linker.Optimization.Ref)
		macro.add(opt.Linker.Optimization.Icf)
		macro.add(opt.Linker.IgnoreLibrary('"LIBCMT.lib"'))
		macro.add(opt.Linker.LinkTimeCodeGeneration.Yes)
		#macro.add(opt.Linker.ManifestUAC.Default)
	def _modifyLibs(self) :
		import ots.make.options as opt
		macro=self.macros.libs
		macro.add("cudart.lib")
		


		
class _IdlRule(IInitialRule_and_IFinalRule) :
	def _isApplicable(self) :
		return self.file.ext=='idl'
	def _process(self) :
		result=File(dirPath='$(OUTPUT_DIR)',name=self.file.name,ext='tlb')
		i=self._inputDirIndex
		#MIDL_OPTS
		try :
			self.macros.midlOptions
		except :
			self.macros.midlOptions=self.macros.add(self._makeMidlOptionsMacro(i,result))
		#Clause
		self.clauses.add(self._makeClause(i,result))
		self.clauses.main.addToGoal('"'+str(result)+'"')
		return result
	def _makeClause(self,index,objFile) :
		import ots.io as io
		return Clause(
			goal='"'+str(objFile)
				+'" : "$(SRC_DIR'+str(index)+')'
				+io.separator
				+str(self.file.name)+'.'+str(self.file.ext)+'"',
			makeLines=['midl.exe $(MIDL_OPTS) $?']
			)
	def _makeMidlOptionsMacro(self,index,result) :
		return Macro(
			name='MIDL_OPTS',
			clause=[
				'/D "NDEBUG"',
				"/char signed",
				"/env win32",
				"/Oicf",
				'/tlb "'+str(result)+'"',
				'/h "$(SRC_DIR'+str(index)+')/'+str(self.file.name)+'_h.h"',
				'/iid "$(SRC_DIR'+str(index)+')/'+str(self.file.name)+'_i.c"',
				'/proxy "$(SRC_DIR'+str(index)+')/'+str(self.file.name)+'_p.c"',
				'/error stub_data'
			]
		)
		
class _TlbRule(IRule) :
	def _isApplicable(self) :
		return self.file.ext=='tlb'
	def _process(self) :
		import ots.make.options as opt
		if( self.generator.runtime==opt.Runtime.Msvc71 ) :
			return
		#RC_INCLUDE
		try :
			self.macros.rcIncludeMacro
		except :
			self.macros.rcIncludeMacro=self.macros.add(self._makeRcIncludeMacro())
		#Clause
		result=File(dirPath='$(OBJ_DIR0)',name=self.file.name,ext='res')
		self.clauses.add(self._makeClause(result))
		return result
	def _makeClause(self,result) :
		return Clause(
			goal='"'+str(result)+'"'
				+' : '
				+'"$(RCSCRIPT_FILE)"'
				+' '
				+'"'+str(self.file)+'"',
			makeLines=[
				'rc.exe /d "NDEBUG" /d "_UNICODE" /d "UNICODE" /i '
				+'"$(RC_INCLUDE)" /l 0x409 /fo"'
				+str(result)+'" "$(RCSCRIPT_FILE)"'
			]
		)
	def _makeRcIncludeMacro(self) :
		import ots.make.options as opt
		if( self.generator.runtime==opt.Runtime.Msvc71 ) :
			rootDir=opt.Dir.Msvc71
		if( self.generator.runtime==opt.Runtime.Msvc9 ) :
			rootDir=opt.Dir.Msvc9
		dir=rootDir+'VC'+'atlmfc'+'include'
		return Macro(name='RC_INCLUDE',clause=str(dir))
		
		
class _ResRule(IFinalRule) :	
	def _isApplicable(self) :
		return self.file.ext=='res'
	def _process(self) :
		self.macros.objs.add(self.file)
		return None

class _DefRule(IFinalRule) :		
	def _isApplicable(self) :
		return self.file.ext=='def'
	def _process(self) :
		import ots.make.options as opt
		self.macros.macroAt('LN_OPTS').clause().append(opt.Linker.Def('"'+str(self.file)+'"'))
		return None

class _RcRule(IRule) :
	def _isApplicable(self) :
		return self.file.ext=='rc'
	def _process(self) :
		self.macros.add(Macro(name='RCSCRIPT_FILE',clause=str(self.file)))
		