
class ICompileOption(IOption) :
	def idString(self) :
		return self.__class__.__name__
	def compile(self,**kwargs) :
		return [self]
	def link(self,**kwargs) :
		return []
	def libs(self,**kwargs) :
		return []

class _Compile(ILevel1) :
	#Level1
	def children(self) :
		return [
			Compile.SourceFileType,
			Compile.DbgInfoFormat,
			Compile.OptimizeCode,
			Compile.WarningLevels,
			Compile.BufferSecurityCheck,
			Compile.RunTimeTypeInfo,
			Compile.RunTimeLibrary,
			Compile.Conformance,
			Compile.ExceptionHandling,
			Compile.CompileWithoutLinking
		]
	def name(self) :
		return "Compile"

	class _SourceFileType(ILevel2) :
		"""
The /Tc option specifies that filename is a C source file, even if it does not have a .c extension. The /Tp option specifies that filename is a C++ source file, even if it doesn't have a .cpp or .cxx extension. A space between the option and filename is optional. Each option specifies one file; to specify additional files, repeat the option.
/TC and /TP are global variants of /Tc and /Tp. They specify to the compiler to treat all files named on the command line as C source files (/TC) or C++ source files (/TP), without regard to location on the command line in relation to the option. These global options can be overridden on a single file by means of /Tc or /Tp.		
	/Tcfilename
	/Tpfilename
	/TC
	/TP		
Remarks
	By default, CL assumes that files with the .c extension are C source files and files with the .cpp or the .cxx extension are C++ source files.
	To set this compiler option in the Visual Studio development environment
		Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
		Click the C/C++ folder.
		Click the Advanced property page.
		Modify the Compile As property.
Examples
	The following CL command line specifies that MAIN.c, TEST.prg, and COLLATE.prg are all C source files. CL will not recognize PRINT.prg.
		CL MAIN.C /TcTEST.PRG /TcCOLLATE.PRG PRINT.PRG
	The following CL command line specifies that TEST1.c, TEST2.cxx, TEST3.huh, and TEST4.o are compiled as C++ files, and TEST5.z is compiled as a C file.
		CL TEST1.C TEST2.CXX TEST3.HUH TEST4.O /Tc TEST5.Z /TP		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.SourceFileType.None_,
				Compile.SourceFileType.TP,
				Compile.SourceFileType.TC
			]
		def name(self) :
			return "SourceFileType"
		class _None(ILevel3) :
			"See parent's doc string"
			def parent(self) :
				return Compile.SourceFileType
			def children(self) :
				return []
			def name(self) :
				return "None"
		class _TP(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.SourceFileType
			def name(self) :
				return "-TP"
		class _TC(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.SourceFileType
			def name(self) :
				return "-TC"
	
	class _DbgInfoFormat(ILevel2) :
		"""
Select the type of debugging information created for your program and whether this information is kept in object (.obj) files or in a program database (PDB).
/Z{7|i|I}
Remarks
	The options are described in the following table.
	None
		Produces no debugging information, so compilation is faster.
	/Z7
		Produces an .obj file containing full symbolic debugging information for use with the debugger. The symbolic debugging information includes the names and types of variables, as well as functions and line numbers. No .pdb file is produced.
		For distributors of third-party libraries, there is an advantage to not having a .pdb file. However, the .obj files for the precompiled headers are necessary during the link phase, and debugging. If there is only type information (and no code) in the .pch object files, you will also have to compile with /Yl (Inject PCH Reference for Debug Library).
	/Zi
		Produces a program database (PDB) that contains type information and symbolic debugging information for use with the debugger. The symbolic debugging information includes the names and types of variables, as well as functions and line numbers.
		/Zi does not affect optimizations. However, /Zi does imply /debug; 
		Type information is placed in the .pdb file, and not in the .obj file.
		You can use /Gm (Enable Minimal Rebuild) with /Zi, whereas /Gm is not available when compiling with /Z7.
		When compiling with /Zi and /clr, the DebuggableAttribute attribute will not be placed in the assembly metadata; you must specify it in source code, if you want it. This attribute can affect the runtime performance of the application. 
	/ZI
		Produces a program database, as described above, in a format that supports the Edit and Continue feature. If you want to use Edit and Continue debugging, you must use this option. Because most optimizations are incompatible with Edit and Continue, using /ZI disables any #pragma optimize statements in your code.
		/ZI causes /Gy (Enable Function-Level Linking) to be used in your compilation.
		/ZI is not compatible with /clr (Common Language Runtime Compilation).
	Note
		/ZI is only available in the compiler targeting x86; this compiler option is not available in the compilers targeting x64 or Itanium Processor Family (IPF).
		The compiler names the program database project.pdb. If you compile a file without a project, the compiler creates a database named VCx0.pdb., where x is the major version of Visual C++ in use. The compiler embeds the name of the PDB in each .obj file created using this option, pointing the debugger to the location of symbolic and line-number information. When you use this option, your .obj files will be smaller, because debugging information is stored in the .pdb file rather than in .obj files.
		If you create a library from objects that were compiled using this option, the associated .pdb file must be available when the library is linked to a program. Thus, if you distribute the library, you must distribute the PDB.
		To create a library that contains debugging information without using .pdb files, you must select the compiler's C 7.0-Compatible (/Z7) option. If you use the precompiled headers options, debugging information for both the precompiled header and the rest of the source code is placed in the PDB. The /Yd option is ignored when the Program Database option is specified.
	To set this compiler option in the Visual Studio development environment
		Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
		Click the C/C++ folder.
		Click the General property page.
		Modify the Debug Information Format property.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.DbgInfoFormat.None_,
				Compile.DbgInfoFormat.Z7,
				Compile.DbgInfoFormat.Zi,
				Compile.DbgInfoFormat.ZI
			]
		def name(self) :
			return "DbgInfoFormat"
		class _None(ILevel3) :
			"See parent's doc string"
			def parent(self) :
				return Compile.DbgInfoFormat
			def name(self) :
				return "None"
		class _Z7(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.DbgInfoFormat
			def name(self) :
				return "/Z7"
		class _Zi(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.DbgInfoFormat
			def name(self) :
				return "/Zi"
		class _ZI(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.DbgInfoFormat
			def name(self) :
				return "/ZI"
			
	class _OptimizeCode(ILevel2) :
		"""
The /O options control various optimizations that help you create code for maximum speed or minimum size.
	/O1, optimizes code for minimum size.
	/O2  optimizes code for maximum speed.
	/Ob (Inline Function Expansion) controls inline function expansion.
	/Od (Disable (Debug)) disables optimization, speeding compilation and simplifying debugging.
	/Og (Global Optimizations) enables global optimizations.
	/Oi (Generate Intrinsic Functions) generates intrinsic functions for appropriate function calls.
	/Os, /Ot (Favor Small Code, Favor Fast Code) tells the compiler to favor optimizations for size over optimizations for speed.
	/Os, /Ot (Favor Small Code, Favor Fast Code) (a default setting) tells the compiler to favor optimizations for speed over optimizations for size.
	/Ox (Full Optimization) selects full optimization.
	/Oy (Frame-Pointer Omission) suppresses the creation of frame pointers on the call stack for quicker function calls.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.OptimizeCode.Od,
				Compile.OptimizeCode.Ob
			]
		def name(self) :
			return "OptimizeCode"
		class _Od(ILevel3,ICompileOption) :
			"""
Turns off all optimizations in the program and speeds compilation.
/Od
Remarks
	This option is the default. Because /Od suppresses code movement, it simplifies the debugging process. For information on the compile-for-debugging options, see /Z7, /Zi, /ZI (Debug Information Format), /Z7, /Zi, /ZI (Debug Information Format), and /Z7, /Zi, /ZI (Debug Information Format).
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Optimization property page.
	Modify the Optimization property.			
			"""
			def parent(self) :
				return Compile.OptimizeCode
			def name(self) :
				return "/Od"
		class _Ob(ILevel3,ICompileOption) :
			"""
Controls inline expansion of functions.
	/Ob{0|1|2}
Arguments
	0
	Disables inline expansion, which is on by default.
	1
	Expands only functions marked as inline, __inline, __forceinline or __inline or, in a C++ member function, defined within a class declaration.
	2
	Expands functions marked as inline or __inline and any other function that the compiler chooses (expansion occurs at the compiler's discretion, often referred to as auto-inlining).
/Ob2 is in effect when /O1, /O2 (Minimize Size, Maximize Speed) or /Ox (Full Optimization) is used.
This option requires that you enable optimizations using /O1, /O2, /Ox, or /Og.
Remarks
	The compiler treats the inline expansion options and keywords as suggestions. There is no guarantee that functions will be expanded inline. You cannot force the compiler to inline a particular function.
	You can also use auto_inline to exclude functions from being considered as candidates for inline expansion. Also see intrinsic.
Note
	Information that is gathered from profiling test runs will override optimizations that would otherwise be in effect if you specify /Ob, /Os, or /Ot. 
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Optimization property page.
	Modify the Inline Function Expansion property.			
			"""
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return Compile.OptimizeCode
			def name(self) :
				return "/Ob"+str(self.value)
		class _O(ILevel3,ICompileOption) :
			"""
	/O1
	/O2
Remarks
	The following table describes /O1 and /O2.
	Option	 			Equivalent to	 						Comment
/O1 (Minimize Size)		/Og /Os /Oy /Ob2 /Gs /GF /Gy			Creates the smallest code in the majority of cases.
/O2 (Maximize Speed)	/Og /Oi /Ot /Oy /Ob2 /Gs /GF /Gy		Creates the fastest code in the majority of cases. (default setting for release builds)

	/O1 and /O2 also enable the Named Return Value optimization, which eliminates the copy constructor and destructor of a stack based return value. Consider the following sample. The Test function will not create the copy constructor or destructor. Add output statements to the constructor, destructor and copy constructor to see the effect of the Named Return Value optimization when you run the program. For more information on Named Return Value Optimization in Visual C++, see http://go.microsoft.com/fwlink?linkid = 55900.

		// O1_O2_NRVO.cpp
		// compile with: /O1
		struct A {
		   A() {}
		   ~A() {}
		   A(const A& aa) {}
		};

		A Test() {
		   A a;
		   return a;
		}
		int main() {
		   A aa;
		   aa = Test();
		}

x86 Specific
	These options imply the use of the Frame-Pointer Omission (/Oy) option.

To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Optimization property page.
	Modify the Optimization property.			
			"""
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return Compile.OptimizeCode			
			def name(self) :
				return "/O"+str(self.value)
	
	class _WarningLevels(ILevel2) :
		"""
Specify how the compiler generates warnings for a given compilation.
/w
/Wn
/WX
/Wall
/wln
/wdn
/wen
/won
Remarks
/w
	Disables all compiler warnings.
/W n
	Specifies the highest level of warning generated by the compiler. Valid warning levels for n range from 0 to 4:
	Level 0 disables all warnings.
	Level 1 displays severe warnings. Level 1 is the default warning level at the command line.
	Level 2 displays all level 1 warnings and warnings less severe than level 1.
	Level 3 displays all level 2 warnings and all other warnings recommended for production purposes.
	Level 4 displays all level 3 warnings plus informational warnings, which in most cases can be safely ignored. This option should be used only to provide "lint" level warnings and is not recommended as your usual warning level setting.
	For a new project, it may be best to use /W4 in all compilations. This will ensure the fewest possible hard-to-find code defects.
/Wall
	Enables all warnings, including those disabled by default. See Compiler Warnings That Are Off By Default.
/WX
	Treats all compiler warnings as errors. For a new project, it may be best to use /WX in all compilations; resolving all warnings will ensure the fewest possible hard-to-find code defects.
	The linker also has a /WX option; 
/w ln
	Specifies the level for a particular warning. The first parameter sets the warning level (same as /Wn) and the second parameter is the actual warning number.
	For example, /w14326 causes C4326 to be generated as a level 1 warning.
/wd n
	Disables the specified compiler warning where n is the compiler warning number.
	For example, /wd4326 disables compiler warning C4326.
/we n
	Treats the specific compiler warning as an error where n is a compiler warning.
	For example, /we4326 flags warning number C4326 as an error.
/wo n
	Reports the error only once where n is a compiler warning.
	For example, /wo4326 will cause warning C4326 to be reported only once.
	If you create a precompiled header (/Yc (Create Precompiled Header File)) with one of the /w options, any use of the precompiled header (/Yu (Use Precompiled Header File)) will cause those same /w options to be in effect again. You can override the /w setting in the precompiled header with another /w option at the command line.

Pragma directives in source code are unaffected by the /w option.
You can also use warning to control the level of warning reported at compile time.
The C/C++ Build Errors describes the warnings, indicates each warning's level, and indicates potential problems (rather than actual coding errors) with statements that may not compile as you intend.

To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the General property page and modify the Warning Level or Treat Warnings as Errors properties.
	Click the Advanced property page and modify the Disable Specific Warnings property.
	For the remaining options, click the Command Line property page and type the compiler option in the Additional Options box.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.WarningLevels.W0,
				Compile.WarningLevels.W1,
				Compile.WarningLevels.W2,
				Compile.WarningLevels.W3,
				Compile.WarningLevels.W4,
				Compile.WarningLevels.wd4675
			]
		def name(self) :
			return "WarningLevels"
		class _W(ILevel3,ICompileOption) :
			"See parent's doc string"
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return Compile.WarningLevels
			def name(self) :
				return "/W"+str(self.value)
		class _wd(ILevel3,ICompileOption) :
			"See parent's doc string"
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return Compile.WarningLevels
			def name(self) :
				return "/wd"+str(self.value)
	
	class _RunTimeTypeInfo(ILevel2) :
		"""
Adds code to check object types at run time.
/GR[-]
Remarks
	When /GR is on, the compiler defines the _CPPRTTI preprocessor macro. In Visual C++ 2005, /GR is on by default. /GR- disables run-time type information.
	Use /GR if your code uses dynamic_cast Operator or typeid. /GR does, however, cause the .rdata sections of your image to increase in size. If your code does not use dynamic_cast or typeid, using /GR- may produce a smaller image.
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Language property page.
	Modify the Enable Run-Time Type Info property.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.RunTimeTypeInfo.GR,
				Compile.RunTimeTypeInfo.GRminus
			]
		def name(self) :
			return "RunTimeTypeInfo"
		class _GR(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/GR"
		class _GRminus(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimTypeInfo
			def name(self) :
				return "/GR-"
			
	class _BufferSecurityCheck(ILevel2) :
		"""
Detects some buffer overruns that overwrite the return address, a common technique for exploiting code that does not enforce buffer size restrictions. This is achieved by injecting security checks into the compiled code.
	/GS[-]
Remarks
	/GS is on by default. Use /GS- if you expect your application to have no security exposure.
	For additional information on /GS see Compiler Security Checks In Depth.
	The compiler injects checks in functions with local string buffers or, on x86, functions with exception handling. A string buffer is defined as an array whose element size is one or two bytes, and where the size of the whole array is at least five bytes, or, any buffer allocated with _alloca.
	On all platforms, the compiler injects a cookie to protect the function's return address if the function has local string buffers. This cookie is checked on function exit, and during frame unwinding on 64-bit operating systems or on x86 for functions with some form of exception handling. On x86, the compiler also injects a cookie to protect the address of the function's exception handler. This cookie is checked during frame unwinding.
	/GS primarily attempts to detect direct buffer overruns into the return address. Buffer overruns are more easily exploited on machines with calling conventions that store the return address of function calls on the stack. For example, x86 uses calling conventions that store the return address of function calls on the stack.
	On functions that the compiler thinks might be subject to buffer overrun problems, the compiler will allocate space on the stack before the return address. On function entry, the allocated space is loaded with a security cookie that is computed once at module load. Then, on function exit, a helper function is called to make sure the cookie's value is still the same.
	If the value is not the same, an overwrite of the stack has potentially occurred and the process simply terminates. Prior to Visual C++ 2005 a dialog box displayed reporting the stack overwrite.
	/GS also protects against vulnerable parameters passed into a function. A vulnerable parameter is a pointer, C++ reference, or a C-structure (C++ POD type) that contains a pointer, string buffer, or C++ reference.
	A vulnerable parameter is allocated before the cookie and local variables. A buffer overrun can overwrite these parameters. Code in the function that uses these parameters can lead to an attack before the function returns, thus avoiding the security check. To minimize this danger, the compiler will make a copy of the vulnerable parameters during the function prolog and put them below the storage area for any buffers.

	The compiler does not provide security protection for vulnerable parameters in the following situations:
		Functions that do not contain a buffer.
		If optimizations (/O Options (Optimize Code)) are not enabled.
		Functions with a variable argument list (...).
		Functions marked with naked (C++).
		Functions containing inline assembly code in the first statement.
		If a parameter is used only in ways that are less likely to be exploitable in the event of a buffer overrun.

	/GS requires initialization of the security cookie. This cookie must be initialized before any function using the cookie runs. The security cookie must be initialized upon entry to an EXE or DLL. This is done automatically when using the default CRT entry points (mainCRTStartup, wmainCRTStartup, WinMainCRTStartup, wWinMainCRTStartup, or _DllMainCRTStartup), but must be done manually, via a call to __security_init_cookie, if you use an alternate entry point.
	/GS is supported for managed functions when compiling with /clr (Common Language Runtime Compilation).
	/GS does not protect against all buffer overrun security attacks. For example, if you have a buffer and a vtable in an object, the buffer overrun could corrupt the vtable and allow an attack to occur.
	Even if you use /GS, you should strive to write secure code. That is, make sure that your code has no buffer overruns. /GS might protect your application from buffer overruns that do remain in your code.

To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Code Generation property page.
	Modify the Buffer Security Check property.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.BufferSecurityCheck.GS,
				Compile.BufferSecurityCheck.GSminus
			]
		def name(self) :
			return "BufferSecurityCheck"
		class _GS(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.BufferSecurityCheck
			def name(self) :
				return "/GS"
		class _GSminus(ILevel3,ICompileOption) :
			def parent(self) :
				return Compile.BufferSecurityCheck
			def name(self) :
				return "/GS-"
	
	class _RunTimeLibrary(ILevel2) :
		"""
Lets you indicate if a multithreaded module is a DLL and select retail or debug versions of the run-time library.
	/MD[d]
	/MT[d]
	/LD[d]
Remarks
	/MD
		Causes your application to use the multithread- and DLL-specific version of the run-time library. Defines _MT and _DLL and causes the compiler to place the library name MSVCRT.lib into the .obj file.
		Applications compiled with this option are statically linked to MSVCRT.lib. This library provides a layer of code that allows the linker to resolve external references. The actual working code is contained in MSVCR80.DLL, which must be available at run time to applications linked with MSVCRT.lib.
		When /MD is used with _STATIC_CPPLIB defined (/D_STATIC_CPPLIB) it will cause the application to link with the static multithread Standard C++ Library (libcpmt.lib) instead of the dynamic version (msvcprt.lib) while still dynamically linking to the main CRT via msvcrt.lib.
	/MDd
		Defines _DEBUG, _MT, and _DLL and causes your application to use the debug multithread- and DLL-specific version of the run-time library. It also causes the compiler to place the library name MSVCRTD.lib into the .obj file.
	/MT
		Causes your application to use the multithread, static version of the run-time library. Defines _MT and causes the compiler to place the library name LIBCMT.lib into the .obj file so that the linker will use LIBCMT.lib to resolve external symbols.
	/MTd
		Defines _DEBUG and _MT. This option also causes the compiler to place the library name LIBCMTD.lib into the .obj file so that the linker will use LIBCMTD.lib to resolve external symbols.
	/LD
		Creates a DLL.
		Passes the /DLL option to the linker. The linker looks for, but does not require, a DllMain function. If you do not write a DllMain function, the linker inserts a DllMain function that returns TRUE.
		Links the DLL startup code.
		Creates an import library (.lib), if an export (.exp) file is not specified on the command line; you link the import library to applications that call your DLL.
		Interprets /Fe (Name EXE File) as naming a DLL rather than an .exe file; the default program name becomes basename.dll instead of basename.exe.
		Implies /MT unless you explicitly specify /MD.
	/LDd
		Creates a debug DLL. Defines _MT and _DEBUG.

	All modules passed to a given invocation of the linker must have been compiled with the same run-time library compiler option (/MD, /MT, /LD).

To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Code Generation property page.
	Modify the Runtime Library property.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.RunTimeLibrary.MD,
				Compile.RunTimeLibrary.MDd,
				Compile.RunTimeLibrary.MT,
				Compile.RunTimeLibrary.MTd,
				Compile.RunTimeLibrary.LD,
				Compile.RunTimeLibrary.LDd
			]
		def name(self) :
			return "RunTimeLibrary"
		class _MD(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/MD"
		class _MDd(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/MDd"
		class _MT(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/MT"
		class _MTd(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/MTd"
		class _LD(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/LD"
		class _LDd(ILevel3,ICompileOption) :
			"See parent's doc string"
			def parent(self) :
				return Compile.RunTimeTypeInfo
			def name(self) :
				return "/LDd"

	class _Conformance(ILevel2) :
		"""
WARNING: this options are not compatible with default stlp build if building with msvc71.
The /Zc compiler option lets you specify standard behavior.
The following is the list of options.
	/Zc:forScope
	/Zc:wchar_t		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.Conformance.Zc_forScope,
				Compile.Conformance.Zc_wchar_t
			]
		def name(self) :
			return "Conformance"
		class _Zc_forScope(ILevel3,ICompileOption) :
			"""
WARNING: this option is not compatible with default stlp build  if building with msvc71.
Used to implement standard C++ behavior for The for Statement loops with Microsoft extensions (/Za, /Ze (Disable Language Extensions)). /Zc:forScope is on by default.
	/Zc:forScope[-]
Remarks
	Standard behavior is to let a for loop's initializer go out of scope after the for loop. Under /Za, /Ze (Disable Language Extensions), the for loop's initializer remains in scope until the local scope ends.
	The following code will compile under /Ze but not under /Za:
		// zc_forScope.cpp
		// compile with: /Zc:forScope- /Za
		// C2065 expected
		int main() {
		   // Uncomment the following line to resolve.
		   // int i;
		   for (int i =0; i < 1; i++)
			  ;
		   i = 20;   // i has already gone out of scope under /Za
		}
	If you use /Zc:forScope-, you will get a warning (off by default) if a variable is in scope because of a declaration that was made in a previous scope. To demonstrate this, remove the // characters in the above code to declare int i.
	You can modify the run-time behavior of /Zc:forScope with the conform pragma.
	If you use /Zc:forScope- in a project with an existing .pch file, /Zc:forScope- is ignored (with a warning) and compilation continues with the existing .pch files. If you want a new .pch file generated, use /Yc (Create Precompiled Header File).
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Modifying Project Settings.
	Click the C/C++ folder.
	Click the Language property page.
	Modify the Force Conformance in For Loop Scope property.			
			"""
			def parent(self) :
				return Compile.Conformance
			def name(self) :
				return "/Zc:forScope"
		class _Zc_wchar_t(ILevel3,ICompileOption) :
			"""
WARNING: this option is not compatible with default stlp build  if building with msvc71.
When specified, the type wchar_t becomes a native type that maps to __wchar_t in the same way that short maps to __int16. /Zc:wchar_t is on by default.
	/Zc:wchar_t[-]
Remarks
	If /Zc:wchar_t- is specified, the compiler requires you to either define wchar_t or to include one of the many header files that defines it (for example, wchar.h). Typically, wchar_t is defined as an unsigned short.
	With /Zc:wchar_t, the compiler recognizes wchar_t as a native type. See Data Type Ranges for more information about wchar_t.
	__wchar_t is always available.
	By providing overloads for both the unsigned short and __wchar_t variations of wchar_t, you can create libraries that can easily be linked with code compiled with or without /Zc:wchar_t and avoid the need to provide two different builds of the library (one with and one without /Zc:wchar_t enabled).
	When /Zc:wchar_t is specified, _WCHAR_T_DEFINED and _NATIVE_WCHAR_T_DEFINED symbols are defined; see Predefined Macros for more information.
	Explicit references to comsupp.lib, either from the comment pragma or via the command line, should be changed to now use either comsuppw.lib or comsuppwd.lib, as /Zc:wchar_t is now on by default. comsupp.lib should still be used when compiling with /Zc:wchar_t-.
	For more information about conformance issues with Visual C++, see Compatibility and Compliance Issues in Visual C++.
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Modifying Project Settings.
	Click the C/C++ folder.
	Click the Language property page.
	Modify the Treat wchar_t as Built-in Type property.			
			"""
			def parent(self) :
				return Compile.Conformance
			def name(self) :
				return "/Zc:wchar_t"
	
	class _ExceptionHandling(ILevel2) :
		"""
Specifies the model of exception handling to be used by the compiler and destroys C++ objects that will go out of scope as a result of the exception. If /EH is not specified, the compiler will catch structured and C++ exceptions, but will not destroy C++ objects that will go out of scope as a result of the exception.
	/EH{s|a}[c][-]
Arguments
	a
	The exception-handling model that catches asynchronous (structured) and synchronous (C++) exceptions.
	s
	The exception-handling model that catches C++ exceptions only and tells the compiler to assume that extern C functions do throw an exception.
	c
	If used with s (/EHsc), catches C++ exceptions only and tells the compiler to assume that extern C functions never throw a C++ exception. /EHca is equivalent to /EHa.
Remarks
	Use /EHs to specify the synchronous exception handling model (C++ exception handling without structured exception handling exceptions). If you use /EHs, then your catch clause will not catch asynchronous exceptions. Also, in Visual C++ 2005, all objects in scope when the asynchronous exception is generated will not be destroyed even if the asynchronous exception is handled. Under /EHs, catch(...) will only catch C++ exceptions. Access violations and System.Exception exceptions will not be caught.
	Use /EHa to specify the asynchronous exception handling model (C++ exception handling with structured exception handling exceptions). /EHa may result in a less performant image because the compiler will not optimize a catch block as aggressively, even if the compiler does not see a throw.
	Use /EHa if you want to catch an exception raised with something other than a throw. The following sample will generate an exception:
		// compiler_options_EHA.cpp
		// compile with: /EHa
		#include <iostream>
		#include <excpt.h>
		using namespace std;

		void fail() {   // generates SE and attempts to catch it using catch(...)
		   try {
			  int i = 0, j = 1;
			  j /= i;   // This will throw a SE (divide by zero).
		   }
		   catch(...) {   // catch block will only be executed under /EHa
			  cout<<"Caught an exception in catch(...)."<<endl;
		   }
		}

		int main() {
		   __try {
			  fail(); 
		   }

		   // __except will only catch an exception here
		   __except(EXCEPTION_EXECUTE_HANDLER) {   
		   // if the exception was not caught by the catch(...) inside fail()
			  cout << "An exception was caught in __except." << endl;
		   }
		}
	The /EHc option requires that /EHs or /EHa is specified. Using /clr (Common Language Runtime Compilation) implies /EHa (/clr /EHa is redundant). The compiler will generate an error if /EHs[c] is used after /clr. Optimizations will not affect this behavior. When an exception is caught, the compiler invokes the class destructor or destructors for the object or objects that are in the same scope as the exception. When an exception is not caught, those destructors are not run.
	See _set_se_translator for exception handling restrictions under /clr.
	The option can be cleared by the symbol -. For example, /EHsc- is interpreted as /EHs /EHc- and is equivalent to /EHs.
	See Exception Handling: Default Synchronous Exception Model for more information.
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Code Generation property page.
	Modify the Enable C++ Exceptions property.
Alternately, you can use the following procedure:
To set this compiler option in the Visual Studio development environment
	Click the C/C++ folder.
	Click the Code Generation property page.
	Set Enable C++ Exceptions to No.
	Click the Command Line property page.
	Type the compiler option in the Additional Options box.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [
				Compile.ExceptionHandling.EHa,
				Compile.ExceptionHandling.EHsc,
				Compile.ExceptionHandling.EHc
			]
		def name(self) :
			return "ExceptionHandling"
		class _EH(ILevel3,ICompileOption) :
			"See parent's doc string"
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return Compile.ExceptionHandling
			def name(self) :
				return "/EH"+str(self.value)
	
	class _CompileWithoutLinking(ILevel2) :
		"""
Prevents the automatic call to LINK.
	/c
Remarks
	Compiling with /c creates .obj files only. You must call LINK explicitly with the proper files and options to perform the linking phase of the build.
	Any internal project created in the development environment uses the /c option by default.
Example
	The following command line creates the object files FIRST.obj and SECOND.obj. THIRD.obj is ignored.
		CL /c FIRST.C SECOND.C THIRD.OBJ
	To create an executable file, you must invoke LINK:
		LINK firsti.obj second.obj third.obj /OUT:filename.exe		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [Compile.CompileWithoutLinking.c]
		def name(self) :
			return "CompileWithoutLinking"
		class _c(ILevel3,ICompileOption) :
			"See parent's doc string."
			def parent(self) :
				return Compile.CompileWithoutLinking
			def name(self) :
				return '-c'
	
	class _Detect64BitPortability(ILevel2) :
		"""
Detects 64-bit portability problems on types that are also marked with the __w64 keyword.
	/Wp64
Remarks
	By default, the /Wp64 compiler option is off in the Visual C++ 32-bit compiler and on in the Visual C++ 64-bit compiler.

Important
	The /Wp64 compiler option and __w64 keyword are deprecated in Visual Studio 2010. If you convert a project that uses this switch, the switch will not be migrated during conversion. To use this option in Visual Studio 2010, you must type the compiler switch under Additional Options in the Command Line section of the project properties. If you use the /Wp64 compiler option on the command line, the compiler issues Command-Line Warning D9035. Instead of using this option and keyword to detect 64-bit portability issues, use a Visual C++ compiler that targets a 64-bit platform. For more information, see Configuring Programs for 64-Bit (Visual C++).
	
	Variables of the following types are tested on a 32-bit operating system as if they were being used on a 64-bit operating system:
		int
		long
		pointer

	If you regularly compile your application by using a 64-bit compiler, you can just disable /Wp64 in your 32-bit compilations because the 64-bit compiler will detect all issues. For more information about how to target a Windows 64-bit operating system, see Configuring Programs for 64-Bit (Visual C++).

To set this compiler option in the Visual Studio development environment
	Open the project Property Pages dialog box.
	For more information, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the Command Line property page.
	Modify the Additional Options box to include /Wp64.		
		"""
		def parent(self) :
			return Compile
		def children(self) :
			return [Compile.Detect64BitPortability.Wp64]
		def name(self) :
			return "Detect64BitPortability"
		class _Wp64(ILevel3,ICompileOption) :
			"See parent's doc string."
			def parent(self) :
				return Compile.Detect64BitPortability
			def name(self) :
				return '-Wp64'

	class Include(ILevel2,ICompileOption) :
		"""
Adds a directory to the list of directories searched for include files.
	/I[ ]directory
Arguments
	directory
		The directory to be added to the list of directories searched for include files.
Remarks
	To add more than one directory, use this option more than once. Directories are searched only until the specified include file is found.
	You can use this option with the Ignore Standard Include Paths (/X (Ignore Standard Include Paths)) option.
	The compiler searches for directories in the following order:
	Directories containing the source file.
	Directories specified with the /I option, in the order that CL encounters them.
	Directories specified in the INCLUDE environment variable.
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the General property page.
	Modify the Additional Include Directories property.				
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Compile
		def children(self) :
			return []
		def name(self) :
			return "-I"+str(self.value)
		
		
	class Preprocessor(ILevel2,ICompileOption) :
		"""
Defines a preprocessing symbol for a source file.
	/Dname[= | # [{string | number}] ]
Remarks
	You can use this symbol with #if or #ifdef to compile source conditionally. The symbol definition remains in effect until a redefinition is encountered in source or the symbol is undefined in source by using the #undef directive.
	/D has much the same effect as using the #define directive at the beginning of the source file. However, /D strips quotes on the command line and #define retains them.
	By default, the value associated with a symbol is 1. That is, /DTEST is equivalent to /DTEST=1. In the following example, the definition of TEST is shown to print 1.
	Compiling by using /Dname= causes the symbol to not have an associated value. While the symbol can still be used to conditionally compile code, the symbol otherwise evaluates to nothing. For example, in the sample program, compiling by using /DTEST= causes a compiler error. This behavior resembles using #define with or without a value.
	The following command defines the symbol DEBUG in TEST.c.
		CL /DDEBUG  TEST.C
	The following command removes all occurrences of the keyword __far in TEST.c.
		CL /D__far=  TEST.C
	You cannot set the CL environment variable to a string that contains an equal sign. To use /D with the CL environment variable, you must specify a number sign instead of an equal sign.
		SET CL=/DTEST#0
	When you define a preprocessing symbol at the command prompt, consider shell parsing rules as well as compiler parsing rules. For example, to define one percent-sign preprocessing symbol (%) in your program, specify two percent-sign characters (%%) on the command line. If you specify only one percent-sign, a parsing error is emitted.
		CL /DTEST=%% TEST.C
To set this compiler option in the Visual Studio development environment
	Open the project Property Pages dialog box. For more information, see How to: Open Project Property Pages.
	In the left pane, click C/C++ and then click Preprocessor.
	In the right pane, modify the Preprocessor Definitions property.		
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Compile
		def children(self) :
			return []
		def name(self) :
			return "-D"+str(self.value)
			
	
Compile=_Compile()
Compile.SourceFileType=Compile._SourceFileType()
Compile.SourceFileType.None_=Compile.SourceFileType._None()
Compile.SourceFileType.TP=Compile.SourceFileType._TP()
Compile.SourceFileType.TC=Compile.SourceFileType._TC()
Compile.DbgInfoFormat=Compile._DbgInfoFormat()
Compile.DbgInfoFormat.None_=Compile.DbgInfoFormat._None()
Compile.DbgInfoFormat.Z7=Compile.DbgInfoFormat._Z7()
Compile.DbgInfoFormat.Zi=Compile.DbgInfoFormat._Zi()
Compile.DbgInfoFormat.ZI=Compile.DbgInfoFormat._ZI()
Compile.OptimizeCode=Compile._OptimizeCode()
Compile.OptimizeCode.Od=Compile.OptimizeCode._Od()
Compile.OptimizeCode.Ob0=Compile.OptimizeCode._Ob(0)
Compile.OptimizeCode.Ob1=Compile.OptimizeCode._Ob(1)
Compile.OptimizeCode.Ob2=Compile.OptimizeCode._Ob(2)
Compile.OptimizeCode.O1=Compile.OptimizeCode._O(1)
Compile.OptimizeCode.O2=Compile.OptimizeCode._O(2)
Compile.WarningLevels=Compile._WarningLevels()
Compile.WarningLevels.W0=Compile.WarningLevels._W(0)
Compile.WarningLevels.W1=Compile.WarningLevels._W(1)
Compile.WarningLevels.W2=Compile.WarningLevels._W(2)
Compile.WarningLevels.W3=Compile.WarningLevels._W(3)
Compile.WarningLevels.W4=Compile.WarningLevels._W(4)
Compile.WarningLevels.wd4675=Compile.WarningLevels._wd(4675)
Compile.RunTimeTypeInfo=Compile._RunTimeTypeInfo()
Compile.RunTimeTypeInfo.GR=Compile.RunTimeTypeInfo._GR()
Compile.RunTimeTypeInfo.GRminus=Compile.RunTimeTypeInfo._GRminus()
Compile.BufferSecurityCheck=Compile._BufferSecurityCheck()
Compile.BufferSecurityCheck.GS=Compile.BufferSecurityCheck._GS()
Compile.BufferSecurityCheck.GSminus=Compile.BufferSecurityCheck._GSminus()
Compile.RunTimeLibrary=Compile._RunTimeLibrary()		
Compile.RunTimeLibrary.MD=Compile.RunTimeLibrary._MD()
Compile.RunTimeLibrary.MDd=Compile.RunTimeLibrary._MDd()
Compile.RunTimeLibrary.MT=Compile.RunTimeLibrary._MT()
Compile.RunTimeLibrary.MTd=Compile.RunTimeLibrary._MTd()
Compile.RunTimeLibrary.LD=Compile.RunTimeLibrary._LD()
Compile.RunTimeLibrary.LDd=Compile.RunTimeLibrary._LDd()
Compile.Conformance=Compile._Conformance()
Compile.Conformance.Zc_forScope=Compile.Conformance._Zc_forScope()
Compile.Conformance.Zc_wchar_t=Compile.Conformance._Zc_wchar_t()
Compile.ExceptionHandling=Compile._ExceptionHandling()
Compile.ExceptionHandling.EHa=Compile.ExceptionHandling._EH('a')
Compile.ExceptionHandling.EHs=Compile.ExceptionHandling._EH('s')
Compile.ExceptionHandling.EHsc=Compile.ExceptionHandling._EH('sc')
Compile.CompileWithoutLinking=Compile._CompileWithoutLinking()
Compile.CompileWithoutLinking.c=Compile.CompileWithoutLinking._c()
Compile.Detect64BitPortability=Compile._Detect64BitPortability()
Compile.Detect64BitPortability.Wp64=Compile.Detect64BitPortability._Wp64()


