class ICudaCompileOption(ICudaOption) :
	def idString(self) :
		return self.__class__.__name__
	def cudaCompile(self,**kwargs) :
		return [self]
	def cudaLink(self,**kwargs) :
		return []
	def cudaLibs(self,**kwargs) :
		return []
	
		
class _CudaCompile(ILevel1) :
	#Level1
	def children(self) :
		return [
			CudaCompile.CompilationPhase,
			CudaCompile.ApplicationCompatibility
		]
	def name(self) :
		return "CudaCompile"

	class _CompilationPhase(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [
				CudaCompile.CompilationPhase.Compile
			]
		def name(self) :
			return "CompilationPhase"
		class _Compile(ILevel3,ICudaCompileOption) :
			"Compile an input file into an object file."
			def parent(self) :
				return CudaCompile.CompilationPhase
			def name(self) :
				return "--compile"
				
	class _ApplicationCompatibility(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [
				CudaCompile.ApplicationCompatibility.ComputeCap10,
				CudaCompile.ApplicationCompatibility.ComputeCap20
			]
		def name(self) :
			return "ApplicationCompatibility"
		class _ComputeCap(ILevel3,ICudaCompileOption) :
			"Embeds binary code for device with the given compatibility"
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return CudaCompile.ApplicationCompatibility
			def name(self) :
				return '-gencode=arch=compute_'+ \
					str(self.value)+ \
					',code=\\"sm_'+ \
					str(self.value)+ \
					',compute_'+ \
					str(self.value)+ \
					'\\"'
					
	class _Architecture(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [
				CudaCompile.Architecture.Machine32,
				CudaCompile.Architecture.Machine64
			]
		def name(self) :
			return "Architecture"
		class _Machine(ILevel3,ICudaCompileOption) :
			"Specify 32 vs. 64 bit architecture."
			def __init__(self,value) :
				self.value=value
			def parent(self) :
				return CudaCompile.Architecture
			def name(self) :
				return '--machine '+str(self.value)

	class _FileAndPath(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [ 
				CudaCompile.FileAndPath.CCompilerBinDefault,
				CudaCompile.FileAndPath.IncludeDefault
			]
		def name(self) :
			return "FileAndPath"
		class CCompilerBin(ILevel3,ICudaCompileOption) :
			"""
Specify the directory in which the host compiler executable 
(Microsoft Visual Studio cl, or a gcc derivative) resides.
			"""
			def parent(self) :
				return CudaCompile.FileAndPath
			def name(self) :
				return "--compiler-bindir "+str(self.value)
			def __init__(self,value) :
				self.value=value
		class Include(ILevel3,ICudaCompileOption) :
			"Specify include search paths"
			def parent(self) :
				return CudaCompile.FileAndPath
			def name(self) :
				return "--include-path "+str(self.value)
			def __init__(self,value) :
				self.value=value
	
	class _OptionPassing(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [ 
				CudaCompile.OptionPassing.Msvc9Default
			]
		def name(self) :
			return "OptionPassing"
		class _Compile(ILevel3,ICudaCompileOption) :
			"Specify options directly to the compiler/preprocessor."
			def parent(self) :
				return CudaCompile.OptionPassing
			def name(self) :
				return "--compiler-options "+str(self.value)
			def __init__(self,value) :
				self.value=value
	
	class _GPUCodeGeneration(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [ 
				CudaCompile.GPUCodeGeneration.MaxRegCountDefault
			]
		def name(self) :
			return "GPUCodeGeneration"
		class _MaxRegCount(ILevel3,ICudaCompileOption) :
			"""
Specify the maximum amount of registers that 
GPU functions can use. 
Until a function-specific limit, a higher value 
will generally increase the performance of 
individual GPU threads that execute this 
function. However, because thread registers are 
allocated from a global register pool on each 
GPU, a higher value of this option will also 
reduce the maximum thread block size, thereby 
reducing the amount ofthread parallelism. 
Hence, a good maxrregcount value is the result 
of a trade-off. 
If this option is not specified, then no 
maximum is assumed. Otherwise the specified 
value will be rounded to the next multiple of 4 
registers until the GPU specific maximum of 
128 registers.			
			"""
			def parent(self) :
				return CudaCompile.GPUCodeGeneration
			def name(self) :
				return "--maxrregcount="+str(self.value)
			def __init__(self,value) :
				self.value=value

	class _CodeGeneration(ILevel2) :
		def parent(self) :
			return CudaCompile
		def children(self) :
			return [ 
				CudaCompile.CodeGeneration.Debug
			]
		def name(self) :
			return "CodeGeneration"
		class _Debug(ILevel3,ICudaCompileOption) :
			"Generate debuggable code"
			def parent(self) :
				return CudaCompile.CodeGeneration
			def name(self) :
				return "--debug "+str(self.value)
			def __init__(self,value="") :
				self.value=value
				
		

CudaCompile=_CudaCompile()
CudaCompile.CompilationPhase=CudaCompile._CompilationPhase()
CudaCompile.CompilationPhase.Compile=CudaCompile.CompilationPhase._Compile()
CudaCompile.ApplicationCompatibility=CudaCompile._ApplicationCompatibility()
CudaCompile.ApplicationCompatibility.ComputeCap10=CudaCompile.ApplicationCompatibility._ComputeCap(10)
CudaCompile.ApplicationCompatibility.ComputeCap20=CudaCompile.ApplicationCompatibility._ComputeCap(20)
CudaCompile.Architecture=CudaCompile._Architecture()
CudaCompile.Architecture.Machine32=CudaCompile.Architecture._Machine(32)
CudaCompile.Architecture.Machine64=CudaCompile.Architecture._Machine(64)
CudaCompile.FileAndPath=CudaCompile._FileAndPath()
CudaCompile.FileAndPath.CCompilerBinDefault=CudaCompile.FileAndPath.CCompilerBin('"$(CPP_BIN)"')
CudaCompile.FileAndPath.IncludeDefault=CudaCompile.FileAndPath.Include('"$(CUDA_INCLUDE)"')
CudaCompile.OptionPassing=CudaCompile._OptionPassing()
CudaCompile.OptionPassing.Msvc9Default=CudaCompile.OptionPassing._Compile('"/EHsc /W3 /nologo /O2 /Zi /MT"')
CudaCompile.GPUCodeGeneration=CudaCompile._GPUCodeGeneration()
CudaCompile.GPUCodeGeneration.MaxRegCountDefault=CudaCompile.GPUCodeGeneration._MaxRegCount(0)
CudaCompile.CodeGeneration=CudaCompile._CodeGeneration()
CudaCompile.CodeGeneration.Debug=CudaCompile.CodeGeneration._Debug()
