class ILinkerOption(IOption) :
	def idString(self) :
		return self.__class__.__name__
	def compile(self) :
		return []
	def link(self,**kwargs) :
		return [self]
	def link(self) :
		return []

class _Linker(ILevel1) :
	def children(self) :
		return [
			Linker.SuppressStartupBanner,
			Linker.LinkIncrementally,
			Linker.BuildDll,
			Linker.TargetPlatform,
			Linker.Subsystem
		]
	def name(self) :
		return "Linker"

	class _SuppressStartupBanner(ILevel2) :
		"""
Suppresses the display of the sign-on banner when the compiler starts up and display of informational messages during compiling.
	/nologo
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Click the C/C++ folder.
	Click the General property page.
	Modify the Suppress Startup Banner property.		
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [Linker.SuppressStartupBanner.NoLogo]
		def name(self) :
			return "SuppressStartupBanner"
		class _NoLogo(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.SuppressStartupBanner
			def name(self) :
				return "/NOLOGO"
			
	class _LinkIncrementally(ILevel2) :
		"""
	/INCREMENTAL[:NO]
Remarks
	The /INCREMENTAL option controls how the linker handles incremental linking.
	By default, the linker runs in incremental mode. To override a default incremental link, specify /INCREMENTAL:NO.
	An incrementally linked program is functionally equivalent to a program that is nonincrementally linked. However, because it is prepared for subsequent incremental links, an incrementally linked executable (.exe) file or dynamic-link library (DLL):
	Is larger than a nonincrementally linked program because of padding of code and data. (Padding allows the linker to increase the size of functions and data without recreating the .exe file.)
	May contain jump thunks to handle relocation of functions to new addresses.

Note
	To ensure that your final release build does not contain padding or thunks, link your program nonincrementally.

To link incrementally regardless of the default, specify /INCREMENTAL. When this option is selected, the linker issues a warning if it cannot link incrementally, and then links the program nonincrementally. Certain options and situations override /INCREMENTAL.
Most programs can be linked incrementally. However, some changes are too great, and some options are incompatible with incremental linking. LINK performs a full link if any of the following options are specified:
	Link Incrementally is not selected (/INCREMENTAL:NO)
	/OPT:REF is selected
	/OPT:ICF is selected
	/ORDER is selected
/INCREMENTAL is implied when /DEBUG is specified.
Additionally, LINK performs a full link if any of the following situations occur:
	The incremental status (.ilk) file is missing. (LINK creates a new .ilk file in preparation for subsequent incremental linking.)
	There is no write permission for the .ilk file. (LINK ignores the .ilk file and links nonincrementally.)
	The .exe or .dll output file is missing.
	The timestamp of the .ilk, .exe, or .dll is changed.
	A LINK option is changed. Most LINK options, when changed between builds, cause a full link.
	An object (.obj) file is added or omitted.
	An object that was compiled with the /Yu /Z7 option is changed.

To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the General property page.
	Modify the Enable Incremental Linking property.		
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [
				Linker.LinkIncrementally.Incremental,
				Linker.LinkIncrementally.IncrementalNo
			]
		def name(self) :
			return "LinkIncrementally"
		class _Incremental(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.LinkIncrementally
			def name(self) :
				return "/INCREMENTAL"
		class _IncrementalNo(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.LinkIncrementally
			def name(self) :
				return "/INCREMENTAL:NO"
	
	class _BuildDll(ILevel2) :
		"""
	/DLL
Remarks
	The /DLL option builds a DLL as the main output file. A DLL usually contains exports that can be used by another program. There are three methods for specifying exports, listed in recommended order of use:
		__declspec(dllexport) in the source code
		An EXPORTS statement in a .def file
		An /EXPORT specification in a LINK command
	A program can use more than one method.

	Another way to build a DLL is with the LIBRARY module-definition statement. The /BASE and /DLL options together are equivalent to the LIBRARY statement.
	Do not specify this option within the development environment; this option is for use only on the command line. This option is set when you create a DLL project with an Application Wizard.
	Note that if you create your import library in a preliminary step, before creating your .dll, you must pass the same set of object files when building the .dll, as you passed when building the import library.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Configuration Properties folder.
	Click the General property page.
	Modify the Configuration Type property.
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.BuilDll.Dll	]
		def name(self) :
			return "BuildDll"
		class _Dll(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.BuildDll
			def name(self) :
				return "/DLL"
	
	class _TargetPlatform(ILevel2) :
		"""
	/MACHINE:{ARM|EBC|IA64|MIPS|MIPS16|MIPSFPU|MIPSFPU16|SH4|THUMB|X64|X86}
Remarks
	The /MACHINE option specifies the target platform for the program.
	Usually, you do not need to specify the /MACHINE option. LINK infers the machine type from the .obj files. However, in some circumstances, LINK cannot determine the machine type and issues a linker tools error LNK1113. If such an error occurs, specify /MACHINE.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the Advanced property page.
	Modify the Target Machine property.		
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.TargetPlatform.Machine_X86 ]
		def name(self) :
			return "TargetPlatform"
		class _Machine_X86(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.TargetPlatform
			def name(self) :
				return "/MACHINE:X86"
	
	class _Subsystem(ILevel2) :
		"""
/SUBSYSTEM:{CONSOLE|EFI_APPLICATION|EFI_BOOT_SERVICE_DRIVER|
            EFI_ROM|EFI_RUNTIME_DRIVER|NATIVE|POSIX|WINDOWS|WINDOWSCE}
            [,major[.minor]]
where:
CONSOLE
	Win32 character-mode application. Console applications are given a console by the operating system. If main or wmain is defined, CONSOLE is the default.
Extensible Firmware Interface
	The EFI_* subsystems. See the EFI specification for more information. For example, see the Intel web site. The minimum version and default version is 1.0.
NATIVE
	Device drivers for Windows NT. If /DRIVER:WDM is specified, NATIVE is the default.
POSIX
	Application that runs with the POSIX subsystem in Windows NT.
WINDOWS
	Application does not require a console, probably because it creates its own windows for interaction with the user. If WinMain or wWinMain is defined, WINDOWS is the default.
WINDOWSCE
	Application that runs on a Windows CE device.
majorand minor(optional)
	Specify the minimum required version of the subsystem. The arguments are decimal numbers in the range 0 through 65,535. See the Remarks for details. There are no upper bounds for version numbers.
Remarks
	The /SUBSYSTEM option tells the operating system how to run the .exe file.
	The choice of subsystem affects the entry point symbol (or entry point function) that the linker will choose.
	The optional minimum and default major and minor version numbers for the subsystems are as follows.
Subsystem						 Minimum			 						Default
CONSOLE							4.00 (x86) 5.01 (Itanium)5.02 (x64)		4.00 (x86) 5.01 (Itanium)5.02 (x64)
WINDOWS							4.00 (x86) 5.01 (Itanium) 5.02 (x64)	4.00 (x86) 5.01 (Itanium) 5.02 (x64)
NATIVE (with DRIVER:WDM)		1.00 (x86) 1.10 (Itanium) 1.10 (x64)	1.00 (x86) 1.10 (Itanium) 1.10 (x64)
NATIVE (without /DRIVER:WDM)	4.00 (x86) 5.01 (Itanium) 5.02 (x64)	4.00 (x86) 5.01 (Itanium) 5.02 (x64)
POSIX							1.0										19.90
WINDOWSCE						3.0										4.10
EFI_APPLICATION, 				1.0										1.0
	EFI_BOOT_SERVICE_DRIVER, 
	EFI_ROM, 
	EFI_RUNTIME_DRIVER

To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the System property page.
	Modify the SubSystem property.		
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.Subsystem.Console ]
		def name(self) :
			return "Subsystem"
		class _Console(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Subsystem
			def name(self) :
				return "/subsystem:console"
		class _Windows(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Subsystem
			def name(self) :
				return "/subsystem:windows"

	class Out(ILevel2,ILinkerOption) :
		"""
/OUT:filename
	where:
	filename
		A user-specified name for the output file. It replaces the default name.
Remarks
	The /OUT option overrides the default name and location of the program that the linker creates.
	By default, the linker forms the file name using the base name of the first .obj file specified and the appropriate extension (.exe or .dll).
	This option the default base name for a .mapfile or import library. For details, see Generate Mapfile (/MAP) and /IMPLIB.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the General property page.
	Modify the Output File property.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/out:'+str(self.value)
				
	class ImpLib(ILevel2,ILinkerOption) :
		"""
/IMPLIB:filename
where:
	filename
		A user-specified name for the import library. It replaces the default name.
Remarks
	The /IMPLIB option overrides the default name for the import library that LINK creates when it builds a program that contains exports. The default name is formed from the base name of the main output file and the extension .lib. A program contains exports if one or more of the following are specified:
		The __declspec(dllexport) keyword in the source code
		EXPORTS statement in a .def file
		An /EXPORT specification in a LINK command
	LINK ignores /IMPLIB when an import library is not being created. If no exports are specified, LINK does not create an import library. If an export file is used in the build, LINK assumes that an import library already exists and does not create one. For information on import libraries and export files, see LIB Reference.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the Advanced property page.
	Modify the Import Library property.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/IMPLIB:'+str(self.value)

	class LibPath(ILevel2,ILinkerOption) :
		"""
		/LIBPATH:dir
	where:
		dir
			Specifies a path that the linker will search before it searches the path specified in the LIB environment option.
Remarks
	Use the /LIBPATH option to override the environment library path. The linker will first search in the path specified by this option, and then search in the path specified in the LIB environment variable. You can specify only one directory for each /LIBPATH option you enter. If you want to specify more than one directory, you must specify multiple /LIBPATH options. The linker will then search the specified directories in order.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the General property page.
	Modify the Additional Library Directories property.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/LIBPATH:'+str(self.value)

	class _Optimization(ILevel2) :
		"""
Controls the optimizations that LINK performs during a build.
	/OPT:{REF | NOREF}
	/OPT:{ICF[=iterations] | NOICF}
	/OPT:{WIN98 | NOWIN98}
Arguments
	REF | NOREF
		/OPT:REF eliminates functions and/or data that are never referenced while /OPT:NOREF keeps functions and/or data that are never referenced.
		LINK removes unreferenced packaged functions by default. An object contains packaged functions (COMDATs) if it has been compiled with the /Gy option. This optimization is called transitive COMDAT elimination. To override this default and keep unreferenced COMDATs in the program, specify /OPT:NOREF. You can use the /INCLUDE option to override the removal of a specific symbol.
		If /DEBUG is specified, the default for /OPT is NOREF (otherwise, it is REF), and all functions are preserved in the image. To override this default and optimize a debugging build, specify /OPT:REF. The /OPT:REF option disables incremental linking.
		You have to explicitly mark data as a COMDAT; use __declspec(selectany).
		If /OPT:REF is specified, /OPT:ICF is on by default. If you want /OPT:REF but not /OPT:ICF, you must specify the following:
			link /opt:ref /opt:noicf
		Specifying /OPT:ICF does not activate the /OPT:REF option.
	ICF[= iterations ] | NOICF
		Use /OPT:ICF[=iterations] to perform identical COMDAT folding. Redundant COMDATs can be removed from the linker output. iterations specifies the number of times to traverse the symbols for duplicates. The default number of iterations is two. Additional iterations may locate more duplicates uncovered through folding in the previous iteration.
		Note that there is a difference in linker behavior when ICF is in effect by default with /OPT:REF explicitly specified and when you explicitly specify /OPT:REF,ICF. The default ICF with /OPT:REF does not fold read-only data. This includes any .rdata, .pdata, and .xdata. However, the default ICF with /OPT:REF results in fewer functions folded when producing images for Itanium and x64 because functions in these modules have more read only data dependency, such as .pdata and .xdata. To get full ICF, explicitly specify /OPT:ICF.
		Functions are placed in COMDATs with the /Gy compiler option and const data is placed in COMDATs. See selectany for an example of how to specify data for folding.
		ICF is on by default if REF is on and needs to be explicitly turned on in a debug build. It is possible to specify NOICF if REF is specified.
	Note
		/OPT:ICF can result in the same address being assigned to different functions or read only data members (const variables compiled with /Gy). So, /OPT:ICF can break a program that depends on the address of functions or read-only data members being different. See /Gy (Enable Function-Level Linking) for more information.
	WIN98 | NOWIN98
		WIN98 and NOWIN98 control the section alignment in the final image. For Windows 98 applications, it is optimal to align sections on a 4K boundary to improve load time (allows Windows 98 memory manager to cache executable images with a minimum of wasted space). This is on by default in the linker, so you need to specify /OPT:NOWIN98 to get a trimmed-down (but slower on Windows 98) version of the application.
		WIN98 is on by default. WIN98 is not on when:
			/ALIGN is used.
			/MACHINE does not target x86.
			/SUBSYSTEM specifies something other than WINDOWS or CONSOLE.
		/OPT:WIN98 is not enabled by default for images that would grow (according to the average growth equations, below) by more than 25 percent. In other words, /OPT:WIN98 would not be enabled for smaller images. You should enable /OPT:WIN98 explicitly to ensure that you are not affected by this tuning. Specify /OPT:NOWIN98 to get a smaller (but slower on Windows 98) version of your application.
		The enhancements in Windows 98 only work when the sections in a portable executable image begin on a page boundary. The /OPT:WIN98 option performs the necessary file alignment.
		If you are building components that run only on Windows NT or Windows 2000, you should use /OPT:NOWIN98.
		This change does not impact loading of images or the working set of the process. The only impact is to the on-disk size.
		The following helps you compute the average growth of an image using /OPT:WIN98:
			The average wasted space for 4096-byte file alignment can be characterized by: count-of-sections-in-image * 4096/2
			The average wasted space for the current 512-byte file alignment is: count-of-sections-in-image * 512/2
		The growth is therefore:
			count-of-bytes-Growth = count-of-sections-in-image * (4096/2 - 512/2)
				or, simplified,
				count-of-bytes-Growth = count-of-sections-in-image * 1792
				However, this does not take into account the fact that the image header must be padded to the section alignment. Since the header is always 512 byes or less, the extra growth is a constant of 4096 - 512, or 3584.
			average count-of-bytes-Growth = count-of-sections-in-image * 1792 + 3584
			maximum count-of-bytes-Growth = count-of-sections-in-image * (4096 - 512 + 3584
		To get the count of sections, use the DUMPBIN tool on an executable file. The summary will give you a list of sections in that image. Typically, you will see from 3 to 5 sections added to that value.
		The only time you should not use /OPT:WIN98 is when your portable executable image is very small. Even if an image is slated for downloads, the wasted space is zero-filled and compresses well.
Remarks
	Optimizations generally decrease the image size and increase the program speed at a cost of increased link time.
	You can use the /VERBOSE option to see the functions removed by /OPT:REF and the functions that are folded by /OPT:ICF.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the Optimization property page.
	Modify one of the following properties:
		Enable COMDAT Folding
		Optimize for Windows98
		References
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.Optimization.Ref, Linker.Optimization.Icf ]
		def name(self) :
			return "Optimization"
		class _Ref(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Optimization
			def name(self) :
				return "/opt:ref"
		class _Icf(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Optimization
			def name(self) :
				return "/opt:icf"
	
	class Def(ILevel2,ILinkerOption) :
		"""
/DEF:filename
where:
	filename
		The name of a module-definition file (.def) to be passed to the linker.
Remarks
	The /DEF option passes a module-definition file (.def) to the linker. Only one .def file can be specified to LINK. For details about .def files, see Module-Definition Files.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the Input property page.
	Modify the Module Definition File property.
To specify a .def file from within the development environment, you should add it to the project along with other files and then specify the file to the /DEF option.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/DEF:'+str(self.value)
			
	class _Manifest(ILevel2) :
		"""
	/MANIFEST[:NO]
Remarks
	/MANIFEST specifies that the linker should create a side-by-side manifest file. For more information on manifest files, see Manifest Files Reference.
	The default is /MANIFEST.
	If /MANIFEST is not specified and the linker is passed /MANIFESTDEPENDENCY comments, a manifest file will be created. A manifest file will not be created if /MANIFEST:NO is specified.
	If you specify /MANIFEST, the name of the manifest file will be the same as the name of your output file, with .manifest appended to the file name. For example, if your output file name is MyFile.exe, the manifest file name will be MyFile.exe.manifest. If you specify /MANIFESTFILE:name, the name of the manifest will be what you specify in name.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Expand the Configuration Properties node.
	Expand the Linker node.
	Select the Manifest File property page.
	Modify the Generate Manifest property.
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.Manifest.Yes, Linker.Manifest.No ]
		def name(self) :
			return "Manifest"
		class _Yes(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Manifest
			def name(self) :
				return '/MANIFEST'
		class _No(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.Manifest
			def name(self) :
				return '/MANIFEST:NO'

	class ManifestFile(ILevel2,ILinkerOption) :
		"""
	/MANIFESTFILE:filename
Remarks
	/MANIFESTFILE lets you change the default name of the manifest file. The default name of the manifest file is the file name with .manifest appended.
	/MANIFESTFILE will have no effect if you do not also link with /MANIFEST.

To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Expand the Configuration Properties node.
	Expand the Linker node.
	Select the Manifest File property page.
	Modify the Manifest File property.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/MANIFESTFILE:'+str(self.value)
			
			
	class _ManifestUAC(ILevel2) :
		"""
Specifies whether User Account Control (UAC) information is embedded in the program manifest.
	/MANIFESTUAC
	/MANIFESTUAC:NO
	/MANIFESTUAC:fragment
	/MANIFESTUAC:level=_level
	/MANIFESTUAC:uiAccess=_uiAccess
Parameters
	fragment
		A string that contains the level and uiAccess values. For more information, see the Remarks section later in this topic.
	_level
		One of asInvoker, highestAvailable, or requireAdministrator. Defaults to asInvoker. For more information, see the Remarks section later in this topic.
	_uiAccess
		true if you want the application to bypass user interface protection levels and drive input to higher-permission windows on the desktop; otherwise, false. Defaults to false. Set to true only for user interface accessibility applications.
Remarks
	If you specify multiple /MANIFESTUAC options on the command-line, the last one entered takes precedence.
	The choices for /MANIFESTUAC:level are as follows:
		asInvoker: The application will run with the same permissions as the process that started it. The application can be elevated to a higher permission level by selecting Run as Administrator.
		highestAvailable: The application will run with the highest permission level that it can. If the user who starts the application is a member of the Administrators group, this option is the same as requireAdministrator. If the highest available permission level is higher than the level of the opening process, the system will prompt for credentials.
		requireAdministrator: The application will run with administrator permissions. The user who starts the application must be a member of the Administrators group. If the opening process is not running with administrative permissions, the system will prompt for credentials.
	You can specify the level and uiAccess values in one step by using the /MANIFESTUAC:fragment option. The fragment must be in the following form:
		"level=[ asInvoker | highestAvailable | requireAdministrator ] uiAccess=[ true | false ]"
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see How to: Open Project Property Pages.
	Expand the Configuration Properties node.
	Expand the Linker node.
	Select the Manifest File property page.
	Modify the Enable User Account Control (UAC), UAC Execution Level, and UAC Bypass UI Protection properties.
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.ManifestUAC.Default ]
		def name(self) :
			return "ManifestUAC"
		class _Default(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.ManifestUAC
			def name(self) :
				return "/MANIFESTUAC:"+'"'+"level='asInvoker' uiAccess='false'"+'"'

	class IgnoreLibrary(ILevel2,ILinkerOption) :
		"""
/NODEFAULTLIB[:library] 
where:
	library
		A library that you want the linker to ignore when it resolves external references.
Remarks
	The /NODEFAULTLIB option tells the linker to remove one or more default libraries from the list of libraries it searches when resolving external references.
	To create an .obj file that does not contain references to default libraries, use /Zl (Omit Default Library Name).
	By default, /NODEFAULTLIB removes all default libraries from the list of libraries it searches when resolving external references. The optional library parameter lets you remove a specified library or libraries from the list of libraries it searches when resolving external references. Specify one /NODEFAULTLIB option for each library you want to exclude.
	The linker resolves references to external definitions by searching first in libraries that you explicitly specify, then in default libraries specified with the /DEFAULTLIB option, and then in default libraries named in .obj files.
	/NODEFAULTLIB:library overrides /DEFAULTLIB:library when the same library name is specified in both.
	If you use /NODEFAULTLIB, for example, to build your program without the C run-time library, you may have to also use /ENTRY to specify the entry point (function) in your program. For more information, see C Run-Time Libraries.
To set this linker option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Setting Visual C++ Project Properties.
	Click the Linker folder.
	Click the Input property page.
	Select the Ignore All Default Libraries property or specify a list of the libraries you want to ignore in the Ignore Specific Library property. The Command Line property page will show the effect of the changes you make to these properties.
		"""
		def __init__(self,value) :
			self.value=value
		def parent(self) :
			return Linker
		def children(self) :
			return []
		def name(self) :
			return '/NODEFAULTLIB:'+str(self.value)

			
	class _LinkTimeCodeGeneration(ILevel2) :
		"""
/LTCG[:NOSTATUS|:STATUS|:PGINSTRUMENT|:PGOPTIMIZE|:PGUPDATE]
where:
	:NOSTATUS | :STATUS (optional)
		Specifies whether the linker should display a progress indicator showing what percentage of the link is complete. The default is to not display this status information.
	:PGINSTRUMENT (optional)
		Specifies that the linker should output a .pgd file in preparation for instrumented test runs on the application. You can optionally specify /PGD to create the .pgd file with a nondefault name or location.
		The data collected from the instrumented runs will be used to create an optimized image. For more information, see Profile Guided Optimization. The short form of this option is /LTCG:PGI.
	:PGOPTIMIZE (optional)
		Specifies that the linker should use the profile data created after running the instrumented binary to create an optimized image. All input files must be identical to the files that were specified with /LTCG:PGI. For more information, see Profile Guided Optimization. The short form of this option is /LTCG:PGO.
	:PGUPDATE (optional)
		Allows list of input files to be added or modified from what was specified in the :PGINSTRUMENT phase. However, any new input files will not be optimized with profile-guided optimizations and changed portions of a modified input file that invalidate profile data collected during the instrumentation phase for that code will not be optimized with profile-guided optimizations. For more information, see Profile Guided Optimization. The short form of this option is /LTCG:PGU.
Remarks
	The /LTCG option tells the linker to call the compiler and perform whole program optimization. You can also do profile guided optimization. For more information, see Profile Guided Optimization.
	With the following exceptions, you cannot add additional linker options to the /LTCG:PGOPTIMIZE or /LTCG:PGUPDATE run that were not specified in the /LTCG:PGINSTRUMENT run:
		/BASE
		/FIXED
		/LTCG
		/MAP
		/MAPINFO
		/NOLOGO
		/OUT
		/PGD
		/PDB
		/PDBSTRIPPED
		/STUB
		/VERBOSE
	Any linker options specified to /LTCG:PGINSTRUMENT do not have to be specified to /LTCG:PGOPTIMIZE; they are implied.
	The rest of this topic will only discuss /LTCG in terms of link-time code generation.
	/LTCG is implied with /GL.
	The linker invokes link-time code generation if it is passed a module that was compiled with /GL or an MSIL module (see .netmodule Files as Linker Input for more information). If you do not explicitly specify /LTCG when passing /GL or MSIL modules to the linker, the linker will eventually detect this and restart the link with /LTCG. Explicitly specify /LTCG when passing /GL and MSIL modules to the linker for the fastest possible build performance.
	/LTCG is not valid for use with /INCREMENTAL.
	When /LTCG is used with either /Og, /O1, /O2, or /Ox, the following optimizations are performed:
		Cross-module inlining
		Interprocedural register allocation (64-bit operating systems only)
		Custom calling convention (x86 only)
		Small TLS displacement (x86 only)
		Stack double alignment (x86 only)
		Improved memory disambiguation (better interference information for global variables and input parameters)
	Using /LTCG and /Ogt will result in double-alignment optimization.
	If /LTCG and /Ogs are specified, double alignment will not be performed. If most of the functions in an application are compiled for speed, with a few functions compiled for size (for example, by using the optimize pragma), the compiler will double align these functions that are optimized for size if they call functions that need double alignment.
	If the compiler can identify all the call sites of a function, the compiler will ignore explicit calling-convention modifiers on a function and try to optimize the function's calling convention:
		pass parameters in registers
		reorder parameters for alignment
		remove unused parameters
	If a function is called via function pointer, or if a function make be called outside of a module compiled with /GL, the compiler will not attempt to optimize a function's calling convention.
Note
	If you use /LTCG and redefine mainCRTStartup, your application can have unpredictable behavior relating to user code that executes before global objects are initialized. There are three ways to address this issue: do not redefine mainCRTStartup, do not compile the file containing mainCRTStartup with /LTCG, or initialize global variables and objects statically, if possible.
/LTCG and MSIL Modules
	Modules compiled with /GL and /clr can be used as input to the linker when /LTCG is specified:
		/LTCG can accept native object files; mixed native/managed object files (compiled with /clr), pure object files (compiled with /clr:pure), and safe object files (compiled with /clr:safe)
		/LTCG can accept safe .netmodules, which can be created with /clr:safe /LN in Visual C++ and /target:module with any other Visual Studio compiler. .netmodules produced with /clr or /clr:pure are not accepted by /LTCG.
		/LTCG:PGI does not accept native modules compiled with /GL and /clr, or pure modules (produced with /clr:pure)
To set this compiler option in the Visual Studio development environment
	Open the project's Property Pages dialog box. For details, see Modifying Project Settings.
	Click the Configuration Properties folder.
	Click the General property page.
	Modify the Whole Program Optimization property.
You can also apply /LTCG to specific builds by choosing Profile Guided Optimization from the Build menu, or by right clicking on the project name in Solution Explorer and selecting one of the Profile Guided Optimization options.
		"""
		def parent(self) :
			return Linker
		def children(self) :
			return [ Linker.LinkTimeCodeGeneration.Yes ]
		def name(self) :
			return "LinkTimeCodeGeneration"
		class _Yes(ILevel3,ILinkerOption) :
			"See parent's doc string"
			def parent(self) :
				return Linker.LinkTimeCodeGeneration
			def name(self) :
				return '/LTCG'
			
			
Linker=_Linker()
Linker.SuppressStartupBanner=Linker._SuppressStartupBanner()
Linker.SuppressStartupBanner.NoLogo=Linker.SuppressStartupBanner._NoLogo()
Linker.LinkIncrementally=Linker._LinkIncrementally()
Linker.LinkIncrementally.Incremental=Linker.LinkIncrementally._Incremental()
Linker.LinkIncrementally.IncrementalNo=Linker.LinkIncrementally._IncrementalNo()
Linker.BuildDll=Linker._BuildDll()
Linker.BuildDll.Dll=Linker.BuildDll._Dll()
Linker.TargetPlatform=Linker._TargetPlatform()
Linker.TargetPlatform.Machine_X86=Linker.TargetPlatform._Machine_X86()
Linker.Subsystem=Linker._Subsystem()
Linker.Subsystem.Console=Linker.Subsystem._Console()
Linker.Subsystem.Windows=Linker.Subsystem._Windows()
Linker.ManifestUAC=Linker._ManifestUAC()
Linker.ManifestUAC.Default=Linker.ManifestUAC._Default()
Linker.Manifest=Linker._Manifest()
Linker.Manifest.Yes=Linker.Manifest._Yes()
Linker.Manifest.No=Linker.Manifest._No()
Linker.Optimization=Linker._Optimization()
Linker.Optimization.Ref=Linker.Optimization._Ref()
Linker.Optimization.Icf=Linker.Optimization._Icf()
Linker.LinkTimeCodeGeneration=Linker._LinkTimeCodeGeneration()
Linker.LinkTimeCodeGeneration.Yes=Linker.LinkTimeCodeGeneration._Yes()

