# Assumptions (later to be changed):
#	1. Always creating a multithreaded binary.
#	2. Always using Windows and MS runtime.
#
#

class IOption(object) :
	def idString(self) :
		raise(Exception('idString is not implemented'))
	def compile(self,**kwargs) :
		raise(Exception('compile is not implemented'))
	def link(self,**kwargs) :
		raise(Exception('link is not implemented'))
	def libs(self,**kwargs) :
		raise(Exception('libs is not implemented'))

class ICudaOption(object) :
	def idString(self) :
		raise(Exception('idString is not implemented'))
	def cudaCompile(self,**kwargs) :
		raise(Exception('cudaCompile is not implemented'))
	def cudaLink(self,**kwargs) :
		raise(Exception('cudaLink is not implemented'))
	def cudaLibs(self,**kwargs) :
		raise(Exception('cudaLibs is not implemented'))
		

class ILevel1(object) :
	def parent(self) :
		import ots.make.options as opt
		return opt
	def children(self) :
		raise(Exception('children is not implemented'))
	def name(self) :
		raise(Exception('name is not implemented'))
	def __str__(self) :
		return self.parent().__name__+'.'+self.name()
class ILevel2(object) :	
	def parent(self) :
		raise(Exception('parent is not implemented'))	
	def children(self) :
		raise(Exception('children is not implemented'))
	def name(self) :
		raise(Exception('name is not implemented'))
	def __str__(self) :
		return self.parent().__str__()+'.'+self.name()
class ILevel3(object) :	
	def parent(self) :
		raise(Exception('parent is not implemented'))	
	def name(self) :
		raise(Exception('name is not implemented'))
	def __str__(self) :
		return self.parent().__str__()+'.'+self.name()

class _Version(ILevel1) :
	#ILevel1
	def name(self) :
		return "Version"
	def children(self) :
		return [Version.Release,Version.Debug]
	class _Release(ILevel2,IOption) :
		#ILevel2
		def parent(self) :
			return Version
		def children(self) :
			return []
		def name(self) :
			return "Release"
		#IOption
		def idString(self) :
			return "release"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Compile.OptimizeCode.O2,
				opt.Compile.OptimizeCode.Ob2,
				opt.Compile.WarningLevels.W3,
				opt.Compile.WarningLevels.wd4675,
				opt.Compile.RunTimeTypeInfo.GR,
				opt.Compile.BufferSecurityCheck.GS,
				opt.Compile.RunTimeLibrary.MD,
				opt.Compile.CompileWithoutLinking.c
			]
		def link(self,**kwargs) :
			return []
		def libs(self,**kwargs) :
			return []
	#The options 
	#	opt.Compile.Conformance.Zc_forScope,
	#	opt.Compile.Conformance.Zc_wchar_t,
	#are not compatible with default built of stlp when building with msvc71
	class _Debug(ILevel2,IOption,ICudaOption) :
		#ILevel2
		def parent(self) :
			return Version
		def children(self) :
			return []
		def name(self) :
			return "Debug"
		#IOption
		def idString(self) :
			return "debug"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Compile.DbgInfoFormat.Z7,
				opt.Compile.OptimizeCode.Od,
				opt.Compile.OptimizeCode.Ob0,
				opt.Compile.WarningLevels.W3,
				opt.Compile.WarningLevels.wd4675,
				opt.Compile.RunTimeTypeInfo.GR,
				opt.Compile.BufferSecurityCheck.GS,
				opt.Compile.RunTimeLibrary.MDd,
				opt.Compile.CompileWithoutLinking.c
			]
		def link(self,**kwargs) :
			return []
		def libs(self,**kwargs) :
			return []
		#ICudaOption
		def cudaCompile(self,**kwargs) :
			return [ CudaCompile.CodeGeneration.Debug ]
		def cudaLink(self,**kwargs) :
			return []
		def cudaLibs(self,**kwargs) :
			return []
Version=_Version()
Version.Release=Version._Release()
Version.Debug=Version._Debug()

class IRuntime(object) :
	def setEnvBat(self) :
		raise(Exception('vcvars32bat is not implemented'))
	def nmake(self) :
		raise(Exception('nmake is not implemented'))
	
class _Runtime(ILevel1) :
	#ILevel1
	def children(self) :
		return [Runtime.Msvc71,Runtime.Msvc9]
	def name(self) :
		return "Runtime"
	class _Msvc71(ILevel2,IOption,IRuntime) :
		#ILevel2
		def parent(self) :
			return Runtime
		def children(self) :
			return []
		def name(self) :
			return "Msvc71"
		#IOption
		def idString(self) :
			return "msvc71"
		def compile(self,**kwargs) :
			return []
		def link(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Linker.SuppressStartupBanner.NoLogo,
				opt.Linker.LinkIncrementally.IncrementalNo
			]
		def libs(self,**kwargs) :
			return []
		#IRuntime
		def setEnvBat(self) :
			import ots.make.options as opt
			return opt.Dir.Msvc71Tools+'vsvars32.bat'
		def nmake(self) :
			import ots.make.options as opt
			return opt.Dir.Msvc71Bin+'nmake.exe'			
	#The options 
	#	opt.Compile.Conformance.Zc_forScope,
	#	opt.Compile.Conformance.Zc_wchar_t,
	#are not compatible with default built of stlp when building with msvc71
	class _Msvc9(ILevel2,IOption,IRuntime,ICudaOption) :
		#ILevel2
		def parent(self) :
			return Runtime
		def children(self) :
			return []
		def name(self) :
			return "Msvc9"
		#IOption
		def idString(self) :
			return "msvc9"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Compile.Conformance.Zc_forScope,
				opt.Compile.Conformance.Zc_wchar_t
				]
		def link(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Linker.SuppressStartupBanner.NoLogo,
				opt.Linker.LinkIncrementally.IncrementalNo,
				opt.Linker.TargetPlatform.Machine_X86,
				opt.Linker.Manifest.Yes
			]
		def libs(self,**kwargs) :
			return []
		#IRuntime
		def setEnvBat(self) :
			import ots.make.options as opt
			return opt.Dir.Msvc9Tools+'vsvars32.bat'
		def nmake(self) :
			import ots.make.options as opt
			return opt.Dir.Msvc9Bin+'nmake.exe'			
		#ICudaOption
		def cudaCompile(self,**kwargs) :
			return [
				#CudaCompile.ApplicationCompatibility.ComputeCap10,
				CudaCompile.ApplicationCompatibility.ComputeCap20,
				CudaCompile.ApplicationCompatibility.ComputeCap20,
				CudaCompile.Architecture.Machine32,
				CudaCompile.FileAndPath.CCompilerBinDefault,
				CudaCompile.FileAndPath.IncludeDefault,
				CudaCompile.OptionPassing.Msvc9Default,
				CudaCompile.GPUCodeGeneration.MaxRegCountDefault,
				CudaCompile.CompilationPhase.Compile
			]
		def cudaLink(self,**kwargs) :
			return []
		def cudaLibs(self,**kwargs) :
			return []
				
Runtime=_Runtime()
Runtime.Msvc71=Runtime._Msvc71()
Runtime.Msvc9=Runtime._Msvc9()

class IOutput(object) :
	def ext(self,**kwargs) :
		raise(Exception('ext is not implemented'))
		
class IComOutput(object) :
	def compile(self,**kwargs) :
		import ots.make.options as opt
		r=[
			opt.Compile.ExceptionHandling.EHsc,
			opt.Compile.Preprocessor('WIN32'),
			opt.Compile.Preprocessor('_WINDOWS'),
			opt.Compile.Preprocessor('_MBCS')
		]
		if( kwargs['runtime']==opt.Runtime.Msvc71 ) :
			r.append(opt.Compile.Detect64BitPortability.Wp64)
		else :
			r.append(opt.Compile.Preprocessor('_UNICODE'))
			r.append(opt.Compile.Preprocessor('UNICODE'))
		return r
	def link(self,**kwargs) :
		import ots.make.options as opt
		import ots.io as io
		r=[ 
			opt.Linker.Subsystem.Windows,
			opt.Linker.TargetPlatform.Machine_X86,
			opt.Linker.Optimization.Ref,
			opt.Linker.Optimization.Icf
		]
		runtime=kwargs['runtime']
		if( runtime==opt.Runtime.Msvc9 ) :
			r.append(opt.Linker.Manifest.Yes)
			r.append(opt.Linker.ManifestUAC.Default)
		if( self.idString()=='exe' ) :
			r.append(opt.Linker.ManifestFile(
				'"$(OUTPUT_DIR)'+io.separator+'$(RESULT_NAME).exe.manifest"'
			))
		return r
	def libs(self,**kwargs) :
		return [
			'kernel32.lib',
			'user32.lib',
			'ole32.lib',
			'oleaut32.lib'
		]

	
class _Output(ILevel1) :
	#ILevel1
	def name(self) :
		return "Output"
	def children(self) :
		return [Output.Dll,Output.Pyd,Output.ComDll,Output.ComExe]
	class _Dll(ILevel2,IOption,IOutput) :
		#ILevel2
		def parent(self) :
			return Output
		def children(self) :
			return []
		def name(self) :
			return "Dll"
		#IOption
		def idString(self) :
			return "dll"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Compile.ExceptionHandling.EHs
			]
		def link(self,**kwargs) :
			import ots.make.options as opt
			return [ 
				opt.Linker.BuildDll.Dll,
				opt.Linker.Subsystem.Console
			]
		def libs(self,**kwargs) :
			return []
		#IOutput
		def ext(self,**kwargs) :
			return "dll"
	class _Pyd(ILevel2,IOption,IOutput) :
		#ILevel2
		def parent(self) :
			return Output
		def children(self) :
			return []
		def name(self) :
			return "Pyd"
		#IOption
		def idString(self) :
			return "pyd"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			return [
				opt.Compile.ExceptionHandling.EHs
			]
		def link(self,**kwargs) :
			import ots.make.options as opt
			return [ 
				opt.Linker.BuildDll.Dll,
				opt.Linker.Subsystem.Console
			]
		def libs(self,**kwargs) :
			return []
		#IOutput
		def ext(self,**kwargs) :
			return "pyd"
	class _ComDll(ILevel2,IOption,IOutput,IComOutput) :
		#ILevel2
		def parent(self) :
			return Output
		def children(self) :
			return []
		def name(self) :
			return "ComDll"
		#IOption
		def idString(self) :
			return "dll"
		def compile(self,**kwargs) :
			import ots.make.options as opt
			r=IComOutput.compile(self,**kwargs)
			r.append(opt.Compile.Preprocessor('_USRDLL'))
			r.append(opt.Compile.Preprocessor('_WINDLL'))
			return r
		def link(self,**kwargs) :
			import ots.make.options as opt
			r=IComOutput.link(self,**kwargs)
			r.append(opt.Linker.BuildDll.Dll)
			return r
		def libs(self,**kwargs) :
			return IComOutput.libs(self,**kwargs)
		#IOutput
		def ext(self,**kwargs) :
			return "dll"
	class _ComExe(ILevel2,IOption,IOutput,IComOutput) :
		#ILevel2
		def parent(self) :
			return Output
		def children(self) :
			return []
		def name(self) :
			return "ComExe"
		#IOption
		def idString(self) :
			return "exe"
		def compile(self,**kwargs) :
			r=IComOutput.compile(self,**kwargs)
			import ots.make.options as opt
			if( kwargs['runtime']==opt.Runtime.Msvc71 ) :
				r.append( opt.Compile.Preprocessor('_WIN32_DCOM') )
			return r
		def link(self,**kwargs) :
			import ots.make.options as opt
			r=IComOutput.link(self,**kwargs)
			return r
		def libs(self,**kwargs) :
			return IComOutput.libs(self,**kwargs)
		#IOutput
		def ext(self,**kwargs) :
			return "exe"
Output=_Output()
Output.Dll=Output._Dll()
Output.Pyd=Output._Pyd()
Output.ComDll=Output._ComDll()
Output.ComExe=Output._ComExe()
	
