import numpy as _n
import decimal as _d
import ots.std as _std


def floatToDecimal(f):
    # http://docs.python.org/library/decimal.html#decimal-faq
    "Convert a floating point number to a Decimal with no loss of information"
    n, d = f.as_integer_ratio()
    numerator, denominator = _d.Decimal(n), _d.Decimal(d)
    ctx = _d.Context(prec=60)
    result = ctx.divide(numerator, denominator)
    while ctx.flags[_d.Inexact]:
        ctx.flags[_d.Inexact] = False
        ctx.prec *= 2
        result = ctx.divide(numerator, denominator)
    return result 
	
def ToDecimal(x) :
	if( type(x)==_d.Decimal ) :
		return x
	if( _std.isOfIntType(x) ) :
		return _d.Decimal(x)
	if( _std.isOfFloatType(x) ) :
		return floatToDecimal(x)
	if( type(x)==_n.matrix ) :
		a=[]
		for i in range(0,x.shape[0]) :
			aa=[]
			a.append(aa)
			for j in range(0,x.shape[1]) :
				aa.append(ToDecimal(x[i,j]))
		return _n.matrix(a)
	raise(Exception('Unexpected data type'))

class Poly(object) :
	def __init__(self,data=0) :
		if( _std.isOfIntType(data) ) :
			self._degree=data
			self._coefficients=[_d.Decimal(0) for x in range(0,data+1)]
		elif( type(data)==type([]) ) :
			self._coefficients=[ToDecimal(x) for x in data]
			self._degree=len(data)-1
		else :
			raise(Exception("Polynomial's constructor takes either int or list"))
	def isZero(self) :
		for x in self.coeffs() :
			if( x!=0 ) :
				return False
		return True
	def degree(self) :
		return self._degree
	def coeffs(self) :
		return self._coefficients
	def __call__(self,x) :
		sum=0
		pwr=_d.Decimal(1)
		x=ToDecimal(x)
		for i in range(0,self.degree()+1) :
			sum+=self.coeffs()[i]*pwr
			pwr*=x
		return sum
	def __str__(self) :
		s="Poly(x)="
		for i in range(0,len(self.coeffs())) :
			if(self.coeffs()[i]>=0) :
				s+=str(self.coeffs()[i])
			else :
				s+='('+str(self.coeffs()[i])+')'
			if(i>0) :
				s+="*x"
			if(i>1) :
				s+="^"+str(i)
			s+=" + "
		if( len(self.coeffs())!=0 ) :
			s=s[:-2]
		else :
			s+='0'
		return s
	def __repr__(self) :
		return self.__str__()
	def __add__(self,p) :
		if( _std.isOfNumericType(p) ) :
			c=self.coeffs()
			x=[c[0]+p]
			for i in range(1,len(c)) :
				x.append(c[i])
			return Poly(c)
		if( len(self.coeffs())>len(p.coeffs()) ) :
			C=self.coeffs()
			c=p.coeffs()
		else :
			c=self.coeffs()
			C=p.coeffs()
		r=[]
		r.extend(C)
		for i in range(0,len(c)) :
			r[i]+=c[i]
		return Poly(r)
	def __mul__(self,z) :
		if( _std.isOfNumericType(z) ) :
			z=ToDecimal(z)
			return Poly([z*x for x in self.coeffs()])
		if(not type(z)==type(self)) :
			raise(Exception('Unexpected type in Poly.__mul__'))
		Na=self.degree()
		Nb=z.degree()
		a=self.coeffs()
		b=z.coeffs()
		r=[]
		for p in range(0,Na+Nb+1) :
			sum=0
			for i in range(max(0,p-Nb),min(p,Na)+1) :
				sum+=a[i]*b[p-i]
			r.append(sum)
		return Poly(r)
	def integral(self,piece) :
		sum=0
		a=ToDecimal(piece.a())
		b=ToDecimal(piece.b())
		powA=a
		powB=b
		i=0
		for c in self.coeffs() :
			sum+=c*(powB-powA)/(i+1)
			powA*=a
			powB*=b
			i+=1
		return sum
		
class Piece(object) :
	def __init__(self,a=1,b=0) :
		self._a=a
		self._b=b
	def isEmpty(self) :
		return self.a()>=self.b()
	def a(self) :
		return self._a
	def b(self) :
		return self._b
	def len(self) :
		if( self.a()>=self.b() ) :
			return 0
		return self.b()-self.a()
	def belongs(self,x) :
		return self.a()<=x and x<self.b()
	def inside(self,x) :
		return self.a()<x and x<self.b()
	def disjoint(self,piece) :
		return not self.belongs(piece.a()) and not self.belongs(piece.b())
	def __call__(self,x) :
		if( self.belongs(x) ) :
			return 1
		return 0
	def intersect(self,piece) :
		return Piece(max(self.a(),piece.a()),min(self.b(),piece.b()))
	def union(self,piece) :
		r=Piece(min(self.a(),piece.a()),max(self.b(),piece.b()))
		if( r.len()>self.len()+piece.len() ) :
			raise(Exception('Cannot represent a disjoint union'))
		return r
	def __str__(self) :
		return "["+str(self.a())+","+str(self.b())+")"
	def __repr__(self) :
		return self.__str__()
	def __cmp__(self,z) :
		if(type(z)==type(self)) :
			x=cmp(self.a(),z.a())
			if( x!=0 ) :
				return x
			return cmp(self.b(),z.b())
		if(_std.isOfNumericType(z)) :
			x=cmp(self.b(),z)
			if(x in [-1,0]) :
				return -1
			x=cmp(self.a(),z)
			if(x in [-1,0]) :
				return 0
			return 1
		raise(Exception('Wrong type in Piece.__cmp__()'))
	def difference(self,piece) :
		a1=self.a()
		b1=self.b()
		a2=piece.a()
		b2=piece.b()
		return _n.abs(a1-a2)+_n.abs(b1-b2)
		
class PPPair(object) :
	def __init__(self,poly,piece) :
		if( type(poly)!=type(Poly()) ) :
			raise(Exception("Poly expected"))
		if( type(piece)!=type(Piece()) ) :
			raise(Exception("Piece expected"))
		self._poly=poly
		self._piece=piece
	def piece(self) :
		return self._piece
	def poly(self) :
		return self._poly
	def __cmp__(self,ppp) :
		if(type(ppp)==type(self)):
			return self.piece().__cmp__(ppp.piece())
		return self.piece().__cmp__(ppp)
	def __str__(self) :
		return str(self.poly())+" for x in "+str(self.piece())
	def __repr__(self) :
		return self.__str__()
		
		
class PiecewisePoly(object) :
	def __init__(self,arg=[],cutTails=False) :
		if(type(arg)==type(())) :
			self._check(arg)
		elif(type(arg)==type([])) :
			for x in arg :
				self._check(x)
		else :
			self._check("fail")
		r=[]
		for (x,y) in arg :
			r.append(PPPair(x,y))
		r.sort()
		if(cutTails) :
			iA=0
			L=len(r)
			while(iA<L) :
				if( not r[iA].poly().isZero() ) :
					break
				iA=iA+1
			r=r[iA:]
			iB=len(r)-1
			while(iB>=0) :
				if( not r[iB].poly().isZero() ) :
					break
				iB=iB-1
			r=r[:(iB+1)]
		for i in range(0,len(r)-1) :
			if( r[i].piece().b()>r[i+1].piece().a() ) :
				raise(Exception("Inconsistent pieces"))
		self._args=r
		self._support=None
		self._len=None
	def support(self) :
		if( self._support is None ) :
			if( self.len()==0 ) :
				return Piece(0,0)
			self._support=Piece(self._args[0].piece().a(),self._args[self.len()-1].piece().b())
		return self._support
	def len(self) :
		if(self._len is None ) :
			self._len=len(self._args)
		return self._len
	def structure(self) :
		return [(x.poly(),x.piece()) for x in self._args]
	def _check(self,arg) :
		m="The argument of PiecewisePoly must have the structure [(Poly,Piece),...,(Poly,Piece)]"
		if(type(arg)!=type(())) :
			raise Exception(m)
		if(len(arg)!=2) :
			raise Exception(m)
		if(type(arg[0])!=type(Poly()) or type(arg[1])!=type(Piece())) :
			raise Exception(m)
	def __str__(self) :
		s="PiecewisePoly(\n"
		for x in self._args :
			s+=str(x)+"\n"
		return s+")"
	def __repr__(self) :
		return self.__str__()
	def _search(self,val) :
		if(self.isEmpty()) :
			return -1
		s=self.support()
		#calculate starting position
		try :
			i=int(_n.floor(self.len()*(val-s.a())/(s.b()-s.a())))
		except :
			val=float(str(val))
			i=int(_n.floor(self.len()*(val-s.a())/(s.b()-s.a())))
		if( i<0 or i>self.len() ) :
			return -1
		if( i==self.len() ) :
			i=i-1
		x=cmp(self._args[i],val)
		if(x==0) :
			return i
		step=(-1)*x
		#seek enveloping pair
		while(True) :
			i1=i+step
			if(i1<0) :
				i1=0
				if(i==i1) :
					return -1
			elif( i1>=self.len()) :
				i1=self.len()-1
				if(i==i1) :
					return -1
			else :
				step*=2
			x1=cmp(self._args[i1],val)
			if(x1==0) :
				return i1
			if(x*x1<0) :
				break
			i=i1
			x=x1
		#bisection
		while(True) :
			i0=int(_n.floor((i+i1)*0.5))
			x0=cmp(self._args[i0],val)
			if(x0==0) :
				return i0
			if( i0==i or i0==i1 ) :
				break
			if(x0*x1>0) :
				i1=i0
				x1=x0
			elif(x0*x>0) :
				i=i0
				x=x0
			else :
				raise(Exception('Unexpected situation in _search'))
		#linear search
		i=i0
		x=cmp(self._args[i],val)
		if(x==0) :
			return i
		step=(-1)*x
		while(True) :
			i+=step
			if(i<0 or i>=self.len()):
				return -1
			x=cmp(self._args[i],val)
			if(x==0) :
				return i
	def __call__(self,val) :
		i=self._search(val)
		if(i==-1) :
			return 0
		return self._args[i].poly()(val)
	def segment(self,piece) :
		i=self._search(piece.a())
		if(i==-1) :
			return PiecewisePoly()
		r=[]
		for j in range(i,self.len()) :
			a=self._args[j]
			p=a.piece().intersect(piece)
			if( p.isEmpty() ) :	
				break
			else:
				r.append((a.poly(),p))
		return PiecewisePoly(r)
	def isEmpty(self) :
		return self.len()==0
	def __add__(self,poly) :
		if( not type(poly)==type(self) ) :
			raise(Exception("Wrong type"))
		points=[]
		for x in self._args :
			p=x.piece()
			points.append(p.a())
			points.append(p.b())
		for x in poly._args :
			p=x.piece()
			points.append(p.a())
			points.append(p.b())
		points.sort()
		r=[]
		for i in range(0,len(points)-1): 
			p=Piece(points[i],points[i+1])
			x=self.segment(p)
			y=poly.segment(p)
			if(x.len()>1 or y.len()>1) :
				raise(Exception("segment() must return len()==1 in this context"))
			if( x.isEmpty() and y.isEmpty() ) :
				continue
			elif( x.isEmpty() ) :
				r.append((y._args[0].poly(),p))
			elif( y.isEmpty() ) :
				r.append((x._args[0].poly(),p))
			else :
				r.append((x._args[0].poly()+y._args[0].poly(),p))
		return PiecewisePoly(r,cutTails=True)
	def __sub__(self,x) :
		return self.__add__(x*(-1))
	def __mul__(self,z) :
		if( _std.isOfNumericType(z) ) :
			r=[]
			for x in self._args :
				r.append((x.poly()*z,x.piece()))
			return PiecewisePoly(r,cutTails=True)
		if( not type(z)==type(self) ) :
			raise(Exception("Wrong type"))
		poly=z
		points=[]
		for x in self._args :
			p=x.piece()
			points.append(p.a())
			points.append(p.b())
		for x in poly._args :
			p=x.piece()
			points.append(p.a())
			points.append(p.b())
		points.sort()
		r=[]
		for i in range(0,len(points)-1): 
			p=Piece(points[i],points[i+1])
			x=self.segment(p)
			y=poly.segment(p)
			if(x.len()>1 or y.len()>1) :
				raise(Exception("segment() must return len()==1 in this context"))
			if( x.isEmpty() or y.isEmpty() ) :
				continue
			else :
				r.append((x._args[0].poly()*y._args[0].poly(),p))
		return PiecewisePoly(r,cutTails=True)
	def integral(self,piece=Piece()) :
		if( piece.isEmpty() ) :
			sum=0
			for x in self._args :
				sum+=x.poly().integral(x.piece())
			return sum
		I=self.support().intersect(piece)
		if(I.isEmpty()) :
			return 0
		iA=self._search(I.a())
		iB=self._search(I.b())
		if( iB==-1 ) :
			iB=self.len()-1
		sum=0
		for i in range(iA,iB+1) :
			x=self._args[i]
			sum+=x.poly().integral(x.piece().intersect(piece))
		return sum
		
class Abs(object) :
	def __init__(self,func) :
		self._func=func
	def __call__(self,val) :
		return _n.abs(self._func(val))
	def support(self) :
		return self._func.support()

class Sqrt(object) :
	def __init__(self,func) :
		self._func=func
	def __call__(self,val) :
		return _n.sqrt(self._func(val))
	def support(self) :
		return self._func.support()
		
def Pow(func,p) :
	if( ( not type(p)==type(int()) ) or p<0 ) :
		raise(Exception('non negative int expected in Pow'))
	if( p==0 ) :
		spt=func.support()
		return PiecewisePoly([(Poly([1]),Piece(spt.a(),spt.b()))])
	r=func
	for i in range(0,p-1) :
		r*=func
	return r

_CBuffer={}
def C(p,i) :  #binomial
	try :
		d=_Cbuffer[p]
	except :
		_CBuffer[p]={}
		d=_CBuffer[p]
	try :
		return _CBuffer[p][i]
	except :
		pass
	r=_d.Decimal(1)
	for j in range(1,p+1) :
		r*=(_d.Decimal(1)/j)*(i-p+j)
	_CBuffer[p][i]=r
	return r

def T( x,k ) :  #Transport
	if( k==0 ) :
		return x
	if( _std.isOfNumericType(x) ) :
		return x+k
	if( type(x)==type(()) and len(x)==2 ) :
		return (T(x[0],k),T(x[1],k))
	if( type(x)==type(Piece()) ) :
		return Piece(T(x.a(),k),T(x.b(),k))
	if( type(x)==type(PiecewisePoly()) ) :
		return PiecewisePoly(T(x.structure(),k))
	if( type(x)==type([]) ) :
		return [T(xx,k) for xx in x]
	if( type(x)==type(Poly()) ) :
		a=x.coeffs()
		b=[]
		k=ToDecimal(k)
		for p in range(0,len(a)) :
			sum=0
			for i in range(p,len(a)) :
				sum+=a[i]*pow(-1,i-p)*C(p,i)*pow(k,i-p)
			b.append(sum)
		return Poly(b)
	raise(Exception("Unexpected case in T"))
	
def S( x,d ) :   #Scale
	if( d==0 ) :
		return x
	if( _std.isOfNumericType(x) ) :
		return x*pow(2,-d)
	if( type(x)==type(()) and len(x)==2 ) :
		return (S(x[0],d),S(x[1],d))
	if( type(x)==type(Piece()) ) :
		return Piece(S(x.a(),d),S(x.b(),d))
	if( type(x)==type(PiecewisePoly()) ) :
		return PiecewisePoly(S(x.structure(),d))
	if( type(x)==type([]) ) :
		return [S(xx,d) for xx in x]
	if( type(x)==type(Poly()) ) :
		a=x.coeffs()
		b=[]
		d=ToDecimal(d)
		for i in range(0,len(a)) :
			b.append(pow(_d.Decimal(2),d*(_d.Decimal(1)/2+i))*a[i])
		return Poly(b)
	raise(Exception("Unexpected case in S"))
	
def Op( func, d, k ) :
	return S(T(func,k),d)
	
def _convPow(x,p) :
	if( x==0 and p==0 ) :
		return _d.Decimal(1)
	return pow(x,p)

def ConvOfPoly2( f, g, alpha1, beta1, alpha2, beta2 ) :
	a=f.coeffs()
	b=g.coeffs()
	K=len(a)-1
	P=len(b)-1
	alpha1=ToDecimal(alpha1)
	alpha2=ToDecimal(alpha2)
	beta1=ToDecimal(beta1)
	beta2=ToDecimal(beta2)
	c=[]
	for r in range(0,K+P+2) :
		sum=0
		for m in range(0,min(r,P)+1) :
			for p in range(max(m,r-K-1),P+1) :
				for k in range(max(0,r-p-1),K+1) :
					q=r-m
					kpm=k+p-m+1
					sum+=a[k]*b[p] \
						*C(m,p)*pow(-1,p-m)/kpm*C(q,kpm) \
							*(_convPow(beta2,kpm-q)*_convPow(alpha2,q) \
								-_convPow(beta1,kpm-q)*_convPow(alpha1,q) \
							)
		c.append(sum)
	return Poly(c)

def ConvOfPoly1( a, A, b, B ) :
	a1=A.a()
	a2=A.b()
	b1=-B.b()
	b2=-B.a()
	r1=PiecewisePoly()
	if( max(a2-b2,a1-b1)<a2-b1 ) :
		i=ConvOfPoly2(f=a,g=b,alpha1=1,beta1=b1,alpha2=0,beta2=a2)
		r1=PiecewisePoly([(i,Piece(max(a2-b2,a1-b1),a2-b1))])
	r2=PiecewisePoly()
	if( a2-b2<a1-b1 ) :
		i=ConvOfPoly2(f=a,g=b,alpha1=0,beta1=a1,alpha2=0,beta2=a2)
		r2=PiecewisePoly([(i,Piece(a2-b2,a1-b1))])
	r3=PiecewisePoly()
	if( a1-b1<a2-b2 ) :
		i=ConfOfPoly2(f=a,g=b,alpha1=1,beta1=b1,alpha2=1,beta2=b2)
		r3=PiecewisePoly([(i,Piece(a1-b1,a2-b2))])
	r4=PiecewisePoly()
	if( a1-b2<min(a2-b2,a1-b1) ) :
		i=ConvOfPoly2(f=a,g=b,alpha1=0,beta1=a1,alpha2=1,beta2=b2)
		r4=PiecewisePoly([(i,Piece(a1-b2,min(a2-b2,a1-b1)))])
	return r1+r2+r3+r4

def Convolution(p1,p2) :
	r=PiecewisePoly()
	for (a,A) in p1.structure() :
		for (b,B) in p2.structure() :
			r=r+ConvOfPoly1(a,A,b,B)
	return r
	
def D(p) :
	if( isinstance(p,PiecewisePoly) ) :
		L=[]
		for pp in p._args :
			p1=D(pp.poly())
			if( p1.isZero() ) :
				continue
			else :
				L.append((p1,pp.piece()))
		return PiecewisePoly(L)
	if( isinstance(p,Poly) ) :
		cc=[]
		c=p.coeffs()
		for i in range(1,len(c)) :
			x=c[i]
			cc.append(i*x)
		return Poly(cc)
	raise(Exception('Unexpected argument'))

	