execfile('common.py')


def doApproxTest(phi,Phi,integrPiece,interpPiece,target,results={}) :
	"""
	The functions checks polynomial approximation properties of scaling function.
	phi is array of basis functions
	Phi is array of dual functions
	integrPiece is the interval of integration when calculating Fourier coefficients
	interpPiece is the interval of integration when calculating difference
	target is targets of approximation
	results is initial dictionary, optional
	The results of verification are packed into a dictionary.
	"""
	results['array of basis functions, phi']=phi
	results['array of dual functions, Phi']=Phi
	results['the interval of integration, integrPiece']=integrPiece
	results['the interval where the approximation is tested, interpPiece']=interpPiece
	results['target of approximation']=target
	import time
	results['time of execution, time.localTime()']=time.localtime()
	tests={}
	results['approximating target_j for variety of j']=tests
	for j in range(0,len(target)) :
		print "j="+str(j)
		rr={}
		rrr={}
		rr['scalar products <Phi_i,target_j> for running i']=rrr
		if( not len(phi)==len(Phi) ) :
			raise(Exception("phi and Phi must have the same length"))
		f=poly.PiecewisePoly()
		for i in range(0,len(phi)) :
			c=(Phi[i]*target[j]).integral(integrPiece)
			rrr[i]=c
			print c
			f=f+phi[i]*c
		L2=numpy.sqrt(((target[j]-f)*(target[j]-f)).integral(interpPiece))
		print "L2="+str(L2)
		rr['the approximating function, f=Sum_i <Phi_i,target_j> phi_i']=f
		rr['the L2 norm of difference target_j-f on interpPiece']=L2
		tests[j]=rr
	return results

def dist(f1,f2,I=poly.Piece(0,1)) :
	d=f1-f2
	d=d*d
	return numpy.sqrt(d.integral(I))
	
def doApproxTest2(phi,piece,target,results={}) :
	"""
	The functions checks polynomial approximation properties of scaling function using
	L2 projection technique.
	phi is array of basis functions
	piece is the interval of integration when calculating L2 projection.
	target is targets of approximation
	results is initial dictionary, optional
	The results of verification are packed into a dictionary.
	"""
	import interval
	results['array of basis functions, phi']=phi
	results['the interval of integration']=piece
	results['target of approximation']=target
	import time
	results['time of execution, time.localTime()']=time.localtime()
	tests={}
	results['approximating target_j for variety of j']=tests
	for j in range(0,len(target)) :
		print "j="+str(j)
		rr={}
		f=interval.Projection(target[j],phi,piece)
		L2=dist(f,target[j],piece)
		print 'L2='+str(L2)
		rr['target']=target[j]
		rr['the approximating function']=f
		rr['the L2 norm of difference target_j-f on interpPiece']=L2
		tests[j]=rr
	return results
	
	
def doLineApproxTest(phi,Phi,integrPiece,interpPiece,maxPwr) :
	"""
	Test that shifts of real line based scaling function would approximate
	powers of x on an interval.
	phi,Phi - scaling functions of dual GMRAs, poly.PiecewisePoly.
	integrPiece - the interval used for calculation of scalar products.
	interpPiece - the interval where the approximation is tested, poly.Piece.
	maxPwr - maximal power x tested.
	"""
	spt=phi.support()
	Spt=Phi.support()
	print "spt="+str(spt)
	print "Spt="+str(Spt)
	#First, we determine shifting range such that the shifted scaling 
	#function have a non-empty intersection of support with
	#the interpPiece.
	k=0
	while(True) :
		if( poly.Op(spt,0,k).intersect(interpPiece).len()!=0 ) :
			k0=k
			break
		if( poly.Op(spt,0,-k).intersect(interpPiece).len()!=0 ) :
			k0=-k
			break
		k=k+1
	k=k0
	while(True) :
		if( poly.Op(spt,0,k).intersect(interpPiece).len()==0 ) :
			kA=k+1
			break
		k=k-1
	k=k0
	while(True) :
		if( poly.Op(spt,0,k).intersect(interpPiece).len()==0 ) :
			kB=k-1
			break
		k=k+1
	#so now for any k in [kA,kB] spt phi_k intersects interpPiece.
	print "kA,kB" +str(kA)+","+str(kB)
	#make arguments
	phis=[poly.Op(phi,0,k) for k in range(kA,kB+1)]
	Phis=[poly.Op(Phi,0,k) for k in range(kA,kB+1)]
	f=poly.PiecewisePoly([(poly.Poly([0,1]),integrPiece)])
	target=[poly.Pow(f,k) for k in range(0,maxPwr+1)]
	results=doApproxTest(phis,Phis,integrPiece,interpPiece,target)
	return {'test that real line based scaling functions approximate powers of x' : results }
	
def doIntervalApproxTest(phiGen) :
	phis=[phiGen.xsid(k) for k in phiGen._kk]
	Phis=[phiGen.Xsid(k) for k in phiGen._KK]
	I01=poly.Piece(0,1)
	f=poly.PiecewisePoly([(poly.Poly([0,1]),I01)])
	target=[poly.Pow(f,k) for k in range(0,phiGen._n)]
	results={}
	results['direct']=doApproxTest(phis,Phis,I01,I01,target)
	results['reverse']=doApproxTest(Phis,phis,I01,I01,target)
	return results
	
def doIntervalApproxTest2(phiGen) :
	phis=[phiGen.xsid(k) for k in phiGen._kk]
	I01=poly.Piece(0,1)
	f=poly.PiecewisePoly([(poly.Poly([0,1]),I01)])
	target=[poly.Pow(f,k) for k in range(0,phiGen._n)]
	results={}
	results['direct']=doApproxTest2(phis,I01,target)
	return results

	
def doSubspaceApproximationTest( psiGen ) :
	"""
		The function tests that V_d+W_d approximates V_{d+1}
	"""
	import numpy
	results={}
	results['psiGen']=psiGen
	r={}
	results['xsi approximation L2 differences']=r
	R={}
	results['Xsi approximation L2 differences']=R
	phiGen1=psiGen._dGen
	phiGen2=psiGen._dp1Gen
	maxDiff=0
	loc=-100
	MaxDiff=0
	Loc=-100
	I=poly.Piece(0,1)
	for k in phiGen2._kk :
		xsi=phiGen2.xsid(k)
		xsi2=poly.PiecewisePoly()
		Xsi=phiGen2.Xsid(k)
		Xsi2=poly.PiecewisePoly()
		for k1 in phiGen1._kk :
			xsi3=phiGen1.xsid(k1)
			Xsi3=phiGen1.Xsid(k1)
			if( Xsi3.support().intersect(xsi.support()).len()>0 ) :
				c=(xsi*Xsi3).integral(I)
				xsi2+=xsi3*c
			if( xsi3.support().intersect(Xsi.support()).len()>0 ) :
				c=(Xsi*xsi3).integral(I)
				Xsi2+=Xsi3*c
		for k2 in phiGen1._kk :
			chi=psiGen.chid(k2)
			Chi=psiGen.Chid(k2)
			if( Chi.support().intersect(xsi.support()).len()>0 ) :
				c=(xsi*Chi).integral(I)
				xsi2+=chi*c
			if( chi.support().intersect(Xsi.support()).len()>0 ) :
				c=(Xsi*chi).integral(I)
				Xsi2+=Chi*c
		df=xsi-xsi2
		L2diff=numpy.sqrt((df*df).integral(I))
		print 'k='+str(k)+' L2 difference='+str(L2diff)
		r[k]=L2diff
		if( L2diff>maxDiff ) :
			maxDiff=L2diff
			loc=k			
		df=Xsi-Xsi2
		L2diff=numpy.sqrt((df*df).integral(I))
		print 'k='+str(k)+' L2 difference='+str(L2diff)
		R[k]=L2diff
		if( L2diff>MaxDiff ) :
			MaxDiff=L2diff
			Loc=k			
	r['maxDiff']=maxDiff
	r['location of max diff']=loc
	R['maxDiff']=MaxDiff
	R['location of max diff']=Loc
	return results
			