import poly
import numpy
import scipy.linalg as la
import time
import decimal


class PsiGenerator(object) :
	"""
	Source of psi/Psi-basis on [0,1].
	"""
	def __init__(self,dGenerator,dp1Generator,psi) :
		self._dGen=dGenerator
		self._dp1Gen=dp1Generator
		self._psi=psi
		if( not self._dGen._n==self._dGen._N ) :
			raise(Exception('N are different'))
		pg0=self._dGen
		pg1=self._dp1Gen
		dimV0=2*pg0._n+len(pg0._kI)
		dimV1=2*pg1._n+len(pg1._kI)
		dDim=dimV1-dimV0
		d2Boundary=dDim-len(pg0._kI)
		if( d2Boundary % 2 != 0 ) :
			raise(Exception('d2Boundary is odd'))
		dBoundary=d2Boundary/2
		NtoDiscard=pg0._n-dBoundary
		if( NtoDiscard<0 ) :
			raise(Exception('NtoDiscard is negative'))
		self._NtoDiscard=NtoDiscard
	def calculate(self) :
		pg0=self._dGen
		pg1=self._dp1Gen
		psi=self._psi
		phi=pg0._phi
		kk0=pg0._kk
		kk1=pg1._kk
		kI0=pg0._kI
		kI1=pg1._kI
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		kL1=pg1._kI[0]-1
		kR1=pg1._kI[-1]+1
		d0=pg0._d
		d1=pg1._d
		I=poly.Piece(0,1)
		n=pg0._n
		xsi0=[]
		for k in kk0 :
			xsi0.append(pg0.xsid(k))
		xsi1=[]
		for k in kk1 :
			xsi1.append(pg1.xsid(k))
		print 'xsi0,xsi0 constructed'
		grammInv=GrammInv(f=xsi0,I=I)
		zi=[]
		print 'calculating zi'
		for k in range(0,n) :
			u=xsi1[k]
			proj=Projection(u=u,f=xsi0,grammInv=grammInv,I=I)
			df=u-proj
			print 'L2 of proj='+str(l2Norm(proj,I))
			L2=l2Norm(df,I)
			print 'L2 of proj df='+str(L2)
			zi.append(df*(decimal.Decimal(1)/L2))
		print 'zi,L constructed'
		for k in kI0 :
			print k
			zi.append(poly.Op(psi,d0,k))
		print 'zi,I constructed'
		for k in range(0,n) :
			u=xsi1[len(xsi1)-n+k]
			proj=Projection(u=u,f=xsi0,grammInv=grammInv,I=I)
			df=u-proj
			print 'L2 of proj='+str(l2Norm(proj,I))
			L2=l2Norm(df,I)
			print 'L2 of proj df='+str(L2)
			zi.append(df*(decimal.Decimal(1)/L2))
		print 'zi,R constructed'
		print 'removing proj_I'
		ziI=zi[n:(len(zi)-n)]
		grammInv=GrammInv(f=ziI,I=I)
		chiL=[]
		for k in range(0,n) :
			chiL.append(zi[k]-Projection(u=zi[k],f=ziI,grammInv=grammInv,I=I))
		chiR=[]
		for k in range(len(zi)-n,len(zi)) :
			chiR.append(zi[k]-Projection(u=zi[k],f=ziI,grammInv=grammInv,I=I))
		print 'chiL,chiR done'
		print 'calculating chi'
		NtoDiscard=self._NtoDiscard
		print 'NtoDiscard='+str(NtoDiscard)
		chiL.reverse()
		chiL2=GrammSchmidt(f=chiL,I=I)
		chiL2.reverse()
		chiR2=GrammSchmidt(f=chiR,I=I)
		if( NtoDiscard==0 ) :
			chi=chiL2+zi[n:(len(zi)-n)]+chiR2
		else :
			chi=chiL2[NtoDiscard:]+zi[n:(len(zi)-n)]+chiR2[:-NtoDiscard]
		print 'chi done'
		self._chid={}
		shft=kk0[NtoDiscard]
		for i in range(0,len(chi)) :
			self._chid[shft+i]=chi[i]
		for k in range(kL0+1,kR0) :
			del self._chid[k]			
	def chid(self,k) :
		pg0=self._dGen
		pg1=self._dp1Gen
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		n=pg0._n
		if( k in range(kL0+1,kR0) ) :
			return poly.Op(self._dGen._phi,pg0._d,k)
		if( kL0-n<k and k<=kL0 ) :
			return self._chid[k]
		if( kR0<=k and k<kR0+n ) :
			return self._chid[k]
		raise(Exception('wrong index k'))
	def makeDp1(self) :
		pg0=self._dGen
		pg1=self._dp1Gen
		psi=self._psi
		phi=pg0._phi
		kk0=pg0._kk
		kk1=pg1._kk
		kI0=pg0._kI
		kI1=pg1._kI
		kL0=pg0._kI[0]-1
		kR0=pg1._kI[-1]+1
		kL1=pg0._kI[0]-1
		kR1=pg1._kI[-1]+1
		d0=pg0._d
		d1=pg1._d
		n=pg0._n
		r=PsiGenerator(
			dGenerator=self._dp1Gen,
			dp1Generator=self._dp1Gen.makeDp1(),
			psi=self._psi
			)
		if( self._NtoDiscard!=r._NtoDiscard ) :
			raise(Exception('NtoDiscard mismatch'))
		r._chid={}
		#find k_d on left
		spt0=poly.Op(psi.support(),d0,kL0)
		spt1=poly.Op(phi.support(),d1,kL1)
		diff=10
		kD=-1000
		for k in range(0,len(kI1)+2*n+1) :
			sptA=poly.Op(spt0,1,k)
			d=numpy.abs(sptA.b()-spt1.b())
			if( d<diff ) :
				diff=d
				kD=k
			sptA=poly.Op(spt0,1,-k)
			d=numpy.abs(sptA.b()-spt1.b())
			if( d<diff ) :
				diff=d
				kD=-k
		kL=kD
		print 'kL='+str(kL)
		#find k_d on right
		spt0=poly.Op(phi.support(),d0,kR0)
		spt1=poly.Op(phi.support(),d1,kR1)
		diff=10
		kD=-1000
		for k in range(0,len(kI1)+2*n+1) :
			sptA=poly.Op(spt0,1,k)
			d=numpy.abs(sptA.b()-spt1.b())
			if( d<diff ) :
				diff=d
				kD=k
			sptA=poly.Op(spt0,1,-k)
			d=numpy.abs(sptA.b()-spt1.b())
			if( d<diff ) :
				diff=d
				kD=-k
		kR=kD	
		print 'kR='+str(kR)
		for i in range(0,n-1) :
			print i
			r._chid[kL1-i]=poly.Op(self.chid(kL0-i),1,kL)
			r._chid[kR1+i]=poly.Op(self.chid(kL0+i),1,kR)
		return r
		
def scalingPsiW12(psiGen) :
	print 'scalingPsi'
	pg0=psiGen._dGen
	pg1=psiGen._dp1Gen
	psi=psiGen._psi
	phi=pg0._phi
	kk0=pg0._kk
	kk1=pg1._kk
	kI0=pg0._kI
	kI1=pg1._kI
	kL0=pg0._kI[0]-1
	kR0=pg0._kI[-1]+1
	kL1=pg1._kI[0]-1
	kR1=pg1._kI[-1]+1
	d0=pg0._d
	d1=pg1._d
	I=poly.Piece(0,1)
	n=pg0._n
	print 'building xsi0'
	xsi0=[]
	for k in kk0 :
		xsi0.append(pg0.xsid(k))
	print 'building xsi1'
	xsi1=[]
	for k in kk1 :
		xsi1.append(pg1.xsid(k))
	print 'building chi'
	chi=[]
	kChi=psiGen._chid.keys()
	kChi.sort()
	for k in kChi :
		chi.append(psiGen.chid(k))
	grammInvW12=GrammInvW12(f=xsi1,I=I)
	scale1=[]
	for u in xsi0 :
		proj=ProjectionW12(u=u,f=xsi1,grammInvW12=grammInvW12,I=I)
		df=u-proj
		W12=w12Norm(df,I)
		print 'W12 df='+str(W12)
		scale1.append(W12)
	scale2=[]
	for u in chi :
		proj=ProjectionW12(u=u,f=xsi1,grammInvW12=grammInvW12,I=I)
		df=u-proj
		W12=w12Norm(df,I)
		print 'W12 df='+str(W12)
		scale2.append(W12)
	r={}
	r['d0']=d0
	r['d1']=d1
	r['xsi0 in xsi1']=scale1
	r['chi in xsi1']=scale2
	return r
	
def scalingPsiL2(psiGen) :
	print 'scalingPsi'
	pg0=psiGen._dGen
	pg1=psiGen._dp1Gen
	psi=psiGen._psi
	phi=pg0._phi
	kk0=pg0._kk
	kk1=pg1._kk
	kI0=pg0._kI
	kI1=pg1._kI
	kL0=pg0._kI[0]-1
	kR0=pg0._kI[-1]+1
	kL1=pg1._kI[0]-1
	kR1=pg1._kI[-1]+1
	d0=pg0._d
	d1=pg1._d
	I=poly.Piece(0,1)
	n=pg0._n
	print 'building xsi0'
	xsi0=[]
	for k in kk0 :
		xsi0.append(pg0.xsid(k))
	print 'building xsi1'
	xsi1=[]
	for k in kk1 :
		xsi1.append(pg1.xsid(k))
	print 'building chi'
	chi=[]
	kChi=psiGen._chid.keys()
	kChi.sort()
	for k in kChi :
		chi.append(psiGen.chid(k))
	grammInv=GrammInv(f=xsi1,I=I)
	scale1=[]
	for u in xsi0 :
		proj=Projection(u=u,f=xsi1,grammInv=grammInv,I=I)
		df=u-proj
		L2=l2Norm(df,I)
		print 'L2 df='+str(L2)
		scale1.append(L2)
	scale2=[]
	for u in chi :
		proj=Projection(u=u,f=xsi1,grammInv=grammInv,I=I)
		df=u-proj
		L2=l2Norm(df,I)
		print 'L2 df='+str(L2)
		scale2.append(L2)
	r={}
	r['d0']=d0
	r['d1']=d1
	r['xsi0 in xsi1']=scale1
	r['chi in xsi1']=scale2
	return r
	
