import poly
import numpy
import scipy.linalg as la
import time
import decimal


def Scalar(f1,f2) :
	"L2 scalar product of two PiecewisePoly-nomials"
	if( f1.support().intersect(f2.support()).len()==0 ) :
		return decimal.Decimal(0)
	else :
		return (f1*f2).integral()
def scalar(f1,f2,I) :
	"L2 scalar product of two PiecewisePoly-nomials"
	if( f1.support().intersect(f2.support()).intersect(I).len()==0 ) :
		return decimal.Decimal(0)
	else :
		return (f1*f2).integral(I)
def Dist(f1,f2) :
	d=f1-f2
	return numpy.sqrt(Scalar(d,d))
def dist(f1,f2,I) :
	d=f1-f2
	return numpy.sqrt(scalar(d,d,I))
def L2Norm(f) :
	return numpy.sqrt(Scalar(f,f))
def l2Norm(f,I) :
	return numpy.sqrt(scalar(f,f,I))

def scalarW12(f1,f2,I) :
	"W12 scalar product of two PiecewisePoly-nomials"
	if( f1.support().intersect(f2.support()).intersect(I).len()==0 ) :
		return decimal.Decimal(0)
	else :
		return (f1*f2).integral(I)+(poly.D(f1)*poly.D(f2)).integral(I)
def distW12(f1,f2,I) :
	d=f1-f2
	return numpy.sqrt(scalarW12(d,d,I))
def w12Norm(f,I) :
	return numpy.sqrt(scalarW12(f,f,I))


def GrammSchmidt(f,I,kernel=None) :
	r=[]
	N=len(f)
	for k in range(0,N) :
		proj=poly.PiecewisePoly()
		for p in range(0,k) :
			prd=f[k]*r[p]
			if( not kernel is None ) :
				prd*=kernel
			c=prd.integral(I)
			proj+=r[p]*c
		rr=f[k]-proj
		sq=rr*rr
		if( not kernel is None ) :
			sq*=kernel
		n=numpy.sqrt(sq.integral(I))
		n=decimal.Decimal(1)/n
		r.append(rr*n)
	return r
def GrammMatrix(f,I) :
	N=len(f)
	a=[]
	for i in range(0,N) :
		print 'GrammMatrix, (i,N)='+str((i,N))
		aa=[]
		a.append(aa)
		for j in range(0,N) :
			aa.append(scalar(f[i],f[j],I))
	return numpy.matrix(a)
def condN(A) :
	(lam,Q)=la.eig(A)
	return max(lam)/min(lam)
def GrammInv(f,I) :
	m=GrammMatrix(f,I)
	print 'GrammInv, condN='+str(condN(m))
	return poly.ToDecimal(m.I)
def ScalarColumn(u,f,I) :
	a=[]
	for x in f :
		a.append([scalar(u,x,I)])
	return numpy.matrix(a)
def Projection(u,f,grammInv,I) :
	print 'calculating projection'
	b=ScalarColumn(u=u,f=f,I=I)
	c=grammInv*b
	proj=poly.PiecewisePoly()
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	print 'projection done'
	return proj
def Projection2(u,f,I) :
	print 'calculating projection'
	grammInv=GrammInv(f=f,I=I)
	b=ScalarColumn(u=u,f=f,I=I)
	c=grammInv*b
	proj=poly.PiecewisePoly()
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)
def Projection3(u,f,grammInv,I) :
	print 'calculating projection'
	b=ScalarColumn(u=u,f=f,I=I)
	c=grammInv*b
	proj=poly.PiecewisePoly()
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)	


def GrammMatrixW12(f,I) :
	a=[]
	N=len(f)
	k=0
	for x in f :
		print 'GrammMatrixW12, k='+str(k)+' of '+str(N)
		k+=1
		aa=[]
		a.append(aa)
		for y in f :
			aa.append(scalarW12(x,y,I))
	return numpy.matrix(a)
def GrammInvW12(f,I) :
	m=GrammMatrixW12(f,I)
	print 'GrammInvW12, condN='+str(condN(m))
	return poly.ToDecimal(m.I)
def ScalarColumnW12(u,f,I) :
	a=[]
	for x in f :
		a.append([scalarW12(u,x,I)])
	return numpy.matrix(a)
def ProjectionW12(u,f,grammInvW12,I) :
	print 'calculating projection w12'
	b=ScalarColumnW12(u=u,f=f,I=I)
	c=grammInvW12*b
	proj=poly.PiecewisePoly()
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	print 'projection done'
	return proj
def GrammSchmidtW12(f,I) :
	print 'GrammSchmidtW12'
	r=[]
	for k in range(0,len(f)) :
		proj=poly.PiecewisePoly()
		for p in range(0,k) :
			c=scalarW12(f[k],r[p],I)
			proj+=r[p]*c
		rr=f[k]-proj
		W12=w12Norm(rr,I)
		print 'GrammSchmidt, W12='+str(W12)
		r.append(rr*(decimal.Decimal(1)/W12))
	print 'GrammSchmidtW12 done'
	return r
	
