execfile('common.py')

def centeredIndex(h) :
	"Calculate the middle index of a scaling filter with zeros."
	for k in range(0,len(h)) :
		if( h[k]!=0 ) :
			break
	kA=k
	for k in range(len(h)-1,-1,-1) :
		if( h[k]!=0 ) :
			break
	kB=k
	return int(numpy.floor((kA+kB)*0.5))
def shiftedDict(h,ci) :
	"Shift indexes of a dictionaty h by ci."
	r={}
	for k in range(0,len(h)) :
		r[k-ci]=h[k]
	return r

def A(eta,h) :
	"The operator A, see notes"
	sum=poly.PiecewisePoly()
	for k in h :
		sum=sum+poly.Op(eta,1,k)*h[k]
	return sum
	
def waveletFilter(h) :
	"Transform scaling filter into wavelet filter (g in Notes)."
	g={}
	for k in h :
		g[1-k]=pow(-1,1-k)*h[k]
	return g
def wavelet(phi,g) :
	"Calculate wavelet from scaling function phi and wavelet filter g."
	r=poly.PiecewisePoly()
	for k in g :
		r=r+poly.Op(phi,1,k)*g[k]
	return r

	
class FourFunctions(object) :
	"""
	Calculates the functions phi,Phi,psi,Psi for a GMRA given by scaling filters h,H.
	"""
	keyFunctions='Approximating functions for every step'
	keyL2Diff='L2-norm difference of consecutive approximations' 
	keyCascade='Results of cascade procedure'
	keyWavelets='Wavelet approximation for every step of cascade procedure'
	keyScalingFilter='Scaling filter'
	keyWaveletFilter='Wavelet filter'
	keyFirst='first'
	keySecond='second'
	doGraphing=False
	def __init__(self,h,H,eta0,steps) :
		"""
			h,H are scaling filters (in the Notes, h and h tilde).
			eta0 is the initial shape.
			steps is the number of step of the scaling procedure.
		"""
		self.h=shiftedDict(h,centeredIndex(h))
		self.H=shiftedDict(H,centeredIndex(H))
		self.steps=steps
		self.eta0=eta0
	def cascade(self,h) :
		"""
			Cascade procedure for calculation of scaling function.
			h is the scaling filter.
		"""
		keyFuncs=self.keyFunctions
		keyL2=self.keyL2Diff
		results={}
		results[keyFuncs]={}
		results[keyL2]={}
		results[self.keyScalingFilter]=h
		z1=self.eta0
		self.graph(z1)
		results[keyFuncs][0]=z1
		for i in range(1,self.steps+1) :
			print "step "+str(i)
			t1=time.clock()
			z2=A(z1,h)
			results[keyFuncs][i]=z2
			t2=time.clock()
			L2=numpy.sqrt(abs(poly.Pow(z1-z2,2).integral()))
			results[keyL2][i]=L2
			self.graph(z2)
			print "L2="+str(L2)
			print "time="+str(t2-t1)
			z1=z2
		return results
	def graph(self,func) :
		if( self.doGraphing ) :
			plotPoly(func)
	def wavelets(self,g,cascade) :
		"""
		Calculate a wavelet for every step of the cascade procedure.
		g - wavelet filter,
		cascade - dictionary of results of cascade procedure. 
		Such dictionary is produced by the member function 'cascade'.
		"""
		results={}
		keyFuncs=self.keyFunctions
		keyL2=self.keyL2Diff
		results[self.keyWaveletFilter]=g
		results[keyFuncs]={}
		results[keyL2]={}
		prev=None
		for k in cascade[keyFuncs] :
			print "k="+str(k)
			t1=time.clock()
			psi=wavelet(cascade[keyFuncs][k],g)
			self.graph(psi)
			if( not prev is None ) :
				L2=numpy.sqrt(abs(poly.Pow(prev-psi,2).integral()))
				print "L2="+str(L2)
				results[keyL2][k]=L2
			prev=psi
			results[keyFuncs][k]=psi
			t2=time.clock()
			print "time="+str(t2-t1)
		return results
	def run(self) :
		"""
		Run cascade procedure for both scaling filters
		and calculate wavelets.
		The results are packed into a dictionary.
		"""
		h=self.h
		H=self.H
		results={}
		results[self.keyFirst]={}
		results[self.keySecond]={}
		cascade=self.cascade(h)
		Cascade=self.cascade(H)
		results[self.keyFirst][self.keyCascade]=cascade
		results[self.keySecond][self.keyCascade]=Cascade
		g=waveletFilter(H)
		G=waveletFilter(h)
		wavelets=self.wavelets(g,cascade)
		Wavelets=self.wavelets(G,Cascade)
		results[self.keyFirst][self.keyWavelets]=wavelets
		results[self.keySecond][self.keyWavelets]=Wavelets		
		return results

def getphi(cascade) :
	return cascade[FourFunctions.keyFirst][FourFunctions.keyCascade][FourFunctions.keyFunctions]
def getPhi(cascade) :
	return cascade[FourFunctions.keySecond][FourFunctions.keyCascade][FourFunctions.keyFunctions]
def getpsi(cascade) :
	return cascade[FourFunctions.keyFirst][FourFunctions.keyWavelets][FourFunctions.keyFunctions]
def getPsi(cascade) :
	return cascade[FourFunctions.keySecond][FourFunctions.keyWavelets][FourFunctions.keyFunctions]
def geth(cascade) :
	return cascade[FourFunctions.keyFirst][FourFunctions.keyCascade][FourFunctions.keyScalingFilter]
def getH(cascade) :
	return cascade[FourFunctions.keySecond][FourFunctions.keyCascade][FourFunctions.keyScalingFilter]
		
