
I=poly2.Piece(0,0,1)
I2=poly2.Piece(0,-1,1)

def check(cond,message) :
	if( not cond ) :
		raise(Exception(message))

class BaseXsi(object) :
	"""
		Base class for phi-holders.
		Expects the following fields:
			d - scale exponent.
			n - number of vanishing moments.
			phi - R-scaling function.
			xsid - dictionary of boundary scaling functions.
			Kd - list of indexes.
			KdI - list of internal indexes.
			Kd1 - list of left indexes.
			Kd2 - list of right indexes. 
	"""
	def phi_(self,k) :
		return poly2.Op(tools.descriptionToPoly(self.phi),self.d+1,k).S(-1).T(-1)
	def xsi(self,k) :
		if k in self.xsid.keys() :
			return self.xsid[k]
		if k in self.KdI :
			return tools.polyToDescription(self.phi_(k))
		raise(Exception("Wrong k"))
	def xsi_(self,k) :
		if k in self.xsid.keys() :
			return tools.descriptionToPoly(self.xsid[k])
		if k in self.KdI :
			return self.phi_(k)
		raise(Exception("Wrong k"))	
	def __str__(self) :
		s=self.__class__.__name__+"(\n"
		s+="d="+str(self.d)+'\n'
		s+="n="+str(self.n)+'\n'
		s+="Kd="+str(self.Kd)+'\n'
		s+="KdI="+str(self.KdI)+'\n'
		s+="Kd1="+str(self.Kd1)+'\n'
		s+="Kd2="+str(self.Kd2)+'\n'
		s+=")\n"
		return s
	def copyBasics(self,source) :
		self.d=source.d
		self.n=source.n
		self.Kd=source.Kd
		self.KdI=source.KdI
		self.Kd1=source.Kd1
		self.Kd2=source.Kd2
		self.phi=source.phi
		self.xsid={}
	def sameLists(self,source) :
		check(self.d==source.d,"same lists failed")
		check(self.n==source.n,"same lists failed")
		check(self.Kd==source.Kd,"same lists failed")
		check(self.Kd1==source.Kd1,"same lists failed")
		check(self.Kd2==source.Kd2,"same lists failed")
		
		
class InitialXsi(BaseXsi) :
	"""
		Initial scaling functions adapted to [-1,1].
		Constructor converts from PhiGenerator object of the script basis2.py.
	"""
	def __init__(self,phiGen) :
		self.d=phiGen._d-1
		self.n=phiGen._n
		self.Kd=phiGen._kk
		self.KdI=phiGen._kI
		self.Kd1=[k for k in self.Kd if k<self.KdI[0]]
		self.Kd2=[k for k in self.Kd if k>self.KdI[-1]]
		self.phi=phiGen._phi
		self.xsid={}
		for k in phiGen._xsid.keys() :
			self.xsid[k]=tools.polyToDescription(phiGen.xsid_(k).S(-1).T(-1))
		

def CrN(u,crN) :
	for i in range(0,crN) :
		u=u.Cr()
	return u
	
def CrTestL2(xsi,crN) :
	I2=poly2.Piece(0,-1,1)
	r=[]
	for k in xsi.Kd :
		u1=xsi.xsi_(k)
		u2=CrN(u1,crN)
		L2=tools.l2Norm(u1-u2,I2)
		r.append(L2)
	return r
def CrTestL2_(xsi,crN) :
	I2=poly2.Piece(0,-1,1)
	r=[]
	for k in xsi._kk :
		u1=xsi.xsid_(k)
		u2=CrN(u1,crN)
		L2=tools.l2Norm(u1-u2,I2)
		r.append(L2)
	return r
def CrTestW12(xsi,crN) :
	I2=poly2.Piece(0,-1,1)
	r=[]
	for k in xsi.Kd :
		u1=xsi.xsi_(k)
		u2=CrN(u1,crN)
		L2=tools.w12Norm(u1-u2,I2)
		r.append(L2)
	return r
def CrTestW12_(xsi,crN) :
	I2=poly2.Piece(0,-1,1)
	r=[]
	for k in xsi._kk :
		u1=xsi.xsid_(k)
		u2=CrN(u1,crN)
		L2=tools.w12Norm(u1-u2,I2)
		r.append(L2)
	return r
	
class CrudeXsi(BaseXsi) :
	"""
		Class that performs crudification of scaling functions.
		Because of numerical issues revealed bu CrTestL2,CrTestL2_,... function tests,
		crudification is performed on the negative side and inverted to the
		positive side.
		The constructor takes the InititalXsi object and the depth of crudification.
	"""
	def __init__(self,xsi,crN) :
		self.copyBasics(xsi)
		self.phi=tools.polyToDescription(CrN(tools.descriptionToPoly(self.phi),crN))
		check(len(self.Kd1)==len(self.Kd2),"Kd1 vs Kd2 mismatch")
		for i in range(0,len(self.Kd1)) :
			k1=self.Kd1[i]
			k2=self.Kd2[-i-1]
			u=CrN(xsi.xsi_(k1),crN)
			self.xsid[k1]=tools.polyToDescription(u)
			self.xsid[k2]=tools.polyToDescription(u.inv())

def CompareXsi(xsi1,xsi2) :
	xsi1.sameLists(xsi2)
	r1=[]
	r2=[]
	max1=0
	max2=0
	for k in xsi1.Kd :
		df=xsi1.xsi_(k)-xsi2.xsi_(k)
		L2=tools.l2Norm(df,I2)
		W12=tools.w12Norm(df,I2)
		max1=max(max1,L2)
		max2=max(max2,W12)
		r1.append(L2)
		r2.append(W12)
	return (max1,max2,r1,r2)
def CompareXsi2(phiGen,xsi2) :
	r1=[]
	r2=[]
	max1=0
	max2=0
	for k in xsi2.Kd :
		df=phiGen.xsid_(k).S(-1).T(-1)-xsi2.xsi_(k)
		L2=tools.l2Norm(df,I2)
		W12=tools.w12Norm(df,I2)
		max1=max(max1,L2)
		max2=max(max2,W12)
		r1.append(L2)
		r2.append(W12)
	return (max1,max2,r1,r2)
			
class BaseChi(object) :
	"""
		Base class for psi-holders.
		Expects the following fields:
			d - scale exponent.
			n - number of vanishing moments.
			psi - R-wavelet.
			chid - dictionary of boundary wavelets.
			Kd - list of indexes.
			KdI - list of internal indexes.
			Kd1 - list of left indexes.
			Kd2 - list of right indexes. 
	"""
	def psi_(self,k) :
		return poly2.Op(tools.descriptionToPoly(self.psi),self.d+1,k).S(-1).T(-1)
	def chi(self,k) :
		if k in self.chid.keys() :
			return self.chid[k]
		if k in self.KdI :
			return tools.polyToDescription(self.psi_(k))
		raise(Exception("Wrong k"))
	def chi_(self,k) :
		if k in self.chid.keys() :
			return tools.descriptionToPoly(self.chid[k])
		if k in self.KdI :
			return self.psi_(k)
		raise(Exception("Wrong k"))	
	def __str__(self) :
		s=self.__class__.__name__+"(\n"
		s+="d="+str(self.d)+'\n'
		s+="n="+str(self.n)+'\n'
		s+="Kd="+str(self.Kd)+'\n'
		s+="KdI="+str(self.KdI)+'\n'
		s+="Kd1="+str(self.Kd1)+'\n'
		s+="Kd2="+str(self.Kd2)+'\n'
		s+=")\n"
		return s
	def copyBasics(self,source) :
		self.d=source.d
		self.n=source.n
		self.Kd=source.Kd
		self.KdI=source.KdI
		self.Kd1=source.Kd1
		self.Kd2=source.Kd2
		self.psi=source.psi
		self.chid={}
	def sameLists(self,source) :
		check(self.d==source.d,"same lists failed")
		check(self.n==source.n,"same lists failed")
		check(self.Kd==source.Kd,"same lists failed")
		check(self.Kd1==source.Kd1,"same lists failed")
		check(self.Kd2==source.Kd2,"same lists failed")

class InitialChi(BaseChi) :
	"""
		Initial wavelets adapted to [-1,1].
		Constructor converts from PsiGenerator object of the script basis2.py.
	"""
	def __init__(self,psiGen) :
		self.d=psiGen._dGen._d-1
		self.n=psiGen._dGen._n
		self.KdI=psiGen._dGen._kI
		self.Kd1=[k for k in psiGen._chid.keys() if k<self.KdI[0]]
		self.Kd2=[k for k in psiGen._chid.keys() if k>self.KdI[-1]]
		self.Kd=self.Kd1+self.KdI+self.Kd2
		self.psi=psiGen._psi
		self.chid={}
		for k in psiGen._chid.keys() :
			self.chid[k]=tools.polyToDescription(psiGen.chid_(k).S(-1).T(-1))

class CrudeChi(BaseChi) :
	"""
		Class that performs crudification of wavelets.
		The constructor takes the InititalChi object and the depth of crudification.
	"""
	def __init__(self,chi,crN) :
		self.copyBasics(chi)
		self.psi=tools.polyToDescription(CrN(tools.descriptionToPoly(self.psi),crN))
		check(len(self.Kd1)==len(self.Kd2),"Kd1 vs Kd2 mismatch")
		for i in range(0,len(self.Kd1)) :
			k1=self.Kd1[i]
			k2=self.Kd2[-i-1]
			u=CrN(chi.chi_(k1),crN)
			self.chid[k1]=tools.polyToDescription(u)
			self.chid[k2]=tools.polyToDescription(u.inv())

def CompareChi(chi1,chi2) :
	chi1.sameLists(chi2)
	r1=[]
	r2=[]
	max1=0
	max2=0
	for k in chi1.Kd :
		df=chi1.chi_(k)-chi2.chi_(k)
		L2=tools.l2Norm(df,I2)
		W12=tools.w12Norm(df,I2)
		max1=max(max1,L2)
		max2=max(max2,W12)
		r1.append(L2)
		r2.append(W12)
	return (max1,max2,r1,r2)
def CompareChi2(psiGen,chi2) :
	r1=[]
	r2=[]
	max1=0
	max2=0
	for k in chi2.Kd :
		df=psiGen.chid_(k).S(-1).T(-1)-chi2.chi_(k)
		L2=tools.l2Norm(df,I2)
		W12=tools.w12Norm(df,I2)
		max1=max(max1,L2)
		max2=max(max2,W12)
		r1.append(L2)
		r2.append(W12)
	return (max1,max2,r1,r2)
			
