execfile('common.py')

import gram
import ots.io as io

name='blackFinal.py'

version="A"
execfile('version_'+version+'.py')
basisFile='dirichlet3a_step'+str(steps)+'_d45_crN'+str(crN)+'_maxD'+str(maxD)+'_result'
gramFile='gram_version_'+version
gridFile='grid_version_'+version

try :
	multiBasis
except :
	multiBasis=load(basisFile)

def _check(cond,msg) :
	if( not cond ) :
		raise(Exception(msg))
		
def _m(a) :
	import numpy
	return numpy.matrix(a)

class BlackEquation(object) :
	def __init__(self,x0,t,T,sigma,precision) :
		self.x0=x0
		self.t=t
		self.T=T
		self.sigma=sigma
		self.precision=precision
		self.minimalNumberOfK=16
		self.minimalD=-7
	def calcInterval(self) :
		import scipy.stats
		import numpy
		N01Inv=scipy.stats.norm(0,1).ppf
		xsi0=abs(N01Inv(self.precision/2))
		sigma=self.sigma
		dt=self.T-self.t
		x1=numpy.exp(sigma*xsi0*numpy.sqrt(dt))
		x2=numpy.exp(-0.5*sigma*sigma*dt)
		x0=self.x0
		d0=self.minimalD
		while(True) :
			kA=int(numpy.floor(poly.pow2_(d0)*x0*x2/x1))
			kB=int(numpy.ceil(poly.pow2_(d0)*x0*x2*x1))
			kk=kB-kA+1
			if( kk>=self.minimalNumberOfK ) :
				break
			d0+=1
		self.int_d0=d0
		self.int_kA=kA
		self.int_kB=kB
	def oneTimeStep(self,tn,tnp1,Kn,Knp1,gram,cnp1) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		Gnp1=_m(gram.gram2(Kn,Knp1))
		G=_m(gram.gram(Kn))
		GI=G.I
		Q=_m(gram.bgram(Kn))
		U=0.5*sigma*sigma*GI*Q.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		h=(I-H)*D*GI*Gnp1*_m(cnp1).T
		for p in range(0,11) :
			H=H*H
			h=(I+H)*h
		basis_n=[gram.func(id) for id in Kn]
		basis_np1=[gram.func(id) for id in Knp1]
		r=poly.LZero(basis_n[0].support())
		for i in range(0,len(basis_n)) :
			r+=basis_n[i]*h[i,0]
		r0=poly.LZero(basis_np1[0].support())
		for i in range(0,len(basis_np1)) :
			r0+=basis_np1[i]*cnp1[i]
		return (h,r,r0,H)
	def oneTimeStepE(self,tn,tnp1,Kn,Knp1,gram,cnp1) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		Gnp1=_m(gram.gram2(Kn,Knp1))
		G=_m(gram.gram(Kn))
		GI=G.I
		Q=_m(gram.bgram(Kn))
		U=0.5*sigma*sigma*GI*Q.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		E=(I-H)*D
		for p in range(0,11) :
			H=H*H
			E=(I+H)*E
		b=GI*Gnp1*_m(cnp1).T
		h=E*b
		basis_n=[gram.func(id) for id in Kn]
		basis_np1=[gram.func(id) for id in Knp1]
		r=poly.LZero(basis_n[0].support())
		for i in range(0,len(basis_n)) :
			r+=basis_n[i]*h[i,0]
		r0=poly.LZero(basis_np1[0].support())
		for i in range(0,len(basis_np1)) :
			r0+=basis_np1[i]*cnp1[i]
		return (h,r,r0,E,H)
	def oneTimeStepEOnly(self,tn,tnp1,Kn,gram) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		G=_m(gram.gram(Kn))
		GI=G.I
		Q=_m(gram.bgram(Kn))
		U=0.5*sigma*sigma*GI*Q.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		E=(I-H)*D
		for p in range(0,11) :
			H=H*H
			E=(I+H)*E
		return (E,GI,H)
	def expTimeStep(self,tn,tnp1,Kn,Knp1,gram,cnp1,dd) :
		dt=tnp1-tn
		steps=pow(2,dd)
		ddt=dt*1.0/steps
		(E,GI,H)=self.oneTimeStepEOnly(tn=tnp1-ddt,tnp1=tnp1,Kn=Kn,gram=gram)
		for p in range(0,dd) :
			E=E*E
		Gnp1=_m(gram.gram2(Kn,Knp1))
		b=GI*Gnp1*_m(cnp1).T
		c=E*b
		return (c,E)
	def formFunc(self,c,grid,gram) :
		basis=[gram.func(id) for id in grid]
		r=poly.LZero(basis[0].support())
		for i in range(0,len(basis)) :
			r+=basis[i]*c[i,0]
		return r
	def formFunc2(self,c,grid,gram) :
		basis=[gram.func(id) for id in grid]
		r=poly.LZero(basis[0].support())
		for i in range(0,len(basis)) :
			r+=basis[i]*c[i]
		return r
	def timeStep(self,tn,tnp1,Knp1,gram,cnp1,dd,precision) :
		(c,_)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=Knp1,Knp1=Knp1,gram=gram,cnp1=cnp1,dd=dd)
		ids=gram.map.keys()
		X0=cnp1
		X1=[x[0,0] for x in c]
		grid0=Knp1
		N=len(grid0)
		ii=range(0,N)
		Q=[abs(X1[i]/X0[i]) for i in ii]
		Q2=[(Q[i],grid0[i]) for i in ii]
		Q2.sort(reverse=True)
		kappa=[-2,-1,0,1,2]
		#new grid candidates
		toAdd=[]
		for (q,(d,k)) in Q2 :
			if( q<1 ) :
				break
			toAdd+=[(d,k+i) for i in kappa if( (d,k+i) in ids )]
		#increasing scale
		(_,(dMax,kMax))=Q2[0]
		gridA=[(dMax,kMax)]
		gridB=[id for id in ids if id[0]==dMax+1]
		g=gram.gram2(gridA,gridB)[0]
		gg=[(abs(g[i]),gridB[i]) for i in range(0,len(g)) if g[i]!=0]
		gg.sort(reverse=True)
		for i in range(0,min(len(kappa),len(gg))) :
			toAdd.append(gg[i][1])
		#making new grid
		grid1=uniqueUnion(grid0,toAdd)
		#making next time step
		(c1,E1)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=grid1,Knp1=grid0,gram=gram,cnp1=cnp1,dd=dd)
		#excluding components with low contribution
		L=[(abs(c1[i,0]),grid1[i]) for i in range(0,len(grid1))]
		L.sort()
		i=0
		sum=0
		while(sum<precision) :
			sum+=L[i][0]
			i+=1
		N=i-1
		grid2=[L[i][1] for i in range(N,len(L))]
		#making time step with corrected grid
		(c2,E2)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=grid2,Knp1=grid0,gram=gram,cnp1=cnp1,dd=dd)
		return (c2,grid2)
	def evolve(self,t,T,grid,gram,c,dd,steps,precision) :
		dt=(T-t)*1.0/steps
		for s in range(0,steps) :
			print "start step "+str(s)
			(c,grid)=self.timeStep(tn=T-(s+1)*dt,tnp1=T-s*dt,Knp1=grid,gram=gram,cnp1=c,dd=dd,precision=precision)
			c=[c[i,0] for i in range(0,len(c))]
			print "end step "+str(s)
			print "grid length " +str(len(grid))
		return (c,grid)
		

def pm(x) :
	import ots.r_com
	ots.r_com.r.plotMatrix(_m(x))
def diag(x) :
	import numpy as np
	return _m(np.diag(np.diag(_m(x))))
def diagSqrt(x) :
	import numpy as np
	d=np.diag(_m(x))
	return _m(np.diag([np.sqrt(dd) for dd in d]))
def cn(x) :
	import ltools
	return ltools.condN(_m(x))
def unit(len) :
	import numpy as np
	return _m(np.diag([1 for x in range(0,len)]))
def sortGrid(g) :
	minScale=1000000
	for x in g :
		try :
			int(x[0])
		except :
			continue
		minScale=min(minScale,x[0])
	r=[]
	for x in g :
		scale=x[0]
		location=x[1]
		try :
			int(scale)
		except :
			scale=minScale
		r.append((location*1.0/pow(2,scale),x[0],x[1]))
	r.sort()
	return [(x[1],x[2]) for x in r]
def makeH2(dt,U) :
	import numpy as np
	U=dt*U
	N=U.shape[0]
	I=unit(N)
	sumUk=[]
	for k in range(0,N) :
		sum=0
		for s in range(0,N) :
			sum+=pow(U[k,s],2)
		sumUk.append(sum)
	Dk=[]
	for k in range(0,N) :
		x=(1+U[k,k])/(1+2*U[k,k]+sumUk)
		Dk.append(x)
	D=diag(Dk)
	H=D-I+D*U
	return (H,D)
class BlackSolution(object) :
	def __init__(self,payoff,sigma,T) :
		import numpy as np
		self.g=payoff
		self.sigma=sigma
		self.T=T
	def func(self,z) :
		import numpy as np
		arg=np.log(z/self.x)+0.5*self.theta2
		return self.g(z)*self.mult1/z*np.exp(self.mult2*pow(arg,2))
	def value(self,t,x) :
		import numpy as np
		import scipy as sp
		self.x=x
		self.theta=sigma*np.sqrt(self.T-t)
		self.theta2=self.theta*self.theta
		self.mult1=1.0/np.sqrt(2*np.pi)/self.theta
		self.mult2=-1.0/2/self.theta2
		spt=self.g.support()
		xA=spt.xA()
		xB=spt.xB()
		return sp.integrate.quad(self.func,xA,xB)
def uniqueUnion(X,Y) :
	Z=X+Y
	Z.sort()
	prev=None
	r=[]
	for x in Z :
		if( x==prev ) :
			continue
		prev=x
		r.append(x)
	return r

black=BlackEquation(x0=x0,t=t,T=T,sigma=sigma,precision=precision)
black.calcInterval()		
		
xsi=multiBasis['xsi']
chi=multiBasis[xsi.d]
msb=gram.MultiScaleBasis(xsi,chi,black.int_d0,black.int_kA,black.int_kB)
		
I=poly.Piece(msb.d,msb.kA,msb.kB)

strikeD=poly.pow2_(I.d())*strike
_check(strikeD-int(strikeD)==0,"strikeD is not int")
strikeD=int(strikeD)
payoff=poly.extend(poly.Piece(I.d(),strikeD,I.b()),[-strike,1])
u=poly.extend(I,[-I.xA(),1])*((I.xB()-strike)/(I.xB()-I.xA()))
payoff=poly.LPoly(payoff-u)
pp(payoff)

try :
	gm
except :
	print 'loading gram file'
	gm=load(gramFile)	
	print 'loaded gram file'
	gm.afterPickle()

grid=load(gridFile)
dt=1

proj=gm.projection2(payoff,grid)

bs=BlackSolution(payoff,sigma,T)
print bs.value(t,x0)	


(c3,grid3)=black.evolve(t=t,T=T,grid=grid,gram=gm,c=proj[1],dd=10,steps=5,precision=1.e-4)
r3=black.formFunc2(c=c3,grid=grid3,gram=gm)
print r3(x0)

