execfile('common.py')

import gram
import ots.io as io

name='blackGd.py'

version="A"
execfile('version_'+version+'.py')
basisFile='dirichlet3a_step'+str(steps)+'_d45_crN'+str(crN)+'_maxD'+str(maxD)+'_result'
gramFile='gram_version_'+version
gridFile='grid_version_'+version

try :
	multiBasis
except :
	multiBasis=load(basisFile)

def _check(cond,msg) :
	if( not cond ) :
		raise(Exception(msg))
		
def _m(a) :
	import numpy
	return numpy.matrix(a)

class BlackEquation(object) :
	def __init__(self,x0,t,T,sigma,precision) :
		self.x0=x0
		self.t=t
		self.T=T
		self.sigma=sigma
		self.precision=precision
		self.minimalNumberOfK=16
		self.minimalD=-7
	def calcInterval(self) :
		import scipy.stats
		import numpy
		N01Inv=scipy.stats.norm(0,1).ppf
		xsi0=abs(N01Inv(self.precision/2))
		sigma=self.sigma
		dt=self.T-self.t
		x1=numpy.exp(sigma*xsi0*numpy.sqrt(dt))
		x2=numpy.exp(-0.5*sigma*sigma*dt)
		x0=self.x0
		d0=self.minimalD
		while(True) :
			kA=int(numpy.floor(poly.pow2_(d0)*x0*x2/x1))
			kB=int(numpy.ceil(poly.pow2_(d0)*x0*x2*x1))
			kk=kB-kA+1
			if( kk>=self.minimalNumberOfK ) :
				break
			d0+=1
		self.int_d0=d0
		self.int_kA=kA
		self.int_kB=kB
	def oneTimeStep(self,tn,tnp1,Kn,Knp1,gram,cnp1) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		Gnp1=_m(gram.gram2(Kn,Knp1))
		G=_m(gram.gram(Kn))
		GI=G.I
		Q=_m(gram.bgram(Kn))
		U=0.5*sigma*sigma*GI*Q.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		h=(I-H)*D*GI*Gnp1*_m(cnp1).T
		for p in range(0,11) :
			H=H*H
			h=(I+H)*h
		basis_n=[gram.func(id) for id in Kn]
		basis_np1=[gram.func(id) for id in Knp1]
		r=poly.LZero(basis_n[0].support())
		for i in range(0,len(basis_n)) :
			r+=basis_n[i]*h[i,0]
		r0=poly.LZero(basis_np1[0].support())
		for i in range(0,len(basis_np1)) :
			r0+=basis_np1[i]*cnp1[i]
		return (h,r,r0,H)
	def oneTimeStepE(self,tn,tnp1,Kn,Knp1,gram,cnp1) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		Gnp1=_m(gram.gram2(Kn,Knp1))
		G=_m(gram.gram(Kn))
		GI=G.I
		Q=_m(gram.bgram(Kn))
		U=0.5*sigma*sigma*GI*Q.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		E=(I-H)*D
		for p in range(0,11) :
			H=H*H
			E=(I+H)*E
		b=GI*Gnp1*_m(cnp1).T
		h=E*b
		basis_n=[gram.func(id) for id in Kn]
		basis_np1=[gram.func(id) for id in Knp1]
		r=poly.LZero(basis_n[0].support())
		for i in range(0,len(basis_n)) :
			r+=basis_n[i]*h[i,0]
		r0=poly.LZero(basis_np1[0].support())
		for i in range(0,len(basis_np1)) :
			r0+=basis_np1[i]*cnp1[i]
		return (h,r,r0,E,H)

def pm(x) :
	import ots.r_com
	ots.r_com.r.plotMatrix(_m(x))
def diag(x) :
	import numpy as np
	return _m(np.diag(np.diag(_m(x))))
def diagSqrt(x) :
	import numpy as np
	d=np.diag(_m(x))
	return _m(np.diag([np.sqrt(dd) for dd in d]))
def cn(x) :
	import ltools
	return ltools.condN(_m(x))
def unit(len) :
	import numpy as np
	return _m(np.diag([1 for x in range(0,len)]))
def sortGrid(g) :
	minScale=1000000
	for x in g :
		try :
			int(x[0])
		except :
			continue
		minScale=min(minScale,x[0])
	r=[]
	for x in g :
		scale=x[0]
		location=x[1]
		try :
			int(scale)
		except :
			scale=minScale
		r.append((location*1.0/pow(2,scale),x[0],x[1]))
	r.sort()
	return [(x[1],x[2]) for x in r]
def makeH2(dt,U) :
	import numpy as np
	U=dt*U
	N=U.shape[0]
	I=unit(N)
	sumUk=[]
	for k in range(0,N) :
		sum=0
		for s in range(0,N) :
			sum+=pow(U[k,s],2)
		sumUk.append(sum)
	Dk=[]
	for k in range(0,N) :
		x=(1+U[k,k])/(1+2*U[k,k]+sumUk)
		Dk.append(x)
	D=diag(Dk)
	H=D-I+D*U
	return (H,D)
class BlackSolution(object) :
	def __init__(self,payoff,sigma,T) :
		import numpy as np
		self.g=payoff
		self.sigma=sigma
		self.T=T
	def func(self,z) :
		import numpy as np
		arg=np.log(z/self.x)+0.5*self.theta2
		return self.g(z)*self.mult1/z*np.exp(self.mult2*pow(arg,2))
	def value(self,t,x) :
		import numpy as np
		import scipy as sp
		self.x=x
		self.theta=sigma*np.sqrt(self.T-t)
		self.theta2=self.theta*self.theta
		self.mult1=1.0/np.sqrt(2*np.pi)/self.theta
		self.mult2=-1.0/2/self.theta2
		spt=self.g.support()
		xA=spt.xA()
		xB=spt.xB()
		return sp.integrate.quad(self.func,xA,xB)
def uniqueUnion(X,Y) :
	Z=X+Y
	Z.sort()
	prev=None
	r=[]
	for x in Z :
		if( x==prev ) :
			continue
		prev=x
		r.append(x)
	return r

black=BlackEquation(x0=x0,t=t,T=T,sigma=sigma,precision=precision)
black.calcInterval()		
		
xsi=multiBasis['xsi']
chi=multiBasis[xsi.d]
msb=gram.MultiScaleBasis(xsi,chi,black.int_d0,black.int_kA,black.int_kB)
		
I=poly.Piece(msb.d,msb.kA,msb.kB)

strikeD=poly.pow2_(I.d())*strike
_check(strikeD-int(strikeD)==0,"strikeD is not int")
strikeD=int(strikeD)
payoff=poly.extend(poly.Piece(I.d(),strikeD,I.b()),[-strike,1])
u=poly.extend(I,[-I.xA(),1])*((I.xB()-strike)/(I.xB()-I.xA()))
payoff=poly.LPoly(payoff-u)
pp(payoff)

try :
	gm
except :
	print 'loading gram file'
	gm=load(gramFile)	
	print 'loaded gram file'
	gm.afterPickle()

grid=load(gridFile)
grid2=sortGrid(grid)
dt=1

proj=gm.projection2(payoff,grid2)
bs=BlackSolution(payoff,sigma,T)
print bs.value(T-dt,x0)	

x1=black.oneTimeStep(tn=T-dt,tnp1=T,Kn=grid2,Knp1=grid2,gram=gm,cnp1=proj[1])
print x1[1](x0)

# L_3=[x for x in gm.map.keys() if x[0]==-3]
# L_2=[x for x in gm.map.keys() if x[0]==-2]
# L_1=[x for x in gm.map.keys() if x[0]==-1]
# L0=[x for x in gm.map.keys() if x[0]==0]
# L1=[x for x in gm.map.keys() if x[0]==1]
# L2=[x for x in gm.map.keys() if x[0]==2]
# L3=[x for x in gm.map.keys() if x[0]==3]
# grid3=uniqueUnion(grid2,L_3+L_2+L_1+L0+L1+L2+L3)
# x2=black.oneTimeStep(tn=T-dt,tnp1=T,Kn=grid3,Knp1=grid2,gram=gm,cnp1=proj[1])
# print x2[1](x0)

x3=black.oneTimeStepE(tn=T-dt,tnp1=T,Kn=grid2,Knp1=grid2,gram=gm,cnp1=proj[1])
print x3[1](x0)
E=x3[3]
print ltools.spectralRadius(E)

dd=10
steps=pow(2,dd)
ddt=dt*1.0/steps
x4=black.oneTimeStepE(tn=T-ddt,tnp1=T,Kn=grid2,Knp1=grid2,gram=gm,cnp1=proj[1])
EE=x4[3]
c=pow(EE,steps)*_m(proj[1]).T
basis_n=[gm.func(id) for id in grid2]
r=poly.LZero(basis_n[0].support())
for i in range(0,len(basis_n)) :
	r+=basis_n[i]*c[i,0]
print r(x0)
