import numpy
import scipy.linalg as la
import time

class Store(object) :
	def __init__(self) :
		self._data={}
	def put(self,name,value) :
		try :
			self._data[name]=[ltools.polyToDescription(x) for x in value]
		except :
			self._data[name]=value
	def dir(self) :
		return self._data.keys()
	def keys(self) :
		return self.dir()
	def __getattr__(self,name) :
		if( name in self.dir() ) :
			try :
				return [ltools.descriptionToPoly(x) for x in self._data[name]]
			except :
				return self._data[name]
		else :
			return super(type(self)).__getattr__(self,name)

class EmptyStore(object) :
	def __init__(self) :
		pass
	def put(self,name,value) :
		pass
	def dir(self) :
		return []
	def keys(self) :
		return self.dir()
	def __getattr__(self,name) :
		return super(type(self)).__getattr__(self,name)

			
			
class PsiGenerator(object) :
	"""
	Source of psi-basis on [0,1] with Dirichlet boundary conditions.
	"""
	def __init__(self,dGenerator,dp1Generator,psi) :
		self._dGen=dGenerator
		self._dp1Gen=dp1Generator
		self._psi=psi
		pg0=self._dGen
		pg1=self._dp1Gen
		dimV0=2*(pg0._n-1)+len(pg0._kI)
		dimV1=2*(pg1._n-1)+len(pg1._kI)
		dDim=dimV1-dimV0
		d2Boundary=dDim-len(pg0._kI)
		if( d2Boundary % 2 != 0 ) :
			raise(Exception('d2Boundary is odd'))
		dBoundary=d2Boundary/2
		NtoDiscard=pg0._n-1-dBoundary
		if( NtoDiscard<0 ) :
			raise(Exception('NtoDiscard is negative'))
		self._NtoDiscard=NtoDiscard
	def calculate(self) :
		pg0=self._dGen
		pg1=self._dp1Gen
		psi=ltools.descriptionToPoly(self._psi)
		phi=pg0._phi
		kk0=pg0._kk
		kk1=pg1._kk
		kI0=pg0._kI
		kI1=pg1._kI
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		kL1=pg1._kI[0]-1
		kR1=pg1._kI[-1]+1
		d0=pg0._d
		d1=pg1._d
		I=poly2.Piece(0,0,1)
		n=pg0._n-1
		xsi0=[]
		for k in kk0 :
			xsi0.append(pg0.xsid_(k))
		xsi1=[]
		for k in kk1 :
			xsi1.append(pg1.xsid_(k))
		print 'xsi0,xsi1 constructed'
		grammInv=ltools.grammInv(f=xsi0,I=I)
		zi=[]
		print 'calculating zi'
		for k in range(0,n) :
			u=xsi1[k]
			proj=ltools.projection(u=u,f=xsi0,grammInv=grammInv,I=I)
			df=u-proj
			print 'L2 of proj='+str(ltools.l2Norm(proj,I))
			L2=ltools.l2Norm(df,I)
			print 'L2 of proj df='+str(L2)
			zi.append(df*(1.0/L2))
		print 'zi,L constructed'
		for k in kI0 :
			print k
			zi.append(psi.Op(d0,k))
		print 'zi,I constructed'
		for k in range(0,n) :
			u=xsi1[len(xsi1)-n+k]
			proj=ltools.projection(u=u,f=xsi0,grammInv=grammInv,I=I)
			df=u-proj
			print 'L2 of proj='+str(ltools.l2Norm(proj,I))
			L2=ltools.l2Norm(df,I)
			print 'L2 of proj df='+str(L2)
			zi.append(df*(1.0/L2))
		print 'zi,R constructed'
		self.store=EmptyStore()
		self.store.put('xsi0',xsi0)
		self.store.put('xsi1',xsi1)
		self.store.put('zi',zi)
		print 'removing proj_I'
		ziI=zi[n:(len(zi)-n)]
		grammInv=ltools.grammInv(f=ziI,I=I)
		chiL=[]
		for k in range(0,n) :
			chiL.append(zi[k]-ltools.projection(u=zi[k],f=ziI,grammInv=grammInv,I=I))
		chiR=[]
		for k in range(len(zi)-n,len(zi)) :
			chiR.append(zi[k]-ltools.projection(u=zi[k],f=ziI,grammInv=grammInv,I=I))
		print 'chiL,chiR done'
		self.store.put('chiL',chiL)
		self.store.put('chiR',chiR)
		print 'calculating chi'
		NtoDiscard=self._NtoDiscard
		print 'NtoDiscard='+str(NtoDiscard)
		chiL.reverse()
		chiL2=ltools.grammSchmidt2(f=chiL,I=I)
		chiL2[0].reverse()
		chiR2=ltools.grammSchmidt2(f=chiR,I=I)
		NtoDiscard=self.calcNtoDiscard(NtoDiscard,chiL2[1],chiR2[1])
		self._NtoDiscard=NtoDiscard
		if( NtoDiscard==0 ) :
			chi=chiL2[0]+zi[n:(len(zi)-n)]+chiR2[0]
		else :
			chi=chiL2[0][NtoDiscard:]+zi[n:(len(zi)-n)]+chiR2[0][:-NtoDiscard]
		self.store.put('chiL2',chiL2[0])
		self.store.put('chiR2',chiR2[0])
		self.store.put('chiL2_1',chiL2[1])
		self.store.put('chiR2_1',chiR2[1])
		self.store.put('chi',chi)
		print 'chi done'
		self._chid={}
		shft=kk0[NtoDiscard]
		for i in range(0,len(chi)) :
			self._chid[shft+i]=ltools.polyToDescription(chi[i])
		for k in range(kL0+1,kR0) :
			del self._chid[k]			
	def calcNtoDiscard(self,n,L1,L2) :
		return n
	def chid(self,k) :
		pg0=self._dGen
		pg1=self._dp1Gen
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		n=pg0._n-1
		if( k in range(kL0+1,kR0) ) :
			return ltools.polyToDescription(
				ltools.descriptionToPoly(self._psi).Op(pg0._d,k)
				)
		if( kL0-n<k and k<=kL0 ) :
			return self._chid[k]
		if( kR0<=k and k<kR0+n ) :
			return self._chid[k]
		raise(Exception('wrong index k'))
	def chid_(self,k) :
		pg0=self._dGen
		pg1=self._dp1Gen
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		n=pg0._n-1
		if( k in range(kL0+1,kR0) ) :
			return ltools.descriptionToPoly(self._psi).Op(pg0._d,k)
		return ltools.descriptionToPoly(self.chid(k))
	def makeDp1(self) :
		pg0=self._dGen
		pg1=self._dp1Gen
		psi=ltools.descriptionToPoly(self._psi)
		phi=ltools.descriptionToPoly(pg0._phi)
		kk0=pg0._kk
		kk1=pg1._kk
		kI0=pg0._kI
		kI1=pg1._kI
		kL0=pg0._kI[0]-1
		kR0=pg0._kI[-1]+1
		kL1=pg1._kI[0]-1
		kR1=pg1._kI[-1]+1
		d0=pg0._d
		d1=pg1._d
		n=pg0._n
		r=PsiGenerator(
			dGenerator=self._dp1Gen,
			dp1Generator=self._dp1Gen.makeDp1(),
			psi=self._psi
			)
		if( self._NtoDiscard!=r._NtoDiscard ) :
			raise(Exception('NtoDiscard mismatch'))
		r._chid={}
		#find k_d on left
		spt0=phi.support().Op(d0,kL0)
		spt1=phi.support().Op(d1,kL1)
		diff=10
		kD=-1000
		for k in range(0,len(kI1)+2*n+1) :
			sptA=spt0.Op(1,k)
			d=numpy.abs(sptA.xB()-spt1.xB())
			if( d<diff ) :
				diff=d
				kD=k
			sptA=spt0.Op(1,-k)
			d=numpy.abs(sptA.xB()-spt1.xB())
			if( d<diff ) :
				diff=d
				kD=-k
		kL=kD
		print 'kL='+str(kL)
		#find k_d on right
		spt0=phi.support().Op(d0,kR0)
		spt1=phi.support().Op(d1,kR1)
		diff=10
		kD=-1000
		for k in range(0,len(kI1)+2*n+1) :
			sptA=spt0.Op(1,k)
			d=numpy.abs(sptA.xB()-spt1.xB())
			if( d<diff ) :
				diff=d
				kD=k
			sptA=spt0.Op(1,-k)
			d=numpy.abs(sptA.xB()-spt1.xB())
			if( d<diff ) :
				diff=d
				kD=-k
		kR=kD	
		print 'kR='+str(kR)
		for i in range(0,n-self._NtoDiscard-1) :
			print i
			r._chid[kL1-i]=ltools.polyToDescription(self.chid_(kL0-i).Op(1,kL))
			r._chid[kR1+i]=ltools.polyToDescription(self.chid_(kR0+i).Op(1,kR))
		try :
			del r._dp1Gen._I
		except :
			pass
		return r
		
	
def scalingPsiW12(psiGen) :
	print 'scalingPsiW12'
	pg0=psiGen._dGen
	pg1=psiGen._dp1Gen
	psi=psiGen._psi
	phi=pg0._phi
	kk0=pg0._kk
	kk1=pg1._kk
	kI0=pg0._kI
	kI1=pg1._kI
	kL0=pg0._kI[0]-1
	kR0=pg0._kI[-1]+1
	kL1=pg1._kI[0]-1
	kR1=pg1._kI[-1]+1
	d0=pg0._d
	d1=pg1._d
	I=poly2.Piece(0,0,1)
	n=pg0._n
	print 'building xsi0'
	xsi0={}
	for k in kk0 :
		xsi0[k]=pg0.xsid_(k)
	print 'building xsi1'
	xsi1=[]
	for k in kk1 :
		xsi1.append(pg1.xsid_(k))
	print 'building chi'
	chi={}
	kChi=psiGen._chid.keys()
	kChi.sort()
	for k in kChi :
		chi[k]=psiGen.chid_(k)
	grammInv=ltools.grammInvW12(f=xsi1,I=I)
	scale1={}
	keys0=xsi0.keys()
	keys0.sort()
	for k in keys0 :
		u=xsi0[k]
		proj=ltools.projectionW12(u=u,f=xsi1,grammInvW12=grammInv,I=I)
		df=u-proj
		L2=ltools.w12Norm(df,I)
		print 'W12 df='+str(L2)
		scale1[k]=L2
	scale2={}
	for k in kChi :
		u=chi[k]
		proj=ltools.projectionW12(u=u,f=xsi1,grammInvW12=grammInv,I=I)
		df=u-proj
		L2=ltools.w12Norm(df,I)
		print 'W12 df='+str(L2)
		scale2[k]=L2
	r={}
	r['d0']=d0
	r['d1']=d1
	r['xsi0 in xsi1']=scale1
	r['chi in xsi1']=scale2
	return r
	
	
def scalingPsiL2(psiGen) :
	print 'scalingPsiL2'
	pg0=psiGen._dGen
	pg1=psiGen._dp1Gen
	psi=psiGen._psi
	phi=pg0._phi
	kk0=pg0._kk
	kk1=pg1._kk
	kI0=pg0._kI
	kI1=pg1._kI
	kL0=pg0._kI[0]-1
	kR0=pg0._kI[-1]+1
	kL1=pg1._kI[0]-1
	kR1=pg1._kI[-1]+1
	d0=pg0._d
	d1=pg1._d
	I=poly2.Piece(0,0,1)
	n=pg0._n
	print 'building xsi0'
	xsi0={}
	for k in kk0 :
		xsi0[k]=pg0.xsid_(k)
	print 'building xsi1'
	xsi1=[]
	for k in kk1 :
		xsi1.append(pg1.xsid_(k))
	print 'building chi'
	chi={}
	kChi=psiGen._chid.keys()
	kChi.sort()
	for k in kChi :
		chi[k]=psiGen.chid_(k)
	grammInv=ltools.grammInv(f=xsi1,I=I)
	scale1={}
	keys0=xsi0.keys()
	keys0.sort()
	for k in keys0 :
		u=xsi0[k]
		proj=ltools.projection(u=u,f=xsi1,grammInv=grammInv,I=I)
		df=u-proj
		L2=ltools.l2Norm(df,I)
		print 'L2 df='+str(L2)
		scale1[k]=L2
	scale2={}
	for k in kChi :
		u=chi[k]
		proj=ltools.projection(u=u,f=xsi1,grammInv=grammInv,I=I)
		df=u-proj
		L2=ltools.l2Norm(df,I)
		print 'L2 df='+str(L2)
		scale2[k]=L2
	r={}
	r['d0']=d0
	r['d1']=d1
	r['xsi0 in xsi1']=scale1
	r['chi in xsi1']=scale2
	return r
	
