import PiecewisePoly as poly
import numpy
import scipy.linalg as la

def _NullScalarProduct( f1,f2 ) :
	raise(Exception("Use funcan.setScalarProduct to supply a function(f1,f2)"))
def _NullZero( spt ) :
	raise(Exception("Use funcan.setZero to supply a function(support) returning a zero object"))
_scalarProduct=_NullScalarProduct
_makeZero=_NullZero
def setScalarProduct( sp ) :
	import funcan as _f
	_f._scalarProduct=sp
def clearScalarProduct() :
	import funcan as _f
	_f._scalarProduct=_NullScalarProduct
def setZero( z ) :
	import funcan as _f
	_f._makeZero=z
def clearZero() :
	import funcan as _f
	_f._makeZero=_NullZero
def _scalarInL2(f1,f2) :
	return f1.mult(f2).integral()
def _scalarInL2_01(f1,f2) :
	I=poly.Piece(0,0,1)
	f1=f1.segment(I)
	f2=f2.segment(I)
	return _scalarInL2(f1,f2)
def _scalarInL2_11(f1,f2) :
	I=poly.Piece(0,-1,1)
	f1=f1.segment(I)
	f2=f2.segment(I)
	return _scalarInL2(f1,f2)
def _scalarInW12(f1,f2) :
	return f1.mult(f2).integral() + f1.D().mult(f2.D()).integral()
def _scalarInW12_01(f1,f2) :
	I=poly.Piece(0,0,1)
	f1=f1.segment(I)
	f2=f2.segment(I)
	return _scalarInW12(f1,f2)
def _scalarInW12_11(f1,f2) :
	I=poly.Piece(0,-1,1)
	f1=f1.segment(I)
	f2=f2.segment(I)
	return _scalarInW12(f1,f2)
def _scalarBlack(f1,f2) :
	x2=poly.LPoly(poly.extend(f2.support(),[0,0,1]))
	f1=f1.D()
	f2=f2.mult(x2).D()
	return _scalarInL2(f1,f2)
def _scalarBlack_11(f1,f2) :
	I=poly.Piece(0,-1,1)
	x2=poly.LPoly(poly.extend(I,[0,0,1]))
	f1=f1.segment(I).D()
	f2=f2.segment(I).mult(x2).D()
	return _scalarInL2(f1,f2)
def setL2_01() :
	setScalarProduct(_scalarInL2_01)
def setW12_01() :
	setScalarProduct(_scalarInW12_01)
def setL2_11() :
	setScalarProduct(_scalarInL2_11)
def setW12_11() :
	setScalarProduct(_scalarInW12_11)
def setL2() :
	setScalarProduct(_scalarInL2)
def setW12() :
	setScalarProduct(_scalarInW12)
def setBlack() :
	setScalarProduct(_scalarBlack)
def setBlack_11() :
	setScalarProduct(_scalarBlack_11)
def _LPolyZero(spt) :
	return poly.LZero(spt)
def setLZero() :
	setZero(_LPolyZero)

def _sqrt(x) :
	if( numpy.abs(x)<1.e-16 and x<0 ) :
		return 0.0
	if( x<0 ) :
		raise Exception("_sqrt: x="+str(x))
	return numpy.sqrt(x)
	
def scalar(f1,f2) :
	return _scalarProduct(f1,f2)
def dist(f1,f2) :
	d=f1-f2
	return _sqrt(scalar(d,d))
def norm(f) :
	return _sqrt(scalar(f,f))
	
def condN(A) :
	import scipy.linalg as la
	(lam,Q)=la.eig(A)
	return max(lam)/min(lam)
	
def grammMatrix(f) :
	a=[]
	for x in f :
		aa=[]
		a.append(aa)
		for y in f :
			aa.append(scalar(x,y))
	return numpy.matrix(a)
def grammInv(f) :
	m=grammMatrix(f)
	print 'grammInv, condN='+str(condN(m))
	return m.I
def scalarColumn(u,f) :
	a=[]
	for x in f :
		a.append([scalar(u,x)])
	return numpy.matrix(a)
def projection(u,f,grammInv) :
	b=scalarColumn(u=u,f=f)
	c=grammInv*b
	proj=_makeZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	return proj
def projection2(u,f) :
	print 'calculating projection'
	gInv=grammInv(f=f)
	b=scalarColumn(u=u,f=f)
	c=gInv*b
	proj=_makeZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)
def projection3(u,f,grammInv) :
	b=scalarColumn(u=u,f=f)
	c=grammInv*b
	proj=_makeZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	return (proj,cc)
def grammSchmidt(f) :
	r=[]
	Norms=[]
	for k in range(0,len(f)) :
		proj=_makeZero(f[0].support())
		for p in range(0,k) :
			c=scalar(f[k],r[p])
			proj+=r[p]*c
		rr=f[k]-proj
		L2=norm(rr)
		print 'GrammSchmidt, L2='+str(L2)
		Norms.append(L2)
		r.append(rr/L2)
	return (r,L2)
