import PiecewisePoly as poly
import numpy
import scipy.linalg as la
import dirichlet 
import ltools
import ots.r_com as r_com

def _check(cond,msg) :
	if( not cond ) :
		raise(Exception(msg))
		
def plotPoly(func,N=10000,x0=None,x1=None) :
	"Plot a PiecewisePoly.Poly-nomial via r_com"
	spt=func.support()
	scale=pow(2,spt.d())
	if( x0 is None ) :
		x0=(1.0*spt.a())/scale
	if( x1 is None ):
		x1=(1.0*spt.b())/scale
	h=(x1-x0)*1.0/N
	x=[x0+i*h for i in range(0,N+1)]
	y=func.batch(x)
	r_com.r.set('x',x)
	r_com.r.set('y',y)
	r_com.r.plot(r_com.r('x'),r_com.r('y'))
	

class OneScaleBasis(object) :
	def __init__(self,K1,KI,K2,funcs,d,phi,kStar) :
		self.K1=K1
		self.KI=KI
		self.K2=K2
		self.K=K1+KI+K2
		x={}
		self._funcs=x
		for k in funcs :
			x[k]=ltools.polyToDescription(funcs[k]) 
		self.d=d
		self._phi=ltools.polyToDescription(phi)
		self._kStar=kStar
	def phi_(self) :
		return ltools.descriptionToPoly(self._phi)
	def get_(self,k) :
		if( k in self.K1 or k in self.K2 ) :
			return ltools.descriptionToPoly(self._funcs[k])
		if( k in self.KI ) :
			return self.phi_().T(k+self._kStar).S(self.d)
		raise(Exception("Wrong k="+str(k)))
	def __str__(self) :
		s="OneScaleBasis\n"
		s+="K1="+str(self.K1)+"\n"
		s+="KI="+str(self.KI)+"\n"
		s+="K2="+str(self.K2)+"\n"
		s+="K="+str(self.K)+"\n"
		s+="d="+str(self.d)+"\n"
		s+="kStar="+str(self._kStar)+"\n"
		s+="_funcs.keys()="+str(self._funcs.keys())+"\n"
		return s
		
class MultiScaleBasis(object) :
	def __init__(self,xsi,chi,d,kA,kB) :
		_check(chi.n==xsi.n,"xsi.n!=chi.n")
		_check(chi.d==xsi.d,"xsi.d!=chi.d")
		#interval params
		self.d=d
		self.kA=kA
		self.kB=kB
		self.n=xsi.n
		#shifting auxiliaries
		d0=xsi.d
		k1=poly.pow2_(d0)+kA
		k2=-poly.pow2_(d0)+kB
		_check(k1-int(k1)==0,"k1 is not int")
		_check(k2-int(k2)==0,"k2 is not int")
		k1=int(k1)
		k2=int(k2)
		n=xsi.n
		m=n-1
		#index sets
		M=kB-kA
		Kd1=range(0,m)
		Kd2=range(M-m,M)
		KdI=range(m,M-m)
		#make xsi basis
		self._xsi={}
		Xsi={}
		for i in range(0,m) :
			Xsi[Kd1[0]+i]=xsi.xsi_(xsi.Kd1[0]+i).S(-d0).T(k1).S(d)
			Xsi[Kd2[0]+i]=xsi.xsi_(xsi.Kd2[0]+i).S(-d0).T(k2).S(d)
		kStarD=kA
		phi=xsi.xsi_(xsi.KdI[0]).S(-d0).T(k1-m-kStarD)
		self._xsi[d]=OneScaleBasis(Kd1,KdI,Kd2,Xsi,d,phi,kStarD)
		#make chi basis
		self._chi={}
		Chi={}
		for i in range(0,m) :
			Chi[Kd1[0]+i]=chi.chi_(chi.Kd1[0]+i).S(-d0).T(k1).S(d)
			Chi[Kd2[0]+i]=chi.chi_(chi.Kd2[0]+i).S(-d0).T(k2).S(d)
		psi=chi.chi_(chi.KdI[0]).S(-d0).T(k1-m-kStarD)
		self._chi[d]=OneScaleBasis(Kd1,KdI,Kd2,Chi,d,psi,kStarD)
	def scaleUp(self) :
		#recover data and target
		kk=self._chi.keys()
		kk.sort()
		chi=self._chi[kk[0]]
		d1=kk[-1]+1
		#basic parameters
		d=self.d
		kA=self.kA
		kB=self.kB
		n=self.n
		d0=chi.d
		M=(kB-kA)*poly.pow2_(d1-d)
		k0=poly.pow2_(-d)*(poly.pow2_(d1)-poly.pow2_(d0))
		k1=kA*k0
		k2=kB*k0
		_check(M-int(M)==0,"M is not int")
		_check(k1-int(k1)==0,"k1 is not int")
		_check(k2-int(k2)==0,"k2 is not int")
		k1=int(k1)
		k2=int(k2)
		M=int(M)
		n=self.n
		m=n-1
		#index sets
		Kd1=range(0,m)
		Kd2=range(M-m,M)
		KdI=range(m,M-m)
		#make chi basis
		Chi={}
		for i in range(0,m) :
			Chi[Kd1[0]+i]=chi.get_(chi.K1[0]+i).S(-d0).T(k1).S(d1)
			Chi[Kd2[0]+i]=chi.get_(chi.K2[0]+i).S(-d0).T(k2).S(d1)
		kStarD=int(numpy.floor(poly.pow2_(d1-d)*kA))
		psi=chi.get_(chi.KI[0]).S(-d0).T(k1-m-kStarD)
		self._chi[d1]=OneScaleBasis(Kd1,KdI,Kd2,Chi,d1,psi,kStarD)
		_check(ltools.Dist(self._chi[d1].phi_(),self._chi[d1-1].phi_())<0.000001,"phi do not match")
	def xsi(self,d) :
		return self._xsi[d]
	def chi(self,d) :
		return self._chi[d]
	def __str__(self) :
		s="MultiScaleBasis\n"
		s+="d="+str(self.d)+"\n"
		s+="n="+str(self.n)+"\n"
		s+="kA="+str(self.kA)+"\n"
		s+="kB="+str(self.kB)+"\n"
		s+="_xsi.keys()="+str(self._xsi.keys())+"\n"
		s+="_chi.keys()="+str(self._chi.keys())+"\n"
		return s
	def makeCommonPhi(self) :
		d0=self._xsi.keys()[0]
		xsi0=self._xsi[d0]
		m=xsi0.KI[0]
		phi=CommonPhi(xsi_d0_m=xsi0.get_(m),d0=d0,m=m,d=self.d,kA=self.kA)
		return phi

	
class GramMatrix(object) :
	def __init__(self,msb,interval) :
		self._xsiSP={}
		self._chiSP={}
		self.msb=msb
		self.I=interval
		self.II=(interval.d(),interval.a(),interval.b())
		self.xSq=poly.LPoly(poly.extend(self.I,[0,0,1]))
		self._boundaries={}
		self._D={}
		self._D2={}
		self._makeIds()
		self._calcMatrix()
		self._calcBMatrix()
	def prePickle(self) :
		self.I=None
		self.xSq=None
		self._D={}
		self._D2={}
	def afterPickle(self) :
		self.I=poly.Piece(self.II[0],self.II[1],self.II[2])
		self.xSq=poly.LPoly(poly.extend(self.I,[0,0,1]))
	def _makeIds(self) :
		kk=self.msb._xsi.keys()
		kk.sort()
		xsi=self.msb.xsi(kk[0])
		self.ids=[('xsi',x) for x in xsi.K]
		kk=self.msb._chi.keys()
		kk.sort()
		for k in kk :
			self.ids+=[(k,x) for x in self.msb.chi(k).K]
		self.map={}
		for i in range(0,len(self.ids)) :
			self.map[self.ids[i]]=i
	def _scalar(self,x,y) :
		return self.scalar(self.func(x),self.func(y))
	def _bscalar(self,x,y) :
		u=self.Dfunc(x)
		v=self.D2func(y)
		if(u.support().intersect(v.support()).len()==0) :
			return 0
		return u.mult(v).integral()
	def Dfunc(self,x) :
		try :
			return self._D[x]
		except :
			u=self.func(x).D()
			self._D[x]=u
			return u
	def D2func(self,x) :
		try :
			return self._D2[x]
		except :
			u=self.func(x).mult(self.xSq).D()
			self._D2[x]=u
			return u
	def scalar(self,u,v) :
		if(u.support().intersect(v.support()).len()==0) :
			return 0
		u=u.segment(self.I)
		return u.mult(v).integral()
	def norm(self,v) :
		import numpy
		return numpy.sqrt(self.scalar(v,v))
	def _calcMatrix(self) :
		a=[]
		i=0
		N=len(self.ids)
		for x in self.ids :
			print "calculating gram matrix, row="+str(i)+" of "+str(N)
			i+=1
			aa=[]
			a.append(aa)
			for y in self.ids :
				aa.append(self._scalar(x,y))
		self.matrix=a
	def _calcBMatrix(self) :
		a=[]
		i=0
		N=len(self.ids)
		for x in self.ids :
			print "calculating Black gram matrix, row="+str(i)+" of "+str(N)
			i+=1
			aa=[]
			a.append(aa)
			for y in self.ids :
				aa.append(self._bscalar(x,y))
		self.bmatrix=a
	def func(self,id) :
		(a,b)=id
		if( a=='xsi' ) :
			return self.msb.xsi(self.msb.d).get_(b)
		return self.msb.chi(a).get_(b)
	def bestGrid(self,target,precision,center) :
		grid=[]
		for x in self.ids :
			if(x[0]=='xsi') :
				grid.append(x)
		proj=self.projection(target,grid)
		dTarget=target-proj
		plotPoly(dTarget)
		if( self.norm(dTarget)<precision ) :
			return grid
		maxD=self.msb.d
		while(True) :
			maxNorm=0
			for x in self.ids :
				if( x in grid ) :
					continue
				if( x[0]>maxD ) :
					continue
				norm=abs(self.scalar(dTarget,self.func(x)))
				if( norm>=maxNorm ) :
					maxNorm=norm
					maxId=x
			print "candidate found, len="+str(len(grid))+" d="+str(maxId[0])+" k="+str(maxId[1])+" x0="+str(maxId[1]*pow(0.5,maxId[0]))
			if( maxId[0]==maxD ):
				maxD+=1
				print "maxD="+str(maxD)
			self._checkForBoundaryId(maxId[0],maxId[1])
			kk=self.msb._chi.keys()
			kk.sort()
			if( maxId[0]==kk[-1] ) :
				self.scaleUp(center)
			grid.append(maxId)
			proj=self.projection(target,grid)
			dTarget=target-proj
			plotPoly(dTarget)
			norm=self.norm(dTarget)
			print "norm="+str(norm)
			if( norm<precision ) :
				return grid
	def scaleUp(self,center) :
		N=len(self.ids)
		#add ids
		self.msb.scaleUp()
		kk=self.msb._chi.keys()
		kk.sort()
		k=kk[-1]
		self.ids+=[(k,x) for x in self._reduceIds(k,self.msb.chi(k).K,center)]
		NN=len(self.ids)
		for i in range(N,NN) :
			self.map[self.ids[i]]=i
		#add gram matrix elements
		a=[]
		i=0
		for i in range(0,NN) :
			aa=[]
			a.append(aa)
			print "scaling up, i="+str(i)+" of "+str(NN)
			for j in range(0,NN) :
				if( i<N and j<N ) :
					aa.append(self.matrix[i][j])
				else :
					aa.append(self._scalar(self.ids[i],self.ids[j]))
		self.matrix=a
		#add Black gram matrix elements
		a=[]
		i=0
		for i in range(0,NN) :
			aa=[]
			a.append(aa)
			print "scaling up, black, i="+str(i)+" of "+str(NN)
			for j in range(0,NN) :
				if( i<N and j<N ) :
					aa.append(self.bmatrix[i][j])
				else :
					aa.append(self._bscalar(self.ids[i],self.ids[j]))
		self.bmatrix=a
	def scaleUp0(self,center) :
		N=len(self.ids)
		#add ids
		self.msb.scaleUp()
		kk=self.msb._chi.keys()
		kk.sort()
		k=kk[-1]
		self.ids+=[(k,x) for x in self._reduceIds(k,self.msb.chi(k).K,center)]
		NN=len(self.ids)
		for i in range(N,NN) :
			self.map[self.ids[i]]=i
		#add gram matrix elements
		a=[]
		i=0
		for i in range(0,NN) :
			aa=[]
			a.append(aa)
			print "scaling up, i="+str(i)+" of "+str(NN)
			for j in range(0,NN) :
				if( i<N and j<N ) :
					aa.append(self.matrix[i][j])
				else :
					aa.append(self._scalar(self.ids[i],self.ids[j]))
		self.matrix=a
	def _reduceIds(self,d,Kd,center) :
		self._maxNumK=128
		all=self._maxNumK
		half=0.5*all
		if( len(Kd)<all ) :
			ret=Kd
		else :
			cntr=center*poly.pow2_(d)
			if( cntr<=half ) :
				ret=Kd[:all]
			else :
				low=int(numpy.floor(cntr-half))
				ret=Kd[low:min(low+all,len(Kd))]
		b=(ret[0],ret[-1])
		if( b[0]==Kd[0] ) :
			b=(False,b[1])
		if( b[1]==Kd[-1] ) :
			b=(b[0],False)
		self._boundaries[d]=b
		return ret
	def _checkForBoundaryId(self,d,k) :
		try :
			x=self._boundaries[d]
		except :
			return
		if( ((not x[0]==False) and k<=x[0]) or ((not x[1]==False) and k>=x[1]) ) :
			raise(Exception("Boundary id in use"))
	def gram(self,grid) :
		a=[]
		for x in grid :
			aa=[]
			a.append(aa)
			for y in grid :
				aa.append(self.gramElement(x,y))
		return a
	def gram2(self,grid1,grid2) :
		a=[]
		for x in grid1 :
			aa=[]
			a.append(aa)
			for y in grid2 :
				aa.append(self.gramElement(x,y))
		return a
	def gramElement(self,x,y) :
		return self.matrix[self.map[x]][self.map[y]]
	def bgram(self,grid) :
		a=[]
		for x in grid :
			aa=[]
			a.append(aa)
			for y in grid :
				aa.append(self.bgramElement(x,y))
		return a
	def bgramElement(self,x,y) :
		return self.bmatrix[self.map[x]][self.map[y]]
	def scalarColumn(self,target,grid) :
		a=[]
		for x in grid :
			a.append([self.scalar(target,self.func(x))])
		return a
	def projection(self,target,grid) :
		import numpy
		import PiecewisePoly as poly
		b=numpy.matrix(self.scalarColumn(target,grid))
		a=numpy.matrix(self.gram(grid))
		c=a.I*b
		proj=poly.LZero(self.I)
		for k in range(0,len(grid)) :
			proj+=self.func(grid[k])*c[k,0]
		return proj
	def projection2(self,target,grid) :
		import numpy
		import PiecewisePoly as poly
		b=numpy.matrix(self.scalarColumn(target,grid))
		a=numpy.matrix(self.gram(grid))
		c=a.I*b
		proj=poly.LZero(self.I)
		for k in range(0,len(grid)) :
			proj+=self.func(grid[k])*c[k,0]
		return (proj,[c[k,0] for k in range(0,len(grid))])
		
def testGram(gram) :
	import funcan as fa
	fa.setL2()
	fa.setLZero()
	ff=[gm.func(x) for x in gm.ids]
	ggg=fa.grammMatrix(ff)
	gggg=numpy.matrix(gm.matrix)
	dg=ggg-gggg
	return numpy.sum(numpy.abs(dg))	
	
