import ltools
import numpy
import PiecewisePoly as poly


def centeredIndex(h) :
	"Calculate the middle index of a scaling filter with zeros."
	for k in range(0,len(h)) :
		if( h[k]!=0 ) :
			break
	kA=k
	for k in range(len(h)-1,-1,-1) :
		if( h[k]!=0 ) :
			break
	kB=k
	return int(numpy.floor((kA+kB)*0.5))
def shiftedDict(h,ci) :
	"Shift indexes of a dictionaty h by ci."
	r={}
	for k in range(0,len(h)) :
		r[k-ci]=h[k]
	return r

def A(eta,h) :
	"The operator A, see notes"
	sum=poly.LZero(eta.support())
	for k in h :
		sum=sum+eta.Op(1,k)*h[k]
	return sum
	
def waveletFilter(h) :
	"Transform scaling filter into wavelet filter (g in Notes)."
	g={}
	for k in h :
		g[1-k]=pow(-1,1-k)*h[k]
	return g
def wavelet(phi,g) :
	"Calculate wavelet from scaling function phi and wavelet filter g."
	r=poly.LZero(phi.support())
	for k in g :
		r=r+phi.Op(1,k)*g[k]
	return r


class FourFunctions(object) :
	"""
	Calculates the functions phi,Phi,psi,Psi for a GMRA given by scaling filters h,H.
	"""
	def __init__(self,h,H,eta0) :
		"""
			h,H are scaling filters (in the Notes, h and h tilde).
		"""
		self.h=shiftedDict(h,centeredIndex(h))
		self.H=shiftedDict(H,centeredIndex(H))
		self._eta0=eta0
	def cascade(self,steps,z0,crudify=False) :
		r=self._cascade(self.h,steps,z0,crudify)
		self._phi=r['phi']
		self._l2=r['L2']
		self._dl2=r['DL2']
	def Cascade(self,steps,z0,crudify=False) :
		r=self._cascade(self.H,steps,z0,crudify)
		self._Phi=r['phi']
		self._L2=r['L2']
		self._DL2=r['DL2']
	def _cascade(self,h,steps,z0,crudify) :
		z1=ltools.descriptionToPoly(z0)
		minDL2=100000
		toContinue=True
		for i in range(1,steps+1) :
			if( not toContinue ) :
				break
			print '1: '+str(poly.deviceError_getMemoryInfo())
			tmp=z1.moveToHost()
			try :
				del z1
				del z2
			except :
				pass
			print '2: '+str(poly.deviceError_getMemoryInfo())
			print "step "+str(i)
			z1=poly.LPoly(tmp)
			if(crudify) :
				z1=z1.Cr()
			z2=A(z1,h)
			L2new=ltools.Dist(z1,z2)
			DL2new=ltools.Dist(z1.D(),z2.D())
			if( i>5 and L2new>L2 and DL2new>DL2 ) :
				toContinue=False
			L2=L2new
			DL2=DL2new
			if( DL2<minDL2 ) :
				minDL2=DL2
				minL2=L2
				bestZ=ltools.polyToDescription(z1)
			print "L2="+str(L2)
			print "DL2="+str(DL2)
			z1=z2
		return {'L2':minL2,'DL2':minDL2,'phi':bestZ}
	def wavelet(self) :
		g=waveletFilter(self.H)
		psi=wavelet(ltools.descriptionToPoly(self._phi),g)
		self._psi=ltools.polyToDescription(psi)
		return self._psi
	def Wavelet(self) :
		g=waveletFilter(self.h)
		psi=wavelet(ltools.descriptionToPoly(self._Phi),g)
		self._Psi=ltools.polyToDescription(psi)
		return self._Psi
	def run(self,steps) :
		self._steps=steps
		z0=self._eta0
		self.cascade(steps,z0,False)
		self.cascade(4*steps,self._phi,True)
		self.Cascade(steps,z0,False)
		self.Cascade(4*steps,self._phi,True)
		self.wavelet()
		self.Wavelet()
	def __str__(self) :
		s=''
		s+='_eta0 '+str(self._eta0)+'\n'
		s+='_steps '+str(self._steps)+'\n'
		s+='_l2 '+str(self._l2)+'\n'
		s+='_dl2 '+str(self._dl2)+'\n'
		s+='_L2 '+str(self._L2)+'\n'
		s+='_DL2 '+str(self._DL2)+'\n'
		return s
	def phi(self) :
		return self._phi
	def Phi(self) :
		return self._Phi
	def psi(self) :
		return self._psi
	def Psi(self) :
		return self._Psi
	def phi_(self) :
		return ltools.descriptionToPoly(self._phi)
	def Phi_(self) :
		return ltools.descriptionToPoly(self._Phi)
	def psi_(self) :
		return ltools.descriptionToPoly(self._psi)
	def Psi_(self) :
		return ltools.descriptionToPoly(self._Psi)
		
		