import PiecewisePoly as poly
import numpy
import scipy.linalg as la


def Scalar(f1,f2) :
	"L2 scalar product of two PiecewisePoly-nomials"
	return f1.mult(f2).integral()
def scalar(f1,f2,I) :
	"L2 scalar product of two PiecewisePoly-nomials"
	f1=f1.segment(I)
	f2=f2.segment(I)
	return f1.mult(f2).integral()
def Sqrt(x) :
	if( numpy.abs(x)<1.e-6 and x<0 ) :
		return numpy.sqrt(-x)
	if( x<0 ) :
		raise Exception("Sqrt: x="+str(x))
	return numpy.sqrt(x)
def Dist(f1,f2) :
	d=f1-f2
	return Sqrt(Scalar(d,d))
def dist(f1,f2,I) :
	d=f1-f2
	return Sqrt(scalar(d,d,I))
def L2Norm(f) :
	return Sqrt(Scalar(f,f))
def l2Norm(f,I) :
	return Sqrt(scalar(f,f,I))

def ScalarW12(f1,f2) :
	return f1.mult(f2).integral()+f1.D().mult(f2.D()).integral()
def scalarW12(f1,f2,I) :
	f1=f1.segment(I)
	f2=f2.segment(I)
	return f1.mult(f2).integral()+f1.D().mult(f2.D()).integral()
def DistW12(f1,f2,I) :
	d=f1-f2
	return Sqrt(ScalarW12(d,d))
def distW12(f1,f2,I) :
	d=f1-f2
	return Sqrt(scalarW12(d,d,I))
def W12Norm(f,I) :
	return Sqrt(ScalarW12(f,f,I))
def w12Norm(f,I) :
	return Sqrt(scalarW12(f,f,I))

	
def condN(A) :
	import scipy.linalg as la
	(lam,Q)=la.eig(A)
	return max(lam)/min(lam)
def maximalSingularValue(A) :
	import numpy as np
	import scipy.linalg as la
	(lam,Q)=la.eig(A.T*A)
	return np.sqrt(max(abs(lam)))
def spectralRadius(A) :
	import scipy.linalg as la
	(lam,Q)=la.eig(A)
	return max(abs(lam))
	
def GrammMatrix(f) :
	a=[]
	N=len(f)
	for x in f :
		aa=[]
		a.append(aa)
		for y in f :
			aa.append(Scalar(x,y))
	return numpy.matrix(a)
def GrammInv(f) :
	m=GrammMatrix(f)
	print 'GrammInv, condN='+str(condN(m))
	return m.I
def ScalarColumn(u,f) :
	a=[]
	for x in f :
		a.append([Scalar(u,x)])
	return numpy.matrix(a)
def Projection(u,f,grammInv) :
	b=ScalarColumn(u=u,f=f)
	c=grammInv*b
	proj=poly.LZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	return proj
def Projection2(u,f) :
	print 'calculating projection'
	gInv=GrammInv(f=f)
	b=ScalarColumn(u=u,f=f)
	c=gInv*b
	proj=poly.LZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)
def GrammSchmidt(f) :
	r=[]
	for k in range(0,len(f)) :
		proj=poly.LZero(f[0].support())
		for p in range(0,k) :
			c=Scalar(f[k],r[p])
			proj+=r[p]*c
		rr=f[k]-proj
		L2=L2Norm(rr)
		print 'GrammSchmidt, L2='+str(L2)
		r.append(rr/L2)
	return r
	


def grammSchmidt(f,I,kernel=None) :
	ff=[x.segment(I) for x in f]
	if( not kernel is None ) :
		kernel=kernel.segment(I)
	f=ff
	r=[]
	N=len(f)
	for k in range(0,N) :
		proj=poly.LZero(I)
		for p in range(0,k) :
			prd=f[k].mult(r[p])
			if( not kernel is None ) :
				prd=prd.mult(kernel)
			c=prd.integral()
			proj+=r[p]*c
		rr=f[k]-proj
		sq=rr.mult(rr)
		if( not kernel is None ) :
			sq=sq.mult(kernel)
		n=numpy.sqrt(sq.integral())
		n=1.0/n
		r.append(rr*n)
	return r
def grammSchmidt2(f,I,kernel=None) :
	ff=[x.segment(I) for x in f]
	if( not kernel is None ) :
		kernel=kernel.segment(I)
	f=ff
	r=[]
	L2=[]
	N=len(f)
	for k in range(0,N) :
		proj=poly.LZero(I)
		for p in range(0,k) :
			prd=f[k].mult(r[p])
			if( not kernel is None ) :
				prd=prd.mult(kernel)
			c=prd.integral()
			proj+=r[p]*c
		rr=f[k]-proj
		sq=rr.mult(rr)
		if( not kernel is None ) :
			sq=sq.mult(kernel)
		n=numpy.sqrt(sq.integral())
		nn=1.0/n
		r.append(rr*nn)
		L2.append(n)
	return (r,L2)
def grammMatrix(f,I) :
	N=len(f)
	a=[]
	for i in range(0,N) :
		print 'grammMatrix, (i,N)='+str((i,N))
		aa=[]
		a.append(aa)
		for j in range(0,N) :
			aa.append(scalar(f[i],f[j],I))
	return numpy.matrix(a)
def grammInv(f,I) :
	m=grammMatrix(f,I)
	print 'GrammInv, condN='+str(condN(m))
	return m.I
def scalarColumn(u,f,I) :
	a=[]
	print poly.deviceError_getMemoryInfo()
	for x in f :
		a.append([scalar(u,x,I)])
	return numpy.matrix(a)
def projection(u,f,grammInv,I) :
	print 'calculating projection'
	b=scalarColumn(u=u,f=f,I=I)
	c=grammInv*b
	proj=poly.LZero(I)
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	print 'projection done'
	return proj
def projection2(u,f,I) :
	print 'calculating projection'
	gInv=grammInv(f=f,I=I)
	b=scalarColumn(u=u,f=f,I=I)
	c=gInv*b
	proj=poly.LZero(I)
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)
def projection3(u,f,grammInv,I) :
	print 'calculating projection'
	b=scalarColumn(u=u,f=f,I=I)
	c=grammInv*b
	proj=poly.LZero(I)
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	cc=[c[i,0] for i in range(0,len(f))]
	print 'projection done'
	return (proj,cc)	

	
def grammMatrixW12(f,I) :
	a=[]
	N=len(f)
	k=0
	for x in f :
		print 'GrammMatrixW12, k='+str(k)+' of '+str(N)
		k+=1
		aa=[]
		a.append(aa)
		for y in f :
			aa.append(scalarW12(x,y,I))
	return numpy.matrix(a)
def grammInvW12(f,I) :
	m=grammMatrixW12(f,I)
	print 'GrammInvW12, condN='+str(condN(m))
	return m.I
def scalarColumnW12(u,f,I) :
	a=[]
	for x in f :
		a.append([scalarW12(u,x,I)])
	return numpy.matrix(a)
def projectionW12(u,f,grammInvW12,I) :
	print 'calculating projection w12'
	b=scalarColumnW12(u=u,f=f,I=I)
	c=grammInvW12*b
	proj=poly.LZero(u.support())
	for k in range(0,len(f)) :
		proj+=f[k]*c[k,0]
	print 'projection done'
	return proj
def grammSchmidtW12(f,I) :
	print 'GrammSchmidtW12'
	ff=[x.segment(I) for x in f]
	f=ff
	r=[]
	for k in range(0,len(f)) :
		proj=poly.LZero(f[0].support())
		for p in range(0,k) :
			c=scalarW12(f[k],r[p],I)
			proj+=r[p]*c
		rr=f[k]-proj
		W12=w12Norm(rr,I)
		print 'GrammSchmidt, W12='+str(W12)
		r.append(rr*(1.0/W12))
	print 'GrammSchmidtW12 done'
	return r
	
	
def polyToDescription(p) :
	d=p.description()
	return ( [d['d'],d['a'],d['b'],d['degree']], d['coeffs'], d['scales'] )
def dictToDescription(d) :
	return ( [d['d'],d['a'],d['b'],d['degree']], d['coeffs'], d['scales'] )
def descriptionToPoly(x) :
	return poly.LPoly(x[0],x[1],x[2])
	
def crudify_(phi,cr) :
	for i in range(0,cr) :
		phi=phi.Cr()
	return phi
def CheckScale(phi,h,cr=0) :
	phi=descriptionToPoly(phi)
	phi=crudify_(phi,cr)
	return CheckScale_(phi,h,cr)
def CheckScale_(phi,h,cr=0) :
	p=poly.LZero(phi.support())
	for k in h :
		p+=phi.Op(1,k)*h[k]
	d=Dist(phi,p)
	d1=Dist(phi.D(),p.D())
	print 'scale difference L2='+str(d)
	print 'scale difference DL2='+str(d1)
	print 'phi.len()='+str(phi.len())
	return (d,d1,phi.len())
	