execfile('common.py')

import gram
import ots.io as io

name='mrFinal.py'

version="B"
execfile('version_'+version+'.py')
basisFile='dirichlet3a_step'+str(steps)+'_d45_crN'+str(crN)+'_maxD'+str(maxD)+'_result'
gramFile='gram_version_'+version
gridFile='grid_version_'+version
mrGramFile='mrGram_version_'+version

class MeanRevertingEquation(object) :
	def __init__(self,x0,t,T,mu,meanRev,a,sigma,precision) :
		self.x0=x0
		self.t=t
		self.T=T
		self.mu=mu
		self.meanRev=meanRev
		self.a=a
		self.sigma=sigma
		self.precision=precision
		self.minimalNumberOfK=16
		self.minimalD=-7
	def calcInterval(self) :
		import scipy.stats
		import numpy
		N01Inv=scipy.stats.norm(0,1).ppf
		xsi0=abs(N01Inv(self.precision/2))
		x0=self.x0		
		mu=self.mu
		meanRev=self.meanRev
		a=self.a
		sigma=self.sigma
		dt=self.T-self.t
		muTilde=mu-0.5*sigma*sigma
		y1=a+muTilde/meanRev
		y2=numpy.log(x0)-a-muTilde/meanRev
		alpha1=y1+y2
		alpha2=y1+numpy.exp(-meanRev*dt)*y2
		beta=sigma*numpy.sqrt((1-numpy.exp(-2*meanRev*dt))/2/meanRev)
		Ym=min(alpha1,alpha2)-beta*xsi0
		Yp=max(alpha1,alpha2)+beta*xsi0
		A0=numpy.exp(Ym)
		B0=numpy.exp(Yp)
		d0=self.minimalD
		while(True) :
			kA=int(numpy.floor(poly.pow2_(d0)*A0))
			kB=int(numpy.ceil(poly.pow2_(d0)*B0))
			kk=kB-kA+1
			if( kk>=self.minimalNumberOfK ) :
				break
			d0+=1
		self.int_d0=d0
		self.int_kA=kA
		self.int_kB=kB
	def oneTimeStepEOnly(self,tn,tnp1,Kn,gram,mrGram) :
		import numpy as np
		import scipy.linalg as la
		dt=tnp1-tn
		sigma=self.sigma
		P=_m(mrGram.makeP(Kn))
		R=_m(mrGram.makeR(Kn))
		Q=_m(mrGram.makeQ(Kn))
		G=_m(gm.gram(Kn))
		GI=G.I
		B0=(mu+meanRev*meanLevel)*P-meanRev*R-0.5*sigma*sigma*Q
		U=-GI*B0.T
		(H,D)=makeH2(dt=dt,U=U)
		I=unit(H.shape[0])
		E=(I-H)*D
		for p in range(0,11) :
			H=H*H
			E=(I+H)*E
		return (E,GI,H)
	def expTimeStep(self,tn,tnp1,Kn,Knp1,gram,mrGram,cnp1,dd) :
		dt=tnp1-tn
		steps=pow(2,dd)
		ddt=dt*1.0/steps
		(E,GI,H)=self.oneTimeStepEOnly(tn=tnp1-ddt,tnp1=tnp1,Kn=Kn,gram=gram,mrGram=mrGram)
		for p in range(0,dd) :
			E=E*E
		Gnp1=_m(gram.gram2(Kn,Knp1))
		b=GI*Gnp1*_m(cnp1).T
		c=E*b
		return (c,E)
	def timeStep(self,tn,tnp1,Knp1,gram,mrGram,cnp1,dd,precision) :
		(c,_)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=Knp1,Knp1=Knp1,gram=gram,mrGram=mrGram,cnp1=cnp1,dd=dd)
		ids=gram.map.keys()
		X0=cnp1
		X1=[x[0,0] for x in c]
		grid0=Knp1
		N=len(grid0)
		ii=range(0,N)
		Q=[abs(X1[i]/X0[i]) for i in ii]
		Q2=[(Q[i],grid0[i]) for i in ii]
		Q2.sort(reverse=True)
		kappa=[-4,-3,-2,-1,0,1,2,3,4]
		#new grid candidates
		toAdd=[]
		for (q,(d,k)) in Q2 :
			if( q<1 ) :
				break
			toAdd+=[(d,k+i) for i in kappa if( (d,k+i) in ids )]
		#increasing scale
		(_,(dMax,kMax))=Q2[0]
		gridA=[(dMax,kMax)]
		if( type(dMax)!=type('') ) :
			gridB=[id for id in ids if id[0]==dMax+1]
			g=gram.gram2(gridA,gridB)[0]
			gg=[(abs(g[i]),gridB[i]) for i in range(0,len(g)) if g[i]!=0]
			gg.sort(reverse=True)
			for i in range(0,min(len(kappa),len(gg))) :
				toAdd.append(gg[i][1])
		#making new grid
		grid1=uniqueUnion(grid0,toAdd)
		print "new grid len="+str(len(grid1))
		#making next time step
		(c1,E1)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=grid1,Knp1=grid0,gram=gram,mrGram=mrGram,cnp1=cnp1,dd=dd)
		#excluding components with low contribution
		L=[(abs(c1[i,0]),grid1[i]) for i in range(0,len(grid1))]
		L.sort()
		i=0
		sum=0
		while(sum<precision) :
			sum+=L[i][0]
			i+=1
		N=i-1
		grid2=[L[i][1] for i in range(N,len(L))]
		print "corrected grid len="+str(len(grid2))
		#making time step with corrected grid
		(c2,E2)=self.expTimeStep(tn=tn,tnp1=tnp1,Kn=grid2,Knp1=grid0,gram=gram,mrGram=mrGram,cnp1=cnp1,dd=dd)
		return (c2,grid2)
	def evolve(self,t,T,grid,gram,mrGram,c,dd,steps,precision) :
		dt=(T-t)*1.0/steps
		for s in range(0,steps) :
			print "start step "+str(s)
			(c,grid)=self.timeStep(tn=T-(s+1)*dt,tnp1=T-s*dt,Knp1=grid,gram=gram,mrGram=mrGram,cnp1=c,dd=dd,precision=precision)
			c=[c[i,0] for i in range(0,len(c))]
			print "end step "+str(s)
			print "grid length " +str(len(grid))
		return (c,grid)
	
def makeCall(inteval,strike) :
	strikeD=poly.pow2_(I.d())*strike
	_check(strikeD-int(strikeD)==0,"strikeD is not int")
	strikeD=int(strikeD)
	return poly.extend(poly.Piece(I.d(),strikeD,I.b()),[-strike,1])
def makePut(inteval,strike) :
	strikeD=poly.pow2_(I.d())*strike
	_check(strikeD-int(strikeD)==0,"strikeD is not int")
	strikeD=int(strikeD)
	return poly.extend(poly.Piece(I.d(),I.a(),strikeD),[strike,-1])
def makeH2(dt,U) :
	import numpy as np
	U=dt*U
	N=U.shape[0]
	I=unit(N)
	sumUk=[]
	for k in range(0,N) :
		sum=0
		for s in range(0,N) :
			sum+=pow(U[k,s],2)
		sumUk.append(sum)
	Dk=[]
	for k in range(0,N) :
		x=(1+U[k,k])/(1+2*U[k,k]+sumUk)
		Dk.append(x)
	D=diag(Dk)
	H=D-I+D*U
	return (H,D)
class MrSolution(object) :
	def __init__(self,payoff,mu,meanRev,meanLevel,sigma,T) :
		import numpy as np
		self.g=payoff
		self.mu=mu
		self.meanRev=meanRev
		self.meanLevel=meanLevel
		self.sigma=sigma
		self.T=T
	def func(self,z) :
		import numpy as np
		g=self.g
		mu=self.mu
		meanRev=self.meanRev
		meanLevel=self.meanLevel
		sigma=self.sigma
		t=self.t
		T=self.T
		x0=self.x0
		z0=-meanRev*(T-t)
		z1=meanLevel+(mu-0.5*sigma*sigma)/meanRev
		z2=(1-np.exp(2*z0))/2/meanRev
		y1=g(np.exp(z1+np.exp(z0)*(np.log(x0)-z1)+sigma*np.sqrt(z2)*z))
		y2=np.exp(-0.5*z*z)
		return y1*y2
	def value(self,t,x) :
		import scipy as sp
		import numpy as np
		self.x0=x
		self.t=t
		spt=self.g.support()
		xA=spt.xA()
		xB=spt.xB()
		return sp.integrate.quad(self.func,-100,100)/np.sqrt(2*np.pi)

try :
	gm
except :
	gm=load(gramFile)	
	gm.afterPickle()

		
mr=MeanRevertingEquation(x0=x0,t=t,T=T,mu=mu,meanRev=meanRev,a=meanLevel,sigma=sigma,precision=precision)
mr.calcInterval()

try :
	multiBasis
except :
	multiBasis=load(basisFile)

xsi=multiBasis['xsi']
chi=multiBasis[xsi.d]
msb=gram.MultiScaleBasis(xsi,chi,mr.int_d0,mr.int_kA,mr.int_kB)

I=poly.Piece(msb.d,msb.kA,msb.kB)

payoff0=makePut(I,strike)-makePut(I,strike-delta)+makeCall(I,strike)-makeCall(I,strike+delta)
shift=poly.extend(I,[-delta])
payoff=poly.LPoly(payoff0+shift)

#payoff=poly.LPoly(poly.extend(I,[1]))

try :
	grid
except :
	grid=load(gridFile)
	
try :
	mrGram
except :
	mrGram=load(mrGramFile)
	mrGram.afterPickle(gm)
	
sol=MrSolution(payoff=payoff,mu=mu,meanRev=meanRev,meanLevel=meanLevel,sigma=sigma,T=T)
print sol.value(t,x0)

(proj,cnp1)=gm.projection2(payoff,grid)
t=1
(c,grid0)=mr.evolve(t=t,T=T,grid=grid,gram=gm,mrGram=mrGram,c=cnp1,dd=10,steps=5,precision=0.001)

r=makeFunc(grid=grid0,gram=gm,c=c)
print sol.value(t,x0)
print r(x0)
pp(r)

mrGram.beforePickle()
save(mrGram,mrGramFile)
