
class _Omega(object) :
	def __init__(self) :
		self._pow=1.1
	def value(self,x) :
		return pow(max(x,0),self._pow)
	def deriv(self,x) :
		return self._pow*pow(max(x,0),self._pow-1)
	def onVector(self,c) :
		import numpy
		r=[]
		for i in range(0,len(c)) :
			r.append(self.value(c[i,0]))
		return numpy.matrix(r).T
	def onVectorDeriv(self,c) :
		import numpy
		r=[]
		for i in range(0,len(c)) :
			r.append(self.deriv(c[i,0]))
		return numpy.matrix(numpy.diag(r))

class PenaltyGram(object) :
	def __init__(self,minD,wGram) :
		import PiecewisePoly as poly
		self._gram={}
		self._minD=minD
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		self._hut=hut.upscale(-min(0,minD))
		self._speak=True
		self._wGram=wGram
	def beforePickle(self) :
		self._hut=None
		self._wGram=None
	def afterPickle(self,wGram) :
		import PiecewisePoly as poly
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		self._hut=hut.upscale(-min(0,self._minD))
		self._speak=False
		self._wGram=wGram
	def gram(self,hGrid,wGrid) :
		import numpy
		print "calculating Penalty.gram"
		i=0
		a=[]
		for hId in hGrid :
			if( self._speak ) :
				print "i="+str(i)+" of "+str(len(hGrid))
			i+=1
			aa=[]
			a.append(aa)
			for wId in wGrid :
				aa.append(self.scalar(hId,wId))
		print "end of calculating Penalty.gram"
		return numpy.matrix(a)
	def scalar(self,hId,wId) :
		id=(hId,wId)
		try :
			return self._gram[id]
		except KeyError:
			import ltools
			hFunc=self.getHFunc(hId)
			wFunc=self.getWFunc(wId)
			s=ltools.Scalar(hFunc,wFunc)
			self._gram[id]=s
			return s
	def getHFunc(self,id) :
		(p,k)=id
		return self._hut.T(k).S(p)*pow(2.0,-0.5*p)
		#the power included to compensate for density of placement.
	def getWFunc(self,id) :
		return self._wGram.func(id)
	
class Penalty(object) :
	def __init__(self,payoff,gram,penGram) :
		self._payoff=payoff
		self._gram=penGram
		self._lastA=[]
		self._lastWGrid=[]
		self._lastHGrid=[]
		self._wGram=gram
	def Omega(self,c,wGrid,hGrid) :
		import ltools
		import numpy 
		if( self._lastWGrid==wGrid ) :
			a=self._lastA
		else :
			self._lastWGrid=wGrid
			funcs=[self._wGram.func(id) for id in wGrid]
			(_,a)=ltools.Projection2(self._payoff,funcs)
			a=numpy.matrix(a).T
			self._lastA=a
		om=_Omega()
		G=self._gram.gram(hGrid=hGrid,wGrid=wGrid)
		x1=G*(a-c)
		x2=om.onVector(x1)
		x3=G.T*x2
		x4=om.onVectorDeriv(x1)
		x5=-G.T*x4*G
		return (x3,x5)

class L1PenaltyGram(object) :
	def __init__(self,minD,wGram) :
		import PiecewisePoly as poly
		self._gram={}
		self._minD=minD
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		n=hut.integral()
		hut=hut*(1.0/n)
		self._hut=hut.upscale(-min(0,minD))
		self._speak=True
		self._wGram=wGram
	def beforePickle(self) :
		self._hut=None
		self._wGram=None
	def afterPickle(self,wGram) :
		import PiecewisePoly as poly
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		n=hut.integral()
		hut=hut*(1.0/n)
		self._hut=hut.upscale(-min(0,self._minD))
		self._speak=False
		self._wGram=wGram
	def gram(self,hGrid,wGrid) :
		import numpy
		print "calculating L1Penalty.gram"
		i=0
		a=[]
		for hId in hGrid :
			if( self._speak ) :
				print "i="+str(i)+" of "+str(len(hGrid))
			i+=1
			aa=[]
			a.append(aa)
			for wId in wGrid :
				aa.append(self.scalar(hId,wId))
		print "end of calculating L1Penalty.gram"
		return numpy.matrix(a)
	def scalar(self,hId,wId) :
		id=(hId,wId)
		try :
			return self._gram[id]
		except KeyError:
			import ltools
			hFunc=self.getHFunc(hId)
			wFunc=self.getWFunc(wId)
			s=ltools.Scalar(hFunc,wFunc)
			self._gram[id]=s
			return s
	def getHFunc(self,id) :
		(p,k)=id
		f=self._hut.T(k).S(p)
		f=f*(1.0/f.integral())
		return f
	def getWFunc(self,id) :
		return self._wGram.func(id)
		
		
class L2PenaltyGram(object) :
	def __init__(self,minD,wGram) :
		import PiecewisePoly as poly
		self._gram={}
		self._gramTilde={}
		self._minD=minD
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		import ltools
		import numpy as np
		n=ltools.Scalar(hut,hut)
		nn=1.0/np.sqrt(n)
		hut=hut*nn
		self._hut=hut.upscale(-min(0,minD))
		self._speak=True
		self._wGram=wGram
	def beforePickle(self) :
		self._hut=None
		self._wGram=None
	def afterPickle(self,wGram) :
		import PiecewisePoly as poly
		hut=poly.LPoly(poly.Poly([0,-1,1,1],[[1,1],[1,-1]]))
		import ltools
		import numpy as np
		n=ltools.Scalar(hut,hut)
		nn=1.0/np.sqrt(n)
		hut=hut*nn
		self._hut=hut.upscale(-min(0,self._minD))
		self._speak=False
		self._wGram=wGram
	def gram(self,hGrid,wGrid) :
		import numpy
		print "calculating L2Penalty.gram"
		i=0
		a=[]
		for hId in hGrid :
			if( self._speak ) :
				print "i="+str(i)+" of "+str(len(hGrid))
			i+=1
			aa=[]
			a.append(aa)
			for wId in wGrid :
				aa.append(self.scalar(hId,wId))
		print "end of calculating L2Penalty.gram"
		return numpy.matrix(a)
	def _makeGrid2(self,hGrid) :
		(d,k)=hGrid[0]
		minK=k
		maxK=k
		for (dd,kk) in hGrid :
			assert(dd==d)
			minK=min(minK,kk)
			maxK=max(maxK,kk)
		return [(d+1,k) for k in range(2*minK-2,2*maxK+3)]
	def gramTilde(self,hGrid) :
		import numpy
		print "calculating L2Penalty.gramTilde"
		hGrid1=hGrid
		hGrid2=self._makeGrid2(hGrid)
		i=0
		a=[]
		for hId1 in hGrid1 :
			if( self._speak ) :
				print "i="+str(i)+" of "+str(len(hGrid1))
			i+=1
			aa=[]
			a.append(aa)
			for hId2 in hGrid2 :
				aa.append(self.scalarTilde(hId1,hId2))
		print "end of calculating L2Penalty.gram"
		return numpy.matrix(a)		
	def scalarTilde(self,hId1,hId2) :
		id=(hId1,hId2)
		try :
			return self._gramTilde[id]
		except KeyError:
			import ltools
			hFunc1=self.getHFunc(hId1)
			hFunc2=self.getHFunc(hId2)
			s=ltools.Scalar(hFunc1,hFunc2)
			self._gramTilde[id]=s
			return s
	def scalar(self,hId,wId) :
		id=(hId,wId)
		try :
			return self._gram[id]
		except KeyError:
			import ltools
			hFunc=self.getHFunc(hId)
			wFunc=self.getWFunc(wId)
			s=ltools.Scalar(hFunc,wFunc)
			self._gram[id]=s
			return s
	def getHFunc(self,id) :
		(p,k)=id
		return self._hut.T(k).S(p)
	def getWFunc(self,id) :
		return self._wGram.func(id)
